"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.indexerPart = void 0;
const expressionPart_1 = require("./expressionPart");
const Utility_1 = require("typescript-dotnet-commonjs/System/Text/Utility");
const List_1 = require("typescript-dotnet-commonjs/System/Collections/List");
const bindingContext_1 = require("./../../bindingContext");
const reflectionExtender_1 = require("../../utils/reflectionExtender");
const typeMapper_1 = require("../../types/typeMapper");
const utils_1 = require("../utils");
const outputVariables_1 = require("../../utils/outputVariables");
const dictionaryObject_1 = require("../../utils/dictionaryObject");
const entityObject_1 = require("../../utils/entityObject");
const general_1 = require("../../utils/general");
class indexerPart extends expressionPart_1.expressionPart {
    get SubParts() {
        let lst = [];
        lst.push(this.indexerExpression);
        return lst;
    }
    /* override */ get simpleName() {
        return this.indexerExpression.simpleName;
    }
    resolve(expressionData, /* out */ resolved, running, evaluateAllContexts = true, setValue = undefined) {
        if (expressionData != null && this.indexerExpression != null) {
            let indexResolved = new outputVariables_1.outputVariables();
            indexResolved.resolved = false;
            let index = this.indexerExpression.resolve(expressionData, /* out */ indexResolved, /* running:*/ running);
            if (indexResolved.resolved) {
                let tempContext = expressionData.contexts.clone();
                while (!tempContext.isEmpty()) {
                    let contextObject = tempContext.pop();
                    let result = this.callIndexer(contextObject, index, /* out */ resolved);
                    if (resolved.resolved || !evaluateAllContexts) {
                        return result;
                    }
                }
            }
        }
        resolved.resolved = false;
        return undefined;
    }
    callIndexer(context, index, /* out */ resolved, setValue = undefined) {
        let result = undefined;
        resolved.resolved = false;
        // if (typeof context === 'string' || typeof context === 'number' || typeof context === 'boolean' || typeof context === 'undefined' || context instanceof Date) {
        //     result = context;
        //     resolved.resolved = (result != null);
        //     return result as Object;
        // }
        if ((context instanceof dictionaryObject_1.DictionaryObject)) {
            let castedSubject = (context);
            const re = castedSubject[index];
            result = re === undefined ? null : re;
            resolved.resolved = (result !== undefined);
        }
        else if ((context instanceof entityObject_1.EntityObject)) {
            let castedSubject = (context);
            let castedIndex = index; //typeMapper.toType1<string>(index, getType(Object));
            if ((setValue != null)) {
                castedSubject.setItem(setValue, castedIndex);
                result = setValue;
            }
            result = castedSubject.getItem(castedIndex);
            resolved.resolved = (result !== undefined);
        }
        else if (context && context["data"] && context["data"]["customKeyValuePair"]) {
            const array = context["data"]["customKeyValuePair"];
            if (Array.isArray(array)) {
                for (const iterator of array) {
                    if (iterator.key === index) {
                        result = iterator.value;
                        resolved.resolved = (result != undefined);
                    }
                }
            }
        }
        else if (context instanceof bindingContext_1.makeenBindingContext) {
            let castedSubject = (context);
            let castedIndex = typeMapper_1.typeMapper.toType1(index);
            if ((setValue != null)) {
                castedSubject.Item(castedIndex, setValue);
            }
            else {
                result = castedSubject.Item(castedIndex);
            }
            result = setValue;
            resolved.resolved = (result != null);
        }
        else if ((context instanceof List_1.default) && context.count > 0 && context.get(0) instanceof bindingContext_1.makeenBindingContext) {
            let results = new List_1.default();
            for (let subContext of ((context).toArray())) {
                let castedSubject = (subContext);
                let castedIndex = typeMapper_1.typeMapper.toType1(index);
                if ((setValue != null)) {
                    castedSubject.Item(castedIndex, setValue);
                }
                else {
                    // result = castedSubject.Items?.getItem(castedIndex);
                }
                result = setValue;
                if ((result != null)) {
                    results.add(result);
                }
            }
            result = results.count > 0 ? results : undefined;
            resolved.resolved = (results.count > 0);
        }
        else {
            let intIndex = (typeof index == "number") ? index : -1;
            let isIntIndex = (typeof index == "number") ? true : (typeof Number.parseInt(index.toString()) == "number" ? true : false);
            if (this.isArrayofType(context, "Object")) // (typeof context == Object[]))
             {
                if (isIntIndex) {
                    let arr = (context);
                    if ((setValue != null)) {
                        arr[intIndex] = setValue;
                    }
                    result = setValue;
                    if ((arr.length > intIndex)) {
                        result = arr[intIndex];
                        resolved.resolved = true;
                    }
                    else {
                        result = undefined;
                        resolved.resolved = false;
                    }
                }
            }
            else if (this.isArrayofType(context, "Array")) //((context instanceof Array[]))
             {
                if (isIntIndex) {
                    let arr = (context);
                    if ((setValue != null)) {
                        arr[intIndex] = typeMapper_1.typeMapper.toType(setValue);
                    }
                    result = typeMapper_1.typeMapper.toType(setValue);
                    if ((arr.length > intIndex)) {
                        result = arr[intIndex];
                        resolved.resolved = true;
                    }
                    else {
                        result = undefined;
                        resolved.resolved = false;
                    }
                }
            }
            else if (this.isArrayofType(context, "number")) //((context instanceof number[])) 
             {
                if (isIntIndex) {
                    let arr = (context);
                    if ((setValue != null)) {
                        arr[intIndex] = typeMapper_1.typeMapper.toType(setValue);
                    }
                    result = typeMapper_1.typeMapper.toType(setValue);
                    if ((arr.length > intIndex)) {
                        result = arr[intIndex];
                        resolved.resolved = true;
                    }
                    else {
                        result = undefined;
                        resolved.resolved = false;
                    }
                }
            }
            else if (this.isArrayofType(context, "double")) //((context instanceof double[])) 
             {
                if (isIntIndex) {
                    let arr = (context);
                    if ((setValue != null)) {
                        arr[intIndex] = typeMapper_1.typeMapper.toType(setValue);
                    }
                    result = typeMapper_1.typeMapper.toType(setValue);
                    if ((arr.length > intIndex)) {
                        result = arr[intIndex];
                        resolved.resolved = true;
                    }
                    else {
                        result = undefined;
                        resolved.resolved = false;
                    }
                }
            }
            else if (context instanceof List_1.default && isIntIndex) {
                let arr = (context);
                if ((setValue != null)) {
                    arr.set(intIndex, setValue);
                }
                result = setValue;
                if ((arr.count > intIndex)) {
                    result = arr.get(intIndex);
                    resolved.resolved = true;
                }
                else {
                    result = undefined;
                    resolved.resolved = false;
                }
            }
            else {
                if (this.indexerProperty == null || this.contextType != reflectionExtender_1.getType(context)) {
                    if (typeof context === 'object') {
                        const contextResult = general_1.getDirectProperty(context, index);
                        if (contextResult !== undefined) {
                            result = contextResult;
                            resolved.resolved = true;
                            return result;
                        }
                    }
                    this.indexerProperty = utils_1.utils.getIndexer(reflectionExtender_1.getType(context));
                    this.contextType = reflectionExtender_1.getType(context);
                }
                if ((this.indexerProperty != null)) {
                    try {
                        if ((setValue != null)) {
                            this.indexerProperty.setValue(context, [
                                typeMapper_1.typeMapper.toType(setValue, this.indexerProperty.propertyType),
                                typeMapper_1.typeMapper.toType(index, this.indexerProperty.getIndexParameters().parameterType)
                            ]);
                        }
                        let success = new outputVariables_1.outputVariables();
                        let typedIndex = typeMapper_1.typeMapper.toType(index, this.indexerProperty.getIndexParameters().parameterType, /* out */ success);
                        if (success) {
                            let arr = [];
                            arr.push(typedIndex);
                            result = this.indexerProperty.getValue(context, arr);
                            resolved.resolved = true;
                        }
                        else {
                            resolved.resolved = false;
                            result = undefined;
                        }
                    }
                    catch (e) {
                        resolved.resolved = false;
                        result = undefined;
                    }
                }
            }
        }
        return result;
    }
    toString() {
        return Utility_1.format("Item[{0}]", this.indexerExpression);
    }
    isArrayofType(test, type) {
        return Array.isArray(test) && test.every((value) => typeof value === type);
    }
}
exports.indexerPart = indexerPart;
//# sourceMappingURL=indexerPart.js.map