"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.partList = void 0;
const compoundPart_1 = require("./compoundPart");
const stack_1 = require("../../utils/stack");
const outputVariables_1 = require("../../utils/outputVariables");
class partList extends compoundPart_1.compoundPart {
    constructor() {
        super(...arguments);
        this.lastResolvedLevel = 0;
        this.parts = [];
    }
    get Parts() {
        return this.parts;
    }
    get simpleName() {
        return this.parts != null && this.parts.length > 0 ? this.parts[this.parts.length - 1].simpleName : "Item";
    }
    addSubPart(part) {
        this.parts.push(part);
        return true;
    }
    resolve(expressionData, resolved, running, evaluateAllContexts = true, setValue = undefined) {
        let contexts;
        if (evaluateAllContexts) {
            contexts = new stack_1.Stack();
            let temp = expressionData.contexts.clone();
            while (!temp.isEmpty()) {
                contexts.push(temp.pop());
            }
        }
        else {
            contexts = new stack_1.Stack();
            contexts.push(expressionData.contexts.pop());
        }
        let tempData = expressionData.toSubEx();
        if ((evaluateAllContexts
            && ((this.lastResolvedLevel > 0)
                && (this.lastResolvedLevel < contexts.count())))) {
        }
        while ((contexts.count() > 0)) {
            let currentContext = contexts.pop();
            let subResolved = new outputVariables_1.outputVariables();
            let i = 0;
            for (; (i < this.parts.length); i++) {
                let item = this.parts[i];
                let popAtEnd = false;
                if (currentContext === null) {
                    resolved.resolved = true;
                    return currentContext;
                }
                if (tempData.contexts.peek() != currentContext) {
                    tempData.contexts.push(currentContext);
                    popAtEnd = true;
                }
                // parentData.monitorIfRequired(item, currentContext);
                if (i == (this.parts.length - 1)) {
                    currentContext = item.resolve(tempData, subResolved, running, false, setValue);
                    //  For observing observable collections
                    // parentData.monitorIfRequired(undefined, currentContext);
                }
                else {
                    currentContext = item.resolve(tempData, subResolved, running, false, undefined);
                    if ((currentContext === undefined)) {
                        subResolved.resolved = false;
                    }
                }
                if (popAtEnd && !tempData.contexts.isEmpty()) {
                    tempData.contexts.pop();
                }
                if (!subResolved.resolved) {
                    break;
                }
            }
            if (!subResolved.resolved && i > 0) {
                resolved.resolved = false;
                return undefined;
            }
            if (subResolved.resolved) {
                resolved.resolved = true;
                this.lastResolvedLevel = (contexts.count() + 1);
                // parentData.resultCache.setValue(cacheKey, currentContext);
                return currentContext;
            }
        }
        resolved.resolved = false;
        return undefined;
    }
    toString() {
        return this.parts.join(".");
    }
}
exports.partList = partList;
//# sourceMappingURL=partList.js.map