"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.utils = void 0;
const TimeSpan_1 = require("typescript-dotnet-commonjs/System/Time/TimeSpan");
const DateTime_1 = require("typescript-dotnet-commonjs/System/Time/DateTime");
const List_1 = require("typescript-dotnet-commonjs/System/Collections/List");
const StringBuilder_1 = require("typescript-dotnet-commonjs/System/Text/StringBuilder");
const switchPartData_1 = require("./parts/switchPartData");
const expressionCompiler_1 = require("./expressionCompiler");
const operationPart_1 = require("./parts/operationPart");
const indicatorPart_1 = require("./parts/indicatorPart");
const nullPart_1 = require("./parts/nullPart");
const badExpressionException_1 = require("../error/badExpressionException");
const unterminatedEnclosureException_1 = require("../error/unterminatedEnclosureException");
const unterminatedStringLiteralException_1 = require("../error/unterminatedStringLiteralException");
const Types_1 = require("typescript-dotnet-commonjs/System/Types");
const reflectionExtender_1 = require("../utils/reflectionExtender");
const typeMapper_1 = require("../types/typeMapper");
const legacyBuiltInProperty_1 = require("../utils/legacyBuiltInProperty");
const dictionaryObject_1 = require("../utils/dictionaryObject");
const partList_1 = require("./parts/partList");
const propertyPart_1 = require("./parts/propertyPart");
const indexerPart_1 = require("./parts/indexerPart");
const general_1 = require("../utils/general");
class utils {
    static IsNumericType(v) {
        return Types_1.default.isNumber(v);
    }
    static isDecimalValue(val) {
        if (val % 1 != 0) {
            return true;
        }
        return false;
    }
    static supportsOperators(v) {
        return (utils.IsNumericType(v) || v instanceof DateTime_1.DateTime || v instanceof TimeSpan_1.TimeSpan || v === undefined || v === null);
    }
    static toBiggerType(v1, v2, target) {
        if ((v1 instanceof Number || v2 instanceof Number) && !this.isDecimalValue(target)) {
            return typeMapper_1.typeMapper.toType(target);
        }
        else if ((v1 instanceof Number || v2 instanceof Number) && this.isDecimalValue(target)) {
            return typeMapper_1.typeMapper.toType(target);
        }
        // else if ((v1 instanceof ((long || v2) instanceof long))) {
        //     return typeMapper.toType<number>(target);
        // }
        // else if ((v1 instanceof ((int || v2) instanceof int))) {
        //     return typeMapper.toType<number>(target);
        // }
        // else if ((v1 instanceof ((short || v2) instanceof short))) {
        //     return typeMapper.toType<number>(target);
        // }
        // else if ((v1 instanceof ((sbyte || v2) instanceof sbyte))) {
        //     return typeMapper.toType<System.SByte>(target);
        // }
        // let ||: number;
        // let Unknown: number;
        // return typeMapper.toType<number>(target);
        // if ((v1 instanceof ((uint || v2) instanceof uint))) {
        //     return typeMapper.toType<number>(target);
        // }
        // else if ((v1 instanceof ((ushort || v2) instanceof ushort))) {
        //     return typeMapper.toType<number>(target);
        // }
        // else if ((v1 instanceof ((byte || v2) instanceof byte))) {
        //     return typeMapper.toType<number>(target);
        // }
        return target;
    }
    static CheckUndefined(v1, v2) {
        if (v2 != undefined)
            return v2;
        if (v1 != undefined)
            return v1;
        return null;
    }
    static toBiggerTypeOperation(v1, v2, op) {
        if ((typeof v1 == 'number' || typeof v2 == 'number')) {
            switch (op) {
                case '+':
                    if (v1 == undefined || v2 == undefined) {
                        return utils.CheckUndefined(v1, v2);
                    }
                    return ((v1) + (v2));
                case '-':
                    if (v1 == undefined || v2 == undefined) {
                        return utils.CheckUndefined(v1, v2);
                    }
                    return ((v1) - (v2));
                case '*':
                    return ((v1) * (v2));
                case '/':
                    /// Please feel free to remove the isFloat type check 
                    /// whenever V2 complete rollout and V1 would discard
                    if (!general_1.isFloatType(v1) && !general_1.isFloatType(v2))
                        return Math.floor(((v1) / (v2)));
                    return ((v1) / (v2));
                    break;
                case '%':
                    return ((v1) % (v2));
                    break;
                case '^':
                    return ((v1) ^ (v2));
                    break;
                case '=':
                    return ((v1) == (v2));
                    break;
                case '!':
                    return ((v1) != (v2));
                    break;
                case '>':
                    let result = ((v1) > (v2));
                    return result;
                    break;
                case '<':
                    return ((v1) < (v2));
                    break;
            }
        }
        // else if ((v1 instanceof ((short || v2) instanceof short))) {
        //     switch (op) {
        //         case '+':
        //             return (typeMapper.toType<number>(v1) + typeMapper.toType<number>(v2));
        //             break;
        //         case '-':
        //             return (typeMapper.toType<number>(v1) - typeMapper.toType<number>(v2));
        //             break;
        //         case '*':
        //             return (typeMapper.toType<number>(v1) * typeMapper.toType<number>(v2));
        //             break;
        //         case '/':
        //             return (typeMapper.toType<number>(v1) / typeMapper.toType<number>(v2));
        //             break;
        //         case '%':
        //             return (typeMapper.toType<number>(v1) % typeMapper.toType<number>(v2));
        //             break;
        //         case '^':
        //             return (typeMapper.toType<number>(v1) | typeMapper.toType<number>(v2));
        //             break;
        //         case '=':
        //             return (typeMapper.toType<number>(v1) == typeMapper.toType<number>(v2));
        //             break;
        //         case '!':
        //             return (typeMapper.toType<number>(v1) != typeMapper.toType<number>(v2));
        //             break;
        //         case '>':
        //             return (typeMapper.toType<number>(v1) > typeMapper.toType<number>(v2));
        //             break;
        //         case '<':
        //             return (typeMapper.toType<number>(v1) < typeMapper.toType<number>(v2));
        //             break;
        //     }
        // }
        // else if ((v1 instanceof ((sbyte || v2) instanceof sbyte))) {
        //     switch (op) {
        //         case '+':
        //             return (typeMapper.toType<System.SByte>(v1) + typeMapper.toType<System.SByte>(v2));
        //             break;
        //         case '-':
        //             return (typeMapper.toType<System.SByte>(v1) - typeMapper.toType<System.SByte>(v2));
        //             break;
        //         case '*':
        //             return (typeMapper.toType<System.SByte>(v1) * typeMapper.toType<System.SByte>(v2));
        //             break;
        //         case '/':
        //             return (typeMapper.toType<System.SByte>(v1) / typeMapper.toType<System.SByte>(v2));
        //             break;
        //         case '%':
        //             return (typeMapper.toType<System.SByte>(v1) % typeMapper.toType<System.SByte>(v2));
        //             break;
        //         case '^':
        //             return (typeMapper.toType<System.SByte>(v1) | typeMapper.toType<System.SByte>(v2));
        //             break;
        //         case '=':
        //             return (typeMapper.toType<System.SByte>(v1) == typeMapper.toType<System.SByte>(v2));
        //             break;
        //         case '!':
        //             return (typeMapper.toType<System.SByte>(v1) != typeMapper.toType<System.SByte>(v2));
        //             break;
        //         case '>':
        //             return (typeMapper.toType<System.SByte>(v1) > typeMapper.toType<System.SByte>(v2));
        //             break;
        //         case '<':
        //             return (typeMapper.toType<System.SByte>(v1) < typeMapper.toType<System.SByte>(v2));
        //             break;
        //     }
        // }
        // The operator should be an XOR ^ instead of an OR, but not available in CodeDOM
        // The operator should be an XOR ^ instead of an OR, but not available in CodeDOM
        // The operator should be an XOR ^ instead of an OR, but not available in CodeDOM
        // The operator should be an XOR ^ instead of an OR, but not available in CodeDOM
        // The operator should be an XOR ^ instead of an OR, but not available in CodeDOM
        // The operator should be an XOR ^ instead of an OR, but not available in CodeDOM
        // The operator should be an XOR ^ instead of an OR, but not available in CodeDOM
        // let ||: number;
        // let Unknown: number;
        switch (op) {
            // case '+':
            //     return (typeMapper.toType<number>(v1) + typeMapper.toType<number>(v2));
            //     break;
            // case '-':
            //     return (typeMapper.toType<number>(v1) - typeMapper.toType<number>(v2));
            //     break;
            // case '*':
            //     return (typeMapper.toType<number>(v1) * typeMapper.toType<number>(v2));
            //     break;
            // case '/':
            //     return (typeMapper.toType<number>(v1) / typeMapper.toType<number>(v2));
            //     break;
            // case '%':
            //     return (typeMapper.toType<number>(v1) % typeMapper.toType<number>(v2));
            //     break;
            // case '^':
            //     return (typeMapper.toType<number>(v1) | typeMapper.toType<number>(v2));
            // break;
            case '=':
                return (typeMapper_1.typeMapper.toType(v1) == typeMapper_1.typeMapper.toType(v2));
                break;
            case '!':
                return (typeMapper_1.typeMapper.toType(v1) != typeMapper_1.typeMapper.toType(v2));
                break;
            case '>':
                return (typeMapper_1.typeMapper.toType(v1) > typeMapper_1.typeMapper.toType(v2));
                break;
            case '<':
                return (typeMapper_1.typeMapper.toType(v1) < typeMapper_1.typeMapper.toType(v2));
                break;
        }
        // The operator should be an XOR ^ instead of an OR, but not available in CodeDOM
        if ((v1 instanceof Number)) {
            switch (op) {
                // case '+':
                //     return (typeMapper.toType<number>(v1) + typeMapper.toType<number>(v2));
                //     break;
                // case '-':
                //     return (typeMapper.toType<number>(v1) - typeMapper.toType<number>(v2));
                //     break;
                // case '*':
                //     return (typeMapper.toType<number>(v1) * typeMapper.toType<number>(v2));
                //     break;
                // case '/':
                //     return (typeMapper.toType<number>(v1) / typeMapper.toType<number>(v2));
                //     break;
                // case '%':
                //     return (typeMapper.toType<number>(v1) % typeMapper.toType<number>(v2));
                //     break;
                // case '^':
                //     return (typeMapper.toType<number>(v1) | typeMapper.toType<number>(v2));
                //     break;
                case '=':
                    return (typeMapper_1.typeMapper.toType(v1) == typeMapper_1.typeMapper.toType(v2));
                    break;
                case '!':
                    return (typeMapper_1.typeMapper.toType(v1) != typeMapper_1.typeMapper.toType(v2));
                    break;
                case '>':
                    return (typeMapper_1.typeMapper.toType(v1) > typeMapper_1.typeMapper.toType(v2));
                    break;
                case '<':
                    return (typeMapper_1.typeMapper.toType(v1) < typeMapper_1.typeMapper.toType(v2));
                    break;
            }
        }
        // else if ((v1 instanceof ((ushort || v2) instanceof ushort))) {
        //     switch (op) {
        //         case '+':
        //             return (typeMapper.toType<number>(v1) + typeMapper.toType<number>(v2));
        //             break;
        //         case '-':
        //             return (typeMapper.toType<number>(v1) - typeMapper.toType<number>(v2));
        //             break;
        //         case '*':
        //             return (typeMapper.toType<number>(v1) * typeMapper.toType<number>(v2));
        //             break;
        //         case '/':
        //             return (typeMapper.toType<number>(v1) / typeMapper.toType<number>(v2));
        //             break;
        //         case '%':
        //             return (typeMapper.toType<number>(v1) % typeMapper.toType<number>(v2));
        //             break;
        //         case '^':
        //             return (typeMapper.toType<number>(v1) | typeMapper.toType<number>(v2));
        //             break;
        //         case '=':
        //             return (typeMapper.toType<number>(v1) == typeMapper.toType<number>(v2));
        //             break;
        //         case '!':
        //             return (typeMapper.toType<number>(v1) != typeMapper.toType<number>(v2));
        //             break;
        //         case '>':
        //             return (typeMapper.toType<number>(v1) > typeMapper.toType<number>(v2));
        //             break;
        //         case '<':
        //             return (typeMapper.toType<number>(v1) < typeMapper.toType<number>(v2));
        //             break;
        //     }
        // }
        // else if ((v1 instanceof ((byte || v2) instanceof byte))) {
        //     switch (op) {
        //         case '+':
        //             return (typeMapper.toType<number>(v1) + typeMapper.toType<number>(v2));
        //             break;
        //         case '-':
        //             return (typeMapper.toType<number>(v1) - typeMapper.toType<number>(v2));
        //             break;
        //         case '*':
        //             return (typeMapper.toType<number>(v1) * typeMapper.toType<number>(v2));
        //             break;
        //         case '/':
        //             return (typeMapper.toType<number>(v1) / typeMapper.toType<number>(v2));
        //             break;
        //         case '%':
        //             return (typeMapper.toType<number>(v1) % typeMapper.toType<number>(v2));
        //             break;
        //         case '^':
        //             return (typeMapper.toType<number>(v1) | typeMapper.toType<number>(v2));
        //             break;
        //         case '=':
        //             return (typeMapper.toType<number>(v1) == typeMapper.toType<number>(v2));
        //             break;
        //         case '!':
        //             return (typeMapper.toType<number>(v1) != typeMapper.toType<number>(v2));
        //             break;
        //         case '>':
        //             return (typeMapper.toType<number>(v1) > typeMapper.toType<number>(v2));
        //             break;
        //         case '<':
        //             return (typeMapper.toType<number>(v1) < typeMapper.toType<number>(v2));
        //             break;
        //     }
        // }
        else if (v1 instanceof DateTime_1.DateTime && v2 instanceof DateTime_1.DateTime) {
            switch (op) {
                case '+':
                    return typeMapper_1.typeMapper.toType(v1);
                    break;
                    // case '-':
                    //     return (typeMapper.toType<DateTime>(v1) - typeMapper.toType<DateTime>(v2));
                    break;
                case '*':
                    return typeMapper_1.typeMapper.toType(v1);
                    break;
                case '/':
                    return typeMapper_1.typeMapper.toType(v1);
                    break;
                case '%':
                    return typeMapper_1.typeMapper.toType(v1);
                    break;
                case '^':
                    return typeMapper_1.typeMapper.toType(v1);
                    break;
                case '=':
                    return (typeMapper_1.typeMapper.toType(v1) == typeMapper_1.typeMapper.toType(v2));
                    break;
                case '!':
                    return (typeMapper_1.typeMapper.toType(v1) != typeMapper_1.typeMapper.toType(v2));
                    break;
                case '>':
                    return (typeMapper_1.typeMapper.toType(v1) > typeMapper_1.typeMapper.toType(v2));
                    break;
                case '<':
                    return (typeMapper_1.typeMapper.toType(v1) < typeMapper_1.typeMapper.toType(v2));
                    break;
            }
        }
        else if (v1 instanceof TimeSpan_1.TimeSpan && v2 instanceof TimeSpan_1.TimeSpan) {
            switch (op) {
                // case '+':
                //     return (typeMapper.toType<TimeSpan>(v1) + typeMapper.toType<TimeSpan>(v2));
                //     break;
                // case '-':
                //     return (typeMapper.toType<TimeSpan>(v1) - typeMapper.toType<TimeSpan>(v2));
                //     break;
                case '*':
                    return typeMapper_1.typeMapper.toType(v1);
                    break;
                case '/':
                    return typeMapper_1.typeMapper.toType(v1);
                    break;
                case '%':
                    return typeMapper_1.typeMapper.toType(v1);
                    break;
                case '^':
                    return typeMapper_1.typeMapper.toType(v1);
                    break;
                case '=':
                    return (typeMapper_1.typeMapper.toType(v1) == typeMapper_1.typeMapper.toType(v2));
                    break;
                case '!':
                    return (typeMapper_1.typeMapper.toType(v1) != typeMapper_1.typeMapper.toType(v2));
                    break;
                case '>':
                    return (typeMapper_1.typeMapper.toType(v1) > typeMapper_1.typeMapper.toType(v2));
                    break;
                case '<':
                    return (typeMapper_1.typeMapper.toType(v1) < typeMapper_1.typeMapper.toType(v2));
                    break;
            }
        }
        else if (v1 instanceof DateTime_1.DateTime && v2 instanceof TimeSpan_1.TimeSpan) {
            switch (op) {
                // case '+':
                //     return (typeMapper.toType<DateTime>(v1) + typeMapper.toType<TimeSpan>(v2));
                //     break;
                // case '-':
                //     return (typeMapper.toType<DateTime>(v1) - typeMapper.toType<TimeSpan>(v2));
                //     break;
                case '*':
                    return typeMapper_1.typeMapper.toType(v1);
                    break;
                case '/':
                    return typeMapper_1.typeMapper.toType(v1);
                    break;
                case '%':
                    return typeMapper_1.typeMapper.toType(v1);
                    break;
                case '^':
                    return typeMapper_1.typeMapper.toType(v1);
                    break;
                case '=':
                    return false;
                    break;
                case '!':
                    return true;
                    break;
                case '>':
                    return true;
                    break;
                case '<':
                    return false;
                    break;
            }
        }
        else if (v1 instanceof TimeSpan_1.TimeSpan && v2 instanceof DateTime_1.DateTime) {
            switch (op) {
                // case '+':
                //     return (typeMapper.toType<DateTime>(v2) + typeMapper.toType<TimeSpan>(v1));
                //     break;
                // case '-':
                //     return (typeMapper.toType<DateTime>(v2) - typeMapper.toType<TimeSpan>(v1));
                //     break;
                case '*':
                    return typeMapper_1.typeMapper.toType(v2);
                    break;
                case '/':
                    return typeMapper_1.typeMapper.toType(v2);
                    break;
                case '%':
                    return typeMapper_1.typeMapper.toType(v2);
                    break;
                case '^':
                    return typeMapper_1.typeMapper.toType(v2);
                    break;
                case '=':
                    return false;
                    break;
                case '!':
                    return true;
                    break;
                case '>':
                    return false;
                    break;
                case '<':
                    return true;
                    break;
            }
        }
        // The operator should be an XOR ^ instead of an OR, but not available in CodeDOM
        // The operator should be an XOR ^ instead of an OR, but not available in CodeDOM
        // The operator should be an XOR ^ instead of an OR, but not available in CodeDOM
        return null;
    }
    static markEnclosure(path, startIndex, startChar, endChar, /* out */ endIndex) {
        this.splitEnclosure(path, startIndex, startChar, endChar, /* out */ endIndex);
    }
    static splitEnclosure(path, startIndex, startChar, endChar, /* out */ endIndex, delimiter = "0") {
        let tokens = new List_1.default();
        tokens = delimiter != "0" ? new List_1.default() : undefined;
        let level = 1;
        let curlyLevel = 0;
        let squareLevel = 0;
        let roundLevel = 0;
        let inString = 0;
        let inChar = 0;
        let sb = new StringBuilder_1.default();
        let i = (startIndex + 1);
        for (; (i < path.length); i++) {
            let c = path[i];
            let otherLevels = curlyLevel + squareLevel + roundLevel + inString + inChar;
            ;
            // We do not ignore the whitespaces here, they should be ignored by the outer expression compiler code or the function param parsing code
            // if (char.IsWhiteSpace(c) && inString == 0 && inChar == 0)
            // {
            //     continue;
            // }
            // else
            if ((c == startChar)) {
                level++;
            }
            else if ((c == endChar)) {
                if ((level > 0)) {
                    level--;
                    if ((level == 0)) {
                        break;
                    }
                }
                else {
                    throw new unterminatedEnclosureException_1.unterminatedEnclosureException(startChar, endChar);
                }
            }
            else if ((c == "\"")) {
                inString = inString == 0 ? 1 : 0;
            }
            else if ((c == "\'")) {
                inChar = inChar == 0 ? 1 : 0;
            }
            else if ((c == "(")) {
                roundLevel++;
            }
            else if ((c == ")")) {
                if ((roundLevel > 0)) {
                    roundLevel--;
                }
                else {
                    throw new unterminatedEnclosureException_1.unterminatedEnclosureException('(', ')');
                }
            }
            else if ((c == "{")) {
                curlyLevel++;
            }
            else if ((c == "}")) {
                if ((curlyLevel > 0)) {
                    curlyLevel--;
                }
                else {
                    throw new unterminatedEnclosureException_1.unterminatedEnclosureException('{', '}');
                }
            }
            else if ((c == "[")) {
                squareLevel++;
            }
            else if ((c == "]")) {
                if ((squareLevel > 0)) {
                    squareLevel--;
                }
                else {
                    throw new unterminatedEnclosureException_1.unterminatedEnclosureException('[', ']');
                }
            }
            if (delimiter != "0" && level == 1 && otherLevels == 0 && c == delimiter) {
                tokens === null || tokens === void 0 ? void 0 : tokens.add(sb.toString());
                sb.clear();
            }
            else {
                sb.append(c);
            }
        }
        if ((inString > 0)) {
            throw new unterminatedStringLiteralException_1.unterminatedStringLiteralException();
        }
        if (((delimiter != "0")
            && (sb.toString().length > 0))) {
            tokens === null || tokens === void 0 ? void 0 : tokens.add(sb.toString().trim());
        }
        if ((level > 0)) {
            throw new unterminatedEnclosureException_1.unterminatedEnclosureException(startChar, endChar);
        }
        else {
            endIndex.endIndex = i;
        }
        return tokens;
    }
    static buildSwitchCase(path, startIndex, /* out */ endIndex, flags) {
        return this.buildSwitchCaseOverload(this.splitEnclosure(path, startIndex, "{", "}", /* out */ endIndex, ","), flags);
    }
    static buildSwitchCaseOverload(caseList, flags) {
        var _a;
        let data = new switchPartData_1.switchPartData();
        let foundDefaultPart = false;
        let foundNullPart = false;
        for (let caseString of caseList.toArray()) {
            let expression = expressionCompiler_1.expressionCompiler.compile(caseString, undefined, undefined, flags);
            if ((expression instanceof operationPart_1.operationPart && (((_a = (expression).operator) === null || _a === void 0 ? void 0 : _a.name) == "="))) {
                let op = (expression);
                if ((op.parameter1 instanceof indicatorPart_1.indicatorPart && (op.parameter1).value.equals("default"))) {
                    foundDefaultPart = true;
                    data.DefaultValue = op.parameter2;
                }
                else if ((op.parameter1 instanceof nullPart_1.nullPart)) {
                    if (foundNullPart) {
                        throw new badExpressionException_1.badExpressionException("Switch can have only one null value case");
                    }
                    data.NullValue = op.parameter2;
                    foundNullPart = true;
                }
                else {
                    data.Cases.push({ key: op.parameter1, value: op.parameter2 });
                }
            }
            else {
                if (foundDefaultPart) {
                    throw new badExpressionException_1.badExpressionException("Switch can have only one default case");
                }
                data.DefaultValue = expression;
                foundDefaultPart = true;
            }
        }
        return data;
    }
    static getIndexer(type) {
        let indexerProperty = undefined;
        this.getProperties(type).forEach((prop) => {
            if ((prop.name == "Items")) {
                indexerProperty = prop;
            }
        });
        return indexerProperty;
    }
    static getProperties(type) {
        return this.getPropertiesMap(type).getValues();
    }
    static getPropertiesMap(type) {
        let infos = new dictionaryObject_1.DictionaryObject();
        reflectionExtender_1.getRuntimeProperties(type).forEach((pi) => {
            infos.setValue(pi.name, pi);
        });
        return infos;
    }
    static getProperty(type, name) {
        let props = this.getPropertiesMap(type);
        try {
            return props.containsKey(name) ? props.getValue(name) : this.getBuilInProperty(name, type);
        }
        catch (e) {
            return this.getBuilInProperty(name, type);
        }
    }
    static getKeyParamCache(param) {
        let strKey = "";
        if (param instanceof partList_1.partList) {
            for (let index = 0; index < param.Parts.length; index++) {
                const propPart = param.Parts[index];
                if (propPart instanceof propertyPart_1.propertyPart) {
                    const propName = propPart.propertyName;
                    if (propName === "Controls" || propName === 'Login' || propName === 'Var' || propName === 'In') {
                        strKey += propName;
                    }
                }
                if (strKey !== "" && propPart instanceof indexerPart_1.indexerPart) {
                    if (propPart.indexerExpression) {
                        strKey += propPart.indexerExpression.value;
                    }
                }
            }
        }
        return strKey;
    }
    static getBuilInProperty(name, obj) {
        let prop = new reflectionExtender_1.propertyInfo();
        const pr = legacyBuiltInProperty_1.default[name.toLowerCase()];
        prop.name = name.toLowerCase();
        prop.parameterType = pr;
        prop.type = obj;
        return prop;
    }
    static matchDates(d1, d2, matchYear) {
        if (d1 instanceof DateTime_1.DateTime && d2 instanceof DateTime_1.DateTime)
            return matchYear ? d1.date.equals(d2.date) : d1.day == d2.day && d1.month == d2.month;
        else if (typeof d1 == "string") {
            try {
                let d = new DateTime_1.DateTime(d1);
                return d.date.equals(d2.date);
            }
            catch (e) {
                return false;
            }
        }
        return true;
    }
    static compare(v1, v2, op) {
        if (((v1 == v2)
            || ((v1 != null)
                && v1 === v2))) {
            return (op == '=');
        }
        // else if (((v1 != null && v2 != null&& v1 instanceof IComparable && v1.GetType().GetTypeInfo().IsAssignableFrom(v2.GetType().GetTypeInfo())))) {
        //     let compareResult: number = (<IComparable>(v1)).CompareTo(v2);
        //     switch (op) {
        //         case '=':
        //             return (compareResult == 0);
        //             break;
        //         case '!':
        //             return (compareResult != 0);
        //             break;
        //         case '>':
        //             return (compareResult > 0);
        //             break;
        //         case '<':
        //             return (compareResult < 0);
        //             break;
        //     }
        // }
        else if (((v1 != null)
            && ((v2 != null)
                && (this.supportsOperators(v1) && this.supportsOperators(v2))))) {
            let result = this.toBiggerTypeOperation(v1, v2, op);
            if ((result != null)) {
                return (result);
            }
        }
        return (op == '!');
        //  throw new InvalidOperationException(string.Format("Could not compare {0} with {1} for operator {2}", v1 != null ? v1.GetType().ToString() : "null", v2 != null ? v2.GetType().ToString() : "null", op));
    }
}
exports.utils = utils;
//# sourceMappingURL=utils.js.map