"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const stack_1 = require("./utils/stack");
const List_1 = require("typescript-dotnet-commonjs/System/Collections/List");
const expressionCompiler_1 = require("./compiler/expressionCompiler");
// import { getHashCode } from "typescript-dotnet-commonjs/System/Text/Utility";
require("./utils/stringUtil");
class expressionData {
    constructor(expression, pageContext, globalContext, legacyMode = true, convertType = true) {
        this.subExpressions = new List_1.default();
        this.contexts = new stack_1.Stack();
        this.convertType = false;
        this.monitorExpression = false;
        this.cacheEnabled = false;
        this.legacyMode = false;
        this.isAlreadyLegacy = false;
        this.dtFormat = undefined;
        this.constructorInit(expression, pageContext, globalContext, legacyMode, convertType, false);
        this.cacheEnabled = true;
    }
    constructorInit(expression, pageContext, globalContext, legacyMode, convertType = true, isAlreadyLegacy = false) {
        this.expression = expression;
        this.convertType = convertType;
        this.globalContext = this.globalContext;
        this.pageContext = this.pageContext;
        this.isAlreadyLegacy = isAlreadyLegacy;
        this.legacyMode = legacyMode;
        this.cacheEnabled = true;
        this.contexts = new stack_1.Stack();
        if ((this.globalContext != null)) {
            this.contexts.push(this.globalContext);
        }
        if ((this.pageContext != null)) {
            this.contexts.push(this.pageContext);
        }
        this.toLegacyIfNeeded();
    }
    privateConstructor(expression, contexts, pageContext, globalContext, legacyMode, convertType = true, isAlreadyLegacy = false) {
        this.expression = expression;
        this.convertType = convertType;
        this.globalContext = globalContext;
        this.pageContext = pageContext;
        this.contexts = contexts.clone();
        this.isAlreadyLegacy = isAlreadyLegacy;
        this.legacyMode = legacyMode;
        this.cacheEnabled = true;
        this.toLegacyIfNeeded();
        return this;
    }
    toSubEx(currentContext = null) {
        return this.toSubExInit(this.expression, currentContext);
    }
    toSubExInit(newExpression, currentContext = null) {
        let subExpressionData = this.privateConstructor(newExpression, this.contexts, this.pageContext, this.globalContext, this.legacyMode, this.convertType, this.isAlreadyLegacy);
        if (((currentContext != null)
            && ((currentContext != this.pageContext)
                && ((currentContext != this.globalContext)
                    && !subExpressionData.contexts.contains(currentContext))))) {
            subExpressionData.contexts.push(currentContext);
        }
        if (this.parent != null) {
            subExpressionData.parent = this.parent;
        }
        else {
            subExpressionData = this;
        }
        subExpressionData.cacheEnabled = this.cacheEnabled;
        this.subExpressions.add(subExpressionData);
        return subExpressionData;
    }
    /* override */ equals(obj) {
        if ((obj instanceof expressionData)) {
            let ed = (obj);
            return (this.expression == ed.expression);
        }
        return false;
    }
    // getHashCode(): number {
    //     let hash: number = 13;
    //     //  hash = (hash * 7) + (this.Binding != null ? this.Binding.GetHashCode() : 0);
    //     hash = ((hash * 7) + getHashCode(this.expression));
    //     return hash;
    // }
    toLegacyIfNeeded() {
        if ((!this.isAlreadyLegacy
            && this.legacyMode)) {
            this.isAlreadyLegacy = true;
            this.expression = expressionCompiler_1.expressionCompiler.ProcessLegacy(this.expression);
        }
    }
    toString() {
        return this.expression;
    }
}
exports.default = expressionData;
//# sourceMappingURL=expressionData.js.map