"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.expressionEngine = void 0;
const expressionData_1 = require("./expressionData");
const expressionCompiler_1 = require("./compiler/expressionCompiler");
const List_1 = require("typescript-dotnet-commonjs/System/Collections/List");
const reflectionExtender_1 = require("./utils/reflectionExtender");
const outputVariables_1 = require("./utils/outputVariables");
const ItemSourceContexts_1 = require("./utils/ItemSourceContexts");
class expressionEngine {
    constructor() {
    }
    static getInstance() {
        if ((this.instance == null)) {
            this.instance = new expressionEngine();
        }
        return this.instance;
    }
    static evalGeneric(context, expression, secondContext, globalContext, legacyMode = false, data, setValue = undefined) {
        let tType;
        if (!expression) {
            return undefined;
        }
        if (tType == undefined)
            tType = reflectionExtender_1.getType(context);
        if (data != undefined) {
            return expressionEngine.eval(context, "", legacyMode, data, tType, setValue);
        }
        let result = expressionEngine.evalPrivate(context, expression, secondContext, globalContext, tType, legacyMode);
        return result;
    }
    static eval(context, expression, legacyMode = true, data, targetType, setValue = undefined) {
        if (data == undefined) {
            return expressionEngine.evalPrivate(context, expression, undefined, undefined, reflectionExtender_1.getType(Object), legacyMode, null);
        }
        else {
            if (context != null && !data.contexts.contains(context))
                data.contexts.push(context);
            return expressionEngine.valueFor(context, data, targetType, setValue);
        }
    }
    static evalPrivate(context, expression, secondContext, globalContext, targetType, legacyMode = true, setValue = null) {
        let data = new expressionData_1.default(expression, secondContext, globalContext, legacyMode);
        if ((context != null)) {
            data.contexts.push(context);
        }
        if (secondContext != null) {
            data.contexts.push(secondContext);
        }
        if (globalContext != null) {
            data.contexts.push(globalContext);
        }
        let result = undefined;
        result = expressionEngine.eval(context, "", legacyMode, data, undefined, targetType);
        return result;
    }
    static valueFor(subject, data, targetType, setValue) {
        try {
            let expression;
            let compileflag = new expressionCompiler_1.compileFlags();
            compileflag.legacyMode = data.legacyMode;
            expression = expressionCompiler_1.expressionCompiler.compile(data.expression, undefined, undefined, compileflag);
            let resolved = new outputVariables_1.outputVariables();
            resolved.resolved = true;
            if (data.contexts.peek() instanceof ItemSourceContexts_1.ItemSourceContexts) {
                let secondExpressionPart = undefined;
                const itemSourceContext = data.contexts.pop();
                if (itemSourceContext.secondExpression) {
                    secondExpressionPart = expressionCompiler_1.expressionCompiler.compile(itemSourceContext.secondExpression, undefined, undefined, compileflag);
                }
                let results = [];
                let contexts = itemSourceContext.contexts instanceof List_1.default ? itemSourceContext.contexts.toArray() : itemSourceContext.contexts;
                for (const item of contexts) {
                    data.contexts.push(item);
                    let result = expression === null || expression === void 0 ? void 0 : expression.resolveClass(data, resolved, targetType);
                    let result2 = secondExpressionPart === null || secondExpressionPart === void 0 ? void 0 : secondExpressionPart.resolveClass(data, resolved, targetType);
                    results.push({
                        result: result,
                        result2: result2,
                        context: item
                    });
                    data.contexts.pop();
                }
                // if (results.length !== 0)
                return results;
            }
            let result = expression === null || expression === void 0 ? void 0 : expression.resolveClass(data, resolved, targetType);
            return result;
        }
        catch (e) {
            throw e;
        }
    }
}
exports.expressionEngine = expressionEngine;
expressionEngine.DEFAULT_LEGACY_MODE = true;
//# sourceMappingURL=expressionEngine.js.map