"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.functionsList = void 0;
const find_1 = require("./globals/find");
const groupBy_1 = require("./globals/groupBy");
const select_1 = require("./globals/select");
const string_1 = require("./globals/string");
const literals_1 = require("./globals/literals");
const math_1 = require("./globals/math");
const evals_1 = require("./globals/evals");
const sum_1 = require("./aggregates/sum");
const max_1 = require("./aggregates/max");
const min_1 = require("./aggregates/min");
const last_1 = require("./aggregates/last");
const count_1 = require("./aggregates/count");
const groupConcat_1 = require("./aggregates/groupConcat");
const first_1 = require("./aggregates/first");
const date_1 = require("./globals/date");
const join_1 = require("./globals/join");
const sort_1 = require("./globals/sort");
const switch_1 = require("./globals/switch");
const dictionaryObject_1 = require("../utils/dictionaryObject");
class functionsList {
    static get ExpressionFunctions() {
        if (functionsList.functions == undefined) {
            this.init();
        }
        return functionsList.functions;
    }
    static get AggregateFunctions() {
        return this.aggregateFunctions;
    }
    static init() {
        functionsList.functions = new dictionaryObject_1.DictionaryObject();
        functionsList.functions.add("Find", new find_1.find());
        functionsList.functions.add("FindAll", new find_1.findAll());
        functionsList.functions.add("Limit", new find_1.limit());
        functionsList.functions.add("Exists", new find_1.exists());
        functionsList.functions.add("Eval", new evals_1.eval1());
        functionsList.functions.add("GroupBy", new groupBy_1.groupBy());
        let groupByNamed = new groupBy_1.groupBy();
        groupByNamed.namedSelection = true;
        functionsList.functions.add("GroupByNamed", groupByNamed);
        functionsList.functions.add("Select", new select_1.select());
        let selectNamed = new select_1.select();
        selectNamed.namedSelection = true;
        functionsList.functions.add("SelectNamed", selectNamed);
        functionsList.functions.add("Matches", new string_1.matches());
        functionsList.functions.add("Substring", new string_1.substring());
        functionsList.functions.add("Replace", new string_1.replace());
        functionsList.functions.add("Split", new string_1.split());
        functionsList.functions.add("Concat", new string_1.concat());
        functionsList.functions.add("ConcatNonEmpty", new string_1.concatNonEmpty());
        functionsList.functions.add("Coalesce", new literals_1.coalesce());
        functionsList.functions.add("Math_Abs", new math_1.math_Abs());
        functionsList.functions.add("Math_Acos", new math_1.math_Acos());
        functionsList.functions.add("Math_Min", new math_1.math_Min());
        functionsList.functions.add("Math_Pow", new math_1.math_Pow());
        functionsList.functions.add("Math_Pi", new math_1.math_Pi());
        functionsList.functions.add("Math_Round", new math_1.math_Round());
        functionsList.functions.add("Math_Sum", new math_1.math_Sum());
        functionsList.functions.add("Math_Tan", new math_1.math_Tan());
        functionsList.functions.add("Math_Cosh", new math_1.math_Cosh());
        functionsList.functions.add("Math_Tanh", new math_1.math_Tanh());
        functionsList.functions.add("Math_Exp", new math_1.math_Exp());
        functionsList.functions.add("Math_Sin", new math_1.math_Sin());
        functionsList.functions.add("Bool", new evals_1.Bool());
        functionsList.functions.add("Math_Asin", new math_1.math_Asin());
        functionsList.functions.add("Math_Atan", new math_1.math_Atan());
        functionsList.functions.add("Math_Atan2", new math_1.math_Atan2());
        functionsList.functions.add("Math_Cos", new math_1.math_Cos());
        functionsList.functions.add("Math_Log", new math_1.math_Log());
        functionsList.functions.add("Math_Max", new math_1.math_Max());
        functionsList.functions.add("Contains", new string_1.contains());
        functionsList.functions.add("ToDate", new literals_1.toDate());
        functionsList.functions.add("Now", new date_1.now());
        functionsList.functions.add("UTCNow", new date_1.utcNow());
        functionsList.functions.add("IsToday", new date_1.isToday());
        functionsList.functions.add("IsDate", new date_1.isDate());
        functionsList.functions.add("ToList", new literals_1.toList());
        functionsList.functions.add("ToDictionary", new literals_1.toDictionary());
        functionsList.functions.add("Percent", new evals_1.percent());
        functionsList.functions.add("ToLower", new string_1.toLower());
        functionsList.functions.add("ToUpper", new string_1.toUpper());
        functionsList.functions.add("ToLocalTime", new date_1.toLocalTime());
        functionsList.functions.add("Math_Sqrt", new math_1.math_Sqrt());
        functionsList.functions.add("ToNumber", new literals_1.toNumber());
        functionsList.functions.add("ToString", new string_1.toString());
        functionsList.functions.add("ToObjectList", new literals_1.toObjectList());
        functionsList.functions.add("ToObject", new literals_1.toObject());
        functionsList.functions.add("OrderBy", new sort_1.orderBy());
        functionsList.functions.add("Switch", new switch_1.aswitch());
        functionsList.functions.add("RegexReplace", new string_1.regexpReplace());
        functionsList.functions.add("ToUtc", new date_1.toUtc());
        functionsList.functions.add("AddPeriod", new date_1.addPeriod());
        functionsList.functions.add("Format", new string_1.stringformat());
        functionsList.functions.add("InnerJoin", new join_1.innerJoin());
        functionsList.functions.add("Join", new join_1.join());
        functionsList.functions.add("LeftJoin", new join_1.leftJoin());
        functionsList.functions.add("RightJoin", new join_1.rightJoin());
        functionsList.functions.add("FullJoin", new join_1.fullJoin());
        functionsList.aggregateFunctions = new dictionaryObject_1.DictionaryObject();
        functionsList.aggregateFunctions.add("Sum", new sum_1.sum());
        functionsList.aggregateFunctions.add("Count", new count_1.count());
        functionsList.aggregateFunctions.add("Max", new max_1.max());
        functionsList.aggregateFunctions.add("Min", new min_1.min());
        functionsList.aggregateFunctions.add("GroupConcat", new groupConcat_1.groupConcat());
        functionsList.aggregateFunctions.add("First", new first_1.first());
        functionsList.aggregateFunctions.add("Last", new last_1.last());
    }
}
exports.functionsList = functionsList;
//# sourceMappingURL=functionsList.js.map