"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.groupBy = void 0;
const List_1 = require("typescript-dotnet-commonjs/System/Collections/List");
const operationPart_1 = require("../../compiler/parts/operationPart");
const expressionCompiler_1 = require("../../compiler/expressionCompiler");
const invalidParametersException_1 = require("../../error/invalidParametersException");
const mfunction_1 = require("../mfunction");
const groupDataContext_1 = require("../groupDataContext");
const expressionCompiler_2 = require("../../compiler/expressionCompiler");
const StringBuilder_1 = require("typescript-dotnet-commonjs/System/Text/StringBuilder");
const namedSelection_1 = require("../namedSelection");
const aggregateFunctionCallPart_1 = require("../../compiler/parts/aggregateFunctionCallPart");
const outputVariables_1 = require("../../utils/outputVariables");
const selection_1 = require("../selection");
const dictionaryObject_1 = require("../../utils/dictionaryObject");
class groupBy extends mfunction_1.mfunction {
    constructor() {
        super(...arguments);
        this.namedSelection = false;
    }
    /* override */ get minParameters() {
        return 1;
    }
    /* override */ get maxParameters() {
        return -1;
    }
    /* internal */ /* override */ compileParameters(paramList, flags) {
        if ((((this.minParameters > 0)
            && (paramList.count < this.minParameters))
            || ((this.maxParameters >= 0)
                && (paramList.count > this.maxParameters)))) {
            throw new invalidParametersException_1.invalidParametersException(super.getFunctionName(), this.minParameters, this.maxParameters);
        }
        let parameters = [];
        for (let item of paramList.toArray()) {
            let compileF = new expressionCompiler_2.compileFlags(flags);
            compileF.aggregationSection = true;
            parameters.push(expressionCompiler_1.expressionCompiler.compile(item, 0, (item.length - 1), compileF));
        }
        return parameters;
    }
    invoke(expressionData, parameters, /* out */ resolved, running) {
        let obj = expressionData.contexts.peek();
        resolved.resolved = true;
        let tempData = expressionData.toSubEx();
        let isNamedSelection = (this.namedSelection || parameters.some((p) => { var _a; return p instanceof operationPart_1.operationPart && ((_a = p.operator) === null || _a === void 0 ? void 0 : _a.name) == "@"; }));
        if ((obj instanceof List_1.List)) {
            let result = new List_1.List();
            let done = false;
            while (!done) {
                result.clear();
                //  Clear();
                try {
                    let groupedFields = new dictionaryObject_1.DictionaryObject();
                    let groupData = new dictionaryObject_1.DictionaryObject();
                    for (let item of obj.toArray()) {
                        let itemKeyBuilder = new StringBuilder_1.StringBuilder();
                        let itemVals = new Array(parameters.length);
                        tempData.contexts.push(item);
                        for (let i = 0; (i < parameters.length); i++) {
                            let groupExpression = parameters[i];
                            if (!this.HasAggregateFunction(groupExpression)) {
                                let subResolved = new outputVariables_1.outputVariables();
                                let val = groupExpression.resolve(tempData, /* out */ subResolved, running);
                                let groupKey = undefined;
                                if ((isNamedSelection && !(val instanceof dictionaryObject_1.DictionaryObject))) {
                                    let tempVal = val;
                                    val = new dictionaryObject_1.DictionaryObject();
                                    val.add(groupExpression.simpleName, tempVal);
                                }
                                if ((val != undefined && val instanceof dictionaryObject_1.DictionaryObject)) {
                                    groupKey = (val).getValues()[0];
                                }
                                else {
                                    groupKey = val;
                                }
                                itemVals[i] = val;
                                if (subResolved.resolved) {
                                    if (groupKey instanceof dictionaryObject_1.DictionaryObject)
                                        itemKeyBuilder.append(groupKey.getValues()[0]).append(",");
                                    else
                                        itemKeyBuilder.append(groupKey).append(",");
                                }
                            }
                        }
                        let itemKey = itemKeyBuilder.toString();
                        if ((itemKey.length > 0)) {
                            if (!groupedFields.containsKey(itemKey)) {
                                groupedFields[itemKey] = itemVals;
                            }
                            else {
                                itemVals = groupedFields[itemKey];
                            }
                            let itemGroupData;
                            if (groupData.containsKey(itemKey)) {
                                itemGroupData = groupData.getValue(itemKey);
                            }
                            else {
                                itemGroupData = new groupDataContext_1.groupDataContext();
                                groupData.setValue(itemKey, itemGroupData);
                            }
                            for (let i = 0; (i < parameters.length); i++) {
                                let groupExpression = parameters[i];
                                let subResolved = new outputVariables_1.outputVariables();
                                if (this.HasAggregateFunction(groupExpression)) {
                                    //  AggregateFunctionCallPart aggregateFunc = groupExpression as AggregateFunctionCallPart;
                                    //  string groupToken = groupExpression.Resolve<string>(tempData, out subResolved);
                                    let val = groupExpression.resolve(tempData, /* out */ subResolved, /* running:*/ itemGroupData);
                                    if (this.namedSelection && !(val instanceof dictionaryObject_1.DictionaryObject)) {
                                        val = new dictionaryObject_1.DictionaryObject();
                                        val.add(groupExpression.simpleName, val);
                                    }
                                    itemVals[i] = val;
                                }
                            }
                        }
                        tempData.contexts.pop();
                    }
                    for (let values of groupedFields.getValues()) {
                        if (isNamedSelection) {
                            result.add(new namedSelection_1.namedSelection(values));
                        }
                        else if ((values.length == 1)) {
                            result.add(values[0]);
                        }
                        else {
                            result.add(new selection_1.selection(values));
                        }
                    }
                    done = true;
                }
                catch (e) {
                    throw e;
                }
            }
            return result;
        }
        resolved.resolved = false;
        return obj;
    }
    HasAggregateFunction(part) {
        if ((part instanceof aggregateFunctionCallPart_1.aggregateFunctionCallPart)) {
            return true;
        }
        else if ((part.subParts != null)) {
            for (let item of part.subParts) {
                if (this.HasAggregateFunction(item)) {
                    return true;
                }
            }
        }
        return false;
    }
}
exports.groupBy = groupBy;
//# sourceMappingURL=groupBy.js.map