"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.rightJoin = exports.leftJoin = exports.innerJoin = exports.fullJoin = exports.join = exports.joinContext = void 0;
const List_1 = require("typescript-dotnet-commonjs/System/Collections/List");
const mfunction_1 = require("./../mfunction");
const selection_1 = require("./../selection");
const reflectionExtender_1 = require("../../utils/reflectionExtender");
const outputVariables_1 = require("../../utils/outputVariables");
var joinType;
(function (joinType) {
    joinType[joinType["REGULAR"] = 0] = "REGULAR";
    joinType[joinType["FULL"] = 1] = "FULL";
    joinType[joinType["INNER"] = 2] = "INNER";
    joinType[joinType["LEFT"] = 3] = "LEFT";
    joinType[joinType["RIGHT"] = 4] = "RIGHT";
})(joinType || (joinType = {}));
class joinContext {
    constructor() {
    }
    //  get item2(): Object {
    // }
    //  set item2(value: Object)  {
    // }
    get left() {
        return this.item1;
    }
    set left(value) {
        this.item1 = value;
    }
    get right() {
        return this.item2;
    }
    set right(value) {
        this.item2 = value;
    }
}
exports.joinContext = joinContext;
class baseJoin extends mfunction_1.mfunction {
    /* override */ get minParameters() {
        return 2;
    }
    /* override */ get maxParameters() {
        return 2;
    }
    invoke(expressionData, parameters, /* out */ resolved, running) {
        return this.join(expressionData, parameters, this.type, /* out */ resolved, running);
    }
    join(expressionData, joinParams, type, /* out */ resolved, running) {
        let left = expressionData.contexts.peek();
        let tempData = expressionData.toSubEx();
        tempData.cacheEnabled = false;
        if (left.getEnumerator() !== undefined) {
            let rightResolved = new outputVariables_1.outputVariables();
            let right = undefined;
            if ((joinParams.length > 0)) {
                right = joinParams[0].resolve(expressionData, /* out */ rightResolved, running);
            }
            let main = undefined;
            let sub = undefined;
            let joinCondition = undefined;
            switch (type) {
                case joinType.REGULAR:
                case joinType.FULL:
                case joinType.INNER:
                case joinType.LEFT:
                    main = (left);
                    sub = (right);
                    joinCondition = type == joinType.REGULAR ? undefined : joinParams.length > 1 ? joinParams[1] : undefined;
                    break;
                case joinType.RIGHT:
                    main = (right);
                    sub = (left);
                    joinCondition = joinParams.length > 1 ? joinParams[1] : undefined;
                    break;
            }
            let done = false;
            while (!done) {
                try {
                    if ((main != null) && (sub != null)) {
                        let tempResult = new List_1.List();
                        let mainCounter = 0;
                        let _joinContext = new joinContext();
                        for (let item1 of main.toArray()) {
                            let subSelection = new List_1.List();
                            let subCounter = 0;
                            for (let item2 of sub.toArray()) {
                                _joinContext.item1 = type == joinType.RIGHT ? item2 : item1;
                                _joinContext.item2 = type == joinType.RIGHT ? item1 : item2;
                                tempData.contexts.push(_joinContext);
                                let subResolved = new outputVariables_1.outputVariables();
                                let condition = ((joinCondition == null)
                                    || joinCondition.resolveGeneric(tempData, /* out */ subResolved, reflectionExtender_1.getType(Boolean), /* running:*/ running));
                                if (subResolved.resolved && condition) {
                                    var sjoinContext = new joinContext();
                                    sjoinContext.item1 = _joinContext.item1;
                                    sjoinContext.item2 = _joinContext.item2;
                                    subSelection.add(sjoinContext);
                                }
                                tempData.contexts.pop();
                                subCounter++;
                            }
                            if ((subSelection.count > 0)) {
                                for (let item of subSelection.toArray()) {
                                    tempResult.add(item);
                                }
                            }
                            else {
                                switch (type) {
                                    case joinType.RIGHT:
                                    case joinType.LEFT:
                                        let jContext = new joinContext();
                                        jContext.item1 = type == joinType.RIGHT ? null : item1;
                                        jContext.item2 = type == joinType.RIGHT ? item1 : null;
                                        tempResult.add(jContext);
                                        break;
                                }
                            }
                            mainCounter++;
                        }
                        resolved.resolved = true;
                        return baseJoin.normalize(tempResult);
                    }
                    done = true;
                }
                catch (ex) {
                    let d = ex;
                }
            }
        }
        resolved.resolved = false;
        return left;
    }
    static normalize(items) {
        let result = new List_1.List();
        for (let item of items.toArray()) {
            let rSelection = new selection_1.selection(undefined);
            if (item.item1.clear !== undefined) {
                for (let sItem of (item.item1).toArray()) {
                    rSelection.add(sItem);
                }
            }
            else {
                rSelection.add(item.item1);
            }
            if ((item.item2 instanceof selection_1.selection)) {
                for (let sItem in (item.item2)) {
                    rSelection.add(sItem);
                }
            }
            else {
                rSelection.add(item.item2);
            }
            result.add(rSelection);
        }
        return result;
    }
    toArray(v) {
        if (v.values !== undefined) {
            return v;
        }
        else {
            let arrayObject = new List_1.List();
            arrayObject.add(v);
            return v instanceof Object ? v : arrayObject.toArray();
        }
    }
}
class join extends baseJoin {
    /* override */ get minParameters() {
        return 1;
    }
    /* override */ get maxParameters() {
        return 1;
    }
    get type() {
        return joinType.REGULAR;
    }
}
exports.join = join;
class fullJoin extends baseJoin {
    get type() {
        return joinType.FULL;
    }
}
exports.fullJoin = fullJoin;
class innerJoin extends baseJoin {
    get type() {
        return joinType.INNER;
    }
}
exports.innerJoin = innerJoin;
class leftJoin extends baseJoin {
    get type() {
        return joinType.LEFT;
    }
}
exports.leftJoin = leftJoin;
class rightJoin extends baseJoin {
    get type() {
        return joinType.RIGHT;
    }
}
exports.rightJoin = rightJoin;
//# sourceMappingURL=join.js.map