"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.orderBy = void 0;
const sortParameterPart_1 = require("./../../compiler/parts/sortParameterPart");
const expressionCompiler_1 = require("./../../compiler/expressionCompiler");
const List_1 = require("typescript-dotnet-commonjs/System/Collections/List");
const mfunction_1 = require("./../mfunction");
const outputVariables_1 = require("../../utils/outputVariables");
class orderBy extends mfunction_1.mfunction {
    /* override */ get minParameters() {
        return 1;
    }
    /* override */ get maxParameters() {
        return -1;
    }
    /* internal */ /* override */ compileParameters(paramList, flags) {
        let parameters = [];
        for (let item of paramList.toArray()) {
            let spp = new sortParameterPart_1.sortParameterPart();
            let tokens = item.split(' ');
            if ((tokens.length > 1)) {
                spp.ascending = tokens[1].trim().startsWith("asc"); //, stringComparison.OrdinalIgnoreCase);
            }
            else {
                spp.ascending = true;
            }
            spp.expression = expressionCompiler_1.expressionCompiler.compile(tokens[0], undefined, undefined, flags);
            parameters.push(spp);
        }
        return parameters;
    }
    invoke(expressionData, parameters, /* out */ resolved, running) {
        let obj = expressionData.contexts.peek();
        let tempData = expressionData.toSubEx();
        if (obj instanceof List_1.default) {
            let orderedResult = orderBy.sortBy(tempData, obj, (parameters[0]), false, undefined);
            for (let i = 1; (i < parameters.length); i++) {
                orderedResult = orderBy.sortBy(tempData, orderedResult, (parameters[i]), true, undefined);
            }
            let result = new List_1.default();
            resolved.resolved = true;
            orderedResult === null || orderedResult === void 0 ? void 0 : orderedResult.forEach((o) => {
                result.add(o);
            });
            return result;
        }
        resolved.resolved = false;
        return obj;
    }
    static sortBy(tempData, items, sortBy, thenBy, running) {
        let sortKey = sortBy.expression;
        let ascending = sortBy.ascending;
        try {
            const compareFunctions = (a, b) => {
                let resolved = new outputVariables_1.outputVariables();
                tempData.contexts.push(a);
                let r = sortKey.resolve(tempData, resolved, running);
                tempData.contexts.pop();
                resolved = new outputVariables_1.outputVariables();
                tempData.contexts.push(b);
                let r1 = sortKey.resolve(tempData, resolved, running);
                tempData.contexts.pop();
                if (typeof r === 'string' && typeof r1 === 'string') {
                    if (r.toLowerCase() < r1.toLowerCase()) {
                        return -1;
                    }
                    if (r.toLowerCase() > r1.toLowerCase()) {
                        return 1;
                    }
                    return 0;
                }
                else if (typeof r === 'number' && typeof r1 === 'number') {
                    return r - r1;
                }
                return 0;
                // if (r instanceof Date && r1 instanceof Date) {
                // }
            };
            const keyFuncAsc = (a, b) => {
                return compareFunctions(a, b);
            };
            const keyFuncDesc = (a, b) => {
                return compareFunctions(b, a);
            };
            if (thenBy) {
                if (ascending) {
                    let result = items.sort(keyFuncAsc);
                    return result;
                }
                else {
                    let result = items.sort(keyFuncDesc);
                    return result;
                }
            }
            else if (ascending) {
                let result = items.sort(keyFuncAsc);
                return result;
            }
            else {
                let result = items.sort(keyFuncDesc);
                ;
                return result;
            }
        }
        catch (e) {
            throw e;
        }
    }
}
exports.orderBy = orderBy;
//# sourceMappingURL=sort.js.map