"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.DelegateBuilder = void 0;
class DelegateBuilder {
    constructor() {
        this.callees = [];
    }
    invoke(sender, ...parameters) {
        for (let i = 0; i < this.callees.length; i++) {
            const callee = this.callees[i];
            if (callee)
                callee(sender, parameters);
        }
    }
    contains(callee) {
        if (!callee)
            return false;
        return this.callees.indexOf(callee) >= 0;
    }
    add(callee) {
        if (!callee)
            return this;
        if (!this.contains(callee))
            this.callees.push(callee);
        return this;
    }
    remove(callee) {
        if (!callee)
            return this;
        const index = this.callees.indexOf(callee);
        if (index >= 0)
            this.callees.splice(index, 1);
        return this;
    }
    toDelegate() {
        return (sender, ...parameters) => this.invoke(sender, parameters);
    }
}
exports.DelegateBuilder = DelegateBuilder;
//# sourceMappingURL=delegateBuilder.js.map