"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const DateTime_1 = require("typescript-dotnet-commonjs/System/Time/DateTime");
const general_1 = require("./general");
const legacyBuiltInProperty = {
    ["length"]: undefined,
    ["count"]: undefined,
    ["ticks"]: function (value) { return value; },
    ["date"]: function (value) { return Date.parse(new Date(value).toDateString()); },
    ["timeofday"]: function (value) {
        const dt = new Date(value);
        return dt.toLocaleString();
    },
    ["year"]: function (value) { return (new DateTime_1.DateTime(value)).year; },
    ["month"]: function (value) { return (new DateTime_1.DateTime(value)).month; },
    ["minute"]: function (value) { return (new DateTime_1.DateTime(value)).timeOfDay.minute; },
    ["day"]: function (value) { return (new DateTime_1.DateTime(value)).toTimeStamp().day; },
    ["second"]: function (value) { return (new DateTime_1.DateTime(value)).timeOfDay.second; },
    ["hour"]: function (value) { return (new DateTime_1.DateTime(value)).timeOfDay.hour; },
    ["totaldays"]: function (value) { return (new DateTime_1.DateTime(value)).timeOfDay.total.days; },
    ["totalhours"]: function (value) { return (((new Date(value).getTime()) / 1000) / 60) / 60; },
    ["totalminutes"]: function (value) { return ((new Date(value).getTime()) / 1000) / 60; },
    ["totalseconds"]: function (value) {
        const totalSeconds = ((new Date(value).getTime()) / 1000);
        if (general_1.isFloatType(totalSeconds))
            return totalSeconds.toFixed(2);
        return totalSeconds;
    },
};
exports.default = legacyBuiltInProperty;
//# sourceMappingURL=legacyBuiltInProperty.js.map