﻿import { expressionPart } from "./expressionPart";
import expressionData from "../../expressionData";
import { groupDataContext } from "../../functions/groupDataContext";
import { format } from "typescript-dotnet-commonjs/System/Text/Utility";
import List from "typescript-dotnet-commonjs/System/Collections/List";
import { makeenBindingContext } from './../../bindingContext';
import { propertyInfo, getType } from "../../utils/reflectionExtender";
import { typeMapper } from "../../types/typeMapper";
import { utils } from "../utils";
import IList from "typescript-dotnet-commonjs/System/Collections/IList";
import { Type } from "typescript";
import { outputVariables } from "../../utils/outputVariables";
import { DictionaryObject, isDictionaryObject } from '../../utils/dictionaryObject';
import { EntityObject } from "../../utils/entityObject";
import { getDirectProperty } from '../../utils/general';

export class indexerPart extends expressionPart {

    get SubParts(): expressionPart[] {
        let lst: expressionPart[] = [];
        lst.push(this.indexerExpression);
        return lst;
    }

     /* override */ get simpleName(): string {
        return this.indexerExpression.simpleName;
    }

    indexerExpression!: expressionPart
    indexerProperty: propertyInfo | undefined;

    contextType!: Type;

    resolve(expressionData: expressionData, /* out */resolved: outputVariables, running: groupDataContext, evaluateAllContexts: boolean = true, setValue: Object | undefined = undefined): Object | undefined {
        if (expressionData != null && this.indexerExpression != null) {
            let indexResolved: outputVariables = new outputVariables();
            indexResolved.resolved = false;
            let index: Object = this.indexerExpression.resolve(expressionData, /* out */indexResolved, /* running:*/ running) as object;
            if (indexResolved.resolved) {
                let tempContext = expressionData.contexts.clone();
                while (!tempContext.isEmpty()) {
                    let contextObject = tempContext.pop() as Object;
                    let result: Object = this.callIndexer(contextObject, index, /* out */resolved);
                    if (resolved.resolved || !evaluateAllContexts) {
                        return result;
                    }
                }
            }
        }

        resolved.resolved = false;
        return undefined;
    }

    private callIndexer(context: any, index: Object, /* out */resolved: outputVariables, setValue: Object | undefined = undefined): Object {
        let result: Object | undefined = undefined;
        resolved.resolved = false;
        // if (typeof context === 'string' || typeof context === 'number' || typeof context === 'boolean' || typeof context === 'undefined' || context instanceof Date) {
        //     result = context;
        //     resolved.resolved = (result != null);
        //     return result as Object;
        // }
        if ((context instanceof DictionaryObject)) {
            let castedSubject: DictionaryObject = ((context) as DictionaryObject);
            const re = castedSubject[index as string];
            result = re === undefined ? null : re;
            resolved.resolved = (result !== undefined);
        }
        else if ((context instanceof EntityObject)) {
            let castedSubject: EntityObject = ((context) as EntityObject);
            let castedIndex: Object | undefined = index; //typeMapper.toType1<string>(index, getType(Object));
            if ((setValue != null)) {
                castedSubject.setItem(setValue, castedIndex as string);
                result = setValue;
            }
            result = castedSubject.getItem(castedIndex as string);
            resolved.resolved = (result !== undefined);
        }
        else if (context && context["data"] && context["data"]["customKeyValuePair"]) {
            const array = context["data"]["customKeyValuePair"];
            if (Array.isArray(array)) {
                for (const iterator of array) {
                    if (iterator.key === index) {
                        result = iterator.value;
                        resolved.resolved = (result != undefined);
                    }
                }
            }
        }
        else if (context instanceof makeenBindingContext) {
            let castedSubject: makeenBindingContext = (<makeenBindingContext>(context));
            let castedIndex: string = typeMapper.toType1<string>(index) as string;
            if ((setValue != null)) {
                castedSubject.Item(castedIndex, setValue);
            }
            else {
                result = castedSubject.Item(castedIndex);
            }

            result = setValue;
            resolved.resolved = (result != null);
        }
        else if ((context instanceof List) && context.count > 0 && context.get(0) instanceof makeenBindingContext) {
            let results: List<Object> = new List<Object>();
            for (let subContext of ((context).toArray())) {
                let castedSubject: makeenBindingContext = (<makeenBindingContext>(subContext));
                let castedIndex: string = typeMapper.toType1<string>(index) as string;
                if ((setValue != null)) {
                    castedSubject.Item(castedIndex, setValue);
                }
                else {
                    // result = castedSubject.Items?.getItem(castedIndex);
                }

                result = setValue;
                if ((result != null)) {
                    results.add(result);
                }
            }
            result = results.count > 0 ? results : undefined;

            resolved.resolved = (results.count > 0);
        }
        else {
            let intIndex: number = (typeof index == "number") ? index as number : -1;

            let isIntIndex = (typeof index == "number") ? true : (typeof Number.parseInt(index.toString()) == "number" ? true : false);
            if (this.isArrayofType(context, "Object"))// (typeof context == Object[]))
            {
                if (isIntIndex) {
                    let arr: Object[] = (<Object[]>(context));
                    if ((setValue != null)) {
                        arr[intIndex as number] = setValue;
                    }

                    result = setValue;
                    if ((arr.length > intIndex)) {
                        result = arr[intIndex as number];
                        resolved.resolved = true;
                    }
                    else {
                        result = undefined;
                        resolved.resolved = false;
                    }

                }

            }
            else if (this.isArrayofType(context, "Array")) //((context instanceof Array[]))
            {
                if (isIntIndex) {
                    let arr: string[] = (<string[]>(context));
                    if ((setValue != null)) {
                        arr[intIndex as number] = typeMapper.toType<string>(setValue) as string;
                    }

                    result = typeMapper.toType<string>(setValue as object);
                    if ((arr.length > intIndex)) {
                        result = arr[intIndex as number];
                        resolved.resolved = true;
                    }
                    else {
                        result = undefined;
                        resolved.resolved = false;
                    }

                }

            }
            else if (this.isArrayofType(context, "number"))//((context instanceof number[])) 
            {
                if (isIntIndex) {
                    let arr: number[] = (<number[]>(context));
                    if ((setValue != null)) {
                        arr[intIndex as number] = typeMapper.toType<number>(setValue) as number;
                    }

                    result = typeMapper.toType<number>(setValue as object) as number;
                    if ((arr.length > intIndex)) {
                        result = arr[intIndex as number];
                        resolved.resolved = true;
                    }
                    else {
                        result = undefined;
                        resolved.resolved = false;
                    }

                }

            }
            else if (this.isArrayofType(context, "double")) //((context instanceof double[])) 
            {
                if (isIntIndex) {
                    let arr: number[] = (<number[]>(context));
                    if ((setValue != null)) {
                        arr[intIndex as number] = typeMapper.toType<number>(setValue) as number;
                    }

                    result = typeMapper.toType<number>(setValue as number);
                    if ((arr.length > intIndex)) {
                        result = arr[intIndex as number];
                        resolved.resolved = true;
                    }
                    else {
                        result = undefined;
                        resolved.resolved = false;
                    }

                }

            }
            else if (context instanceof List && isIntIndex) {
                let arr = (<IList<Object>>(context));
                if ((setValue != null)) {
                    arr.set(intIndex as number, setValue);
                }

                result = setValue;
                if ((arr.count > intIndex)) {
                    result = arr.get(intIndex as number);
                    resolved.resolved = true;
                }
                else {
                    result = undefined;
                    resolved.resolved = false;
                }

            }
            else {
                if (this.indexerProperty == null || this.contextType != getType(context)) {
                    if (typeof context === 'object') {
                        const contextResult = getDirectProperty(context, index as string);
                        if (contextResult !== undefined) {
                            result = contextResult;
                            resolved.resolved = true;
                            return result as Object;
                        }
                    }
                    this.indexerProperty = utils.getIndexer(getType(context));
                    this.contextType = getType(context);
                }

                if ((this.indexerProperty != null)) {
                    try {
                        if ((setValue != null)) {
                            this.indexerProperty.setValue(context, [
                                typeMapper.toType(setValue, this.indexerProperty.propertyType),
                                typeMapper.toType(index, this.indexerProperty.getIndexParameters().parameterType)]);
                        }

                        let success: outputVariables = new outputVariables();
                        let typedIndex = typeMapper.toType(index, this.indexerProperty.getIndexParameters().parameterType, /* out */success) as Object;
                        if (success) {
                            let arr = [];
                            arr.push(typedIndex);
                            result = this.indexerProperty.getValue(context, arr);
                            resolved.resolved = true;
                        }
                        else {
                            resolved.resolved = false;
                            result = undefined;
                        }

                    }
                    catch (e) {
                        resolved.resolved = false;
                        result = undefined;
                    }

                }

            }

        }

        return result as Object;
    }

    toString(): string {
        return format("Item[{0}]", this.indexerExpression);
    }

    isArrayofType(test: any, type: string): boolean {
        return Array.isArray(test) && test.every((value) => typeof value === type)
    }

}
