﻿import { compoundPart } from './compoundPart';
import { format } from "typescript-dotnet-commonjs/System/Text/Utility";
import { expressionPart } from './expressionPart';
import expressionData from '../../expressionData';
import { groupDataContext } from '../../functions/groupDataContext';
import { operator } from '../../operators/operator';
import { outputVariables } from '../../utils/outputVariables';
import { DictionaryObject } from '../../utils/dictionaryObject';
import { utils } from './../utils';

export class operationPart extends compoundPart {

    get subParts(): expressionPart[] {
        let lst: expressionPart[] = [];
        lst.push(this.parameter1 as expressionPart);
        lst.push(this.parameter2 as expressionPart);

        return lst;
    }

    get simpleName(): string {
        return this.parameter2 != null ? this.parameter2.simpleName : (this.parameter1 as expressionPart).simpleName;
    }

    operator: operator | undefined;

    parameter1: expressionPart | undefined;

    parameter2: expressionPart | undefined;

    ObjectCache1: DictionaryObject | undefined = undefined;
    ObjectCache2: DictionaryObject | undefined = undefined;
    setOnCache(val: Object, param: expressionPart) {
        const strKey = utils.getKeyParamCache(param);
        if (strKey !== "")
            return new DictionaryObject().add(strKey, val);
        return undefined;
    }

    getOnCache(obj: DictionaryObject | undefined, param: expressionPart) {
        if (obj === undefined)
            return undefined;
        const strKey = utils.getKeyParamCache(param);
        if (strKey !== "")
            return obj.getValue(strKey);
        return undefined;
    }

    resolve(expressionData: expressionData, resolved: outputVariables, running: groupDataContext, evaluateAllContexts: boolean = true, setValue: Object | null = null): Object | undefined {
        if ((this.operator != null)) {
            let val1: Object | undefined = undefined;
            let val2: Object | undefined = undefined;
            const cacheValue = this.getOnCache(this.ObjectCache1, this.parameter1 as expressionPart);
            const cache2Value = this.getOnCache(this.ObjectCache2, this.parameter2 as expressionPart);

            if (cacheValue !== undefined)
                val1 = cacheValue
            else
                val1 = this.parameter1?.resolve(expressionData, /* out */resolved, /* running:*/ running, /* evaluateAllContexts:*/ evaluateAllContexts) as Object;

            if (cache2Value !== undefined)
                val2 = cache2Value
            else
                val2 = this.parameter2?.resolve(expressionData, /* out */resolved, /* running:*/ running, /* evaluateAllContexts:*/ evaluateAllContexts) as Object;

            resolved.resolved = true;
            if (this.ObjectCache1 === undefined) {
                this.ObjectCache1 = this.setOnCache(val1 as Object, this.parameter1 as expressionPart);
            }
            if (this.ObjectCache2 === undefined) {
                this.ObjectCache2 = this.setOnCache(val2 as Object, this.parameter2 as expressionPart);
            }
            return this.operator.invoke([val1 as Object, val2 as Object]);
        }

        resolved.resolved = false;
        return undefined;
    }

    addSubPart(part: expressionPart): boolean {
        if ((this.parameter2 == null)) {
            this.parameter2 = part;
            return true;
        }

        return false;
    }

    canAcceptMore(): boolean {
        return (this.parameter2 == null);
    }

    toString(): string {
        return format("{0} {1} {2}", this.parameter1, this.operator?.name, this.parameter2);
    }
}
