﻿import { expressionPart } from "./expressionPart";
import { compoundPart } from './compoundPart';
import expressionData from "../../expressionData";
import { groupDataContext } from '../../functions/groupDataContext';
import { Stack } from "../../utils/stack";
import { outputVariables } from "../../utils/outputVariables";

export class partList extends compoundPart {

    private lastResolvedLevel: number = 0;

    protected parts: expressionPart[] = [];

    get Parts(): expressionPart[] {
        return this.parts;
    }

    get simpleName(): string {
        return this.parts != null && this.parts.length > 0 ? this.parts[this.parts.length - 1].simpleName : "Item";
    }

    addSubPart(part: expressionPart): boolean {
        this.parts.push(part);
        return true;
    }

    resolve(expressionData: expressionData, resolved: outputVariables, running: groupDataContext, evaluateAllContexts: boolean = true, setValue: Object | undefined = undefined): Object | undefined {
        let contexts: Stack<Object>;
        if (evaluateAllContexts) {
            contexts = new Stack<Object>();
            let temp = expressionData.contexts.clone() as Stack<Object>;
            while (!temp.isEmpty()) {
                contexts.push(temp.pop());
            }
        }
        else {
            contexts = new Stack<Object>();
            contexts.push((expressionData.contexts as Stack<Object>).pop());
        }

        let tempData: expressionData = expressionData.toSubEx();

        if ((evaluateAllContexts
            && ((this.lastResolvedLevel > 0)
                && (this.lastResolvedLevel < contexts.count())))) {

        }

        while ((contexts.count() > 0)) {
            let currentContext: Object = contexts.pop();
            let subResolved = new outputVariables();
            let i: number = 0;
            for (; (i < this.parts.length); i++) {
                let item: expressionPart = this.parts[i];
                let popAtEnd: boolean = false;
                if (currentContext === null) {
                    resolved.resolved = true;
                    return currentContext;
                }
                if (tempData.contexts.peek() != currentContext) {
                    tempData.contexts.push(currentContext);
                    popAtEnd = true;
                }

                // parentData.monitorIfRequired(item, currentContext);
                if (i == (this.parts.length - 1)) {
                    currentContext = item.resolve(tempData, subResolved, running, false, setValue) as Object;
                    //  For observing observable collections
                    // parentData.monitorIfRequired(undefined, currentContext);
                }
                else {
                    currentContext = item.resolve(tempData, subResolved, running, false, undefined) as Object;
                    if ((currentContext === undefined)) {
                        subResolved.resolved = false;
                    }

                }

                if (popAtEnd && !(<Stack<Object>>tempData.contexts).isEmpty()) {
                    (<Stack<Object>>tempData.contexts).pop();
                }

                if (!subResolved.resolved) {
                    break;
                }

            }
            if (!subResolved.resolved && i > 0) {
                resolved.resolved = false;

                return undefined;
            }

            if (subResolved.resolved) {
                resolved.resolved = true;
                this.lastResolvedLevel = (contexts.count() + 1);
                // parentData.resultCache.setValue(cacheKey, currentContext);
                return currentContext;
            }

        }

        resolved.resolved = false;
        return undefined;
    }

    toString(): string {
        return this.parts.join(".");
    }
}
