﻿import { expressionPart } from "./expressionPart";
import expressionData from "../../expressionData";
import { groupDataContext } from "../../functions/groupDataContext";
import { utils } from "../utils";
import { Stack } from "../../utils/stack";
import { getType, propertyInfo } from "../../utils/reflectionExtender";
import { dynamicObject } from "../../utils/dynamicObject";
import { Type } from "typescript";
import { outputVariables } from "../../utils/outputVariables";
import { selection } from "../../functions/selection";
import List from "typescript-dotnet-commonjs/System/Collections/List";
import legacyBuiltInProperty from '../../utils/legacyBuiltInProperty';
import { EntityObject } from "../../utils/entityObject";
import { DictionaryObject } from "../../utils/dictionaryObject";
import { getDirectProperty, setDirectProperty } from '../../utils/general';

export class propertyPart extends expressionPart {

    propertyName: string = "";

    property!: propertyInfo;

    contextType!: Type;

    get simpleName(): string {
        return this.propertyName;
    }

    resolve(expressionDataobj: expressionData, resolved: outputVariables, running: groupDataContext, evaluateAllContexts: boolean = true, setValue: Object | undefined = undefined): Object | undefined {
        try {
            if (expressionDataobj != null) {
                let parentData: expressionData;
                parentData = expressionDataobj.parent ?? expressionDataobj;
                let tempContext: Stack<Object> = expressionDataobj.contexts.clone();

                while (!tempContext.isEmpty()) {
                    let context: Object = tempContext.pop();
                    if (context instanceof Stack) {
                        context = context.pop();
                    }

                    if (!String.isNullOrEmpty(this.propertyName)) {
                        const prop = getDirectProperty(context, this.propertyName);
                        if (prop !== undefined) {
                            resolved.resolved = true;
                            return prop;
                        }
                        if (context instanceof List || context instanceof Array) {
                            if (Object.keys(legacyBuiltInProperty).includes(this.propertyName.toLowerCase())) {
                                const innerResult = getDirectProperty(context, this.propertyName.toLowerCase());
                                if (innerResult !== undefined) {
                                    resolved.resolved = true;
                                    return innerResult;
                                }
                            }
                        }

                        if (context instanceof EntityObject) {
                            let castedSubject: EntityObject = ((context) as EntityObject);
                            const result = castedSubject.getItem(this.propertyName);
                            if (result !== undefined) {
                                resolved.resolved = (result != undefined);
                                return result;
                            }
                        }
                        else if (context instanceof DictionaryObject) {
                            const obj = context as unknown as DictionaryObject;

                            if (obj != null) {
                                if (setValue != null)
                                    obj.setValue(this.propertyName, setValue);

                                resolved.resolved = true;
                                return obj[this.propertyName];
                            }
                        }

                        if (this.property == null || this.contextType != getType(context)) {
                            this.property = utils.getProperty(getType(context), this.propertyName) as propertyInfo;
                            this.contextType = getType(context);
                        }

                        if (this.property != null) {
                            if (setValue != null)
                                this.property.setValue(context, setValue);

                            resolved.resolved = true;
                            const result = this.property.getValue(context)
                            // parentData.resultCache.setValue(cacheKey, result)
                            return result;
                        }
                        else if (context instanceof dynamicObject) {
                            let obj = (context as dynamicObject);
                            const memberNames = obj.getDynamicMemberNames();

                            if (obj != null) {
                                if (setValue != null) {
                                    resolved.resolved = true;
                                    setDirectProperty(obj, this.propertyName, setValue);
                                }
                                else if (memberNames != null && memberNames.contains(this.propertyName)) {
                                    resolved.resolved = true;
                                    let memberObj = obj.getItem(this.propertyName);
                                    // parentData.resultCache.setValue(cacheKey, memberObj);
                                    return memberObj;
                                }
                            }
                        }
                        else if (context instanceof selection) {
                            for (let sel of context.toArray()) {
                                let obj = (sel as dynamicObject);
                                const memberNames = obj.getDynamicMemberNames();

                                if (obj != null) {
                                    if (setValue != null) {
                                        resolved.resolved = true;
                                        setDirectProperty(obj, this.propertyName, setValue);
                                    }
                                    else if (memberNames != null && memberNames.contains(this.propertyName)) {
                                        resolved.resolved = true;
                                        let memberObj = obj.getItem(this.propertyName);
                                        if (memberObj != undefined) {
                                            // parentData.resultCache.setValue(cacheKey, memberObj);
                                            return memberObj;
                                        }
                                    }

                                }
                            }
                        }
                    }

                    if (!evaluateAllContexts)
                        break;
                }
            }
        }
        catch (ex) {

            throw ex;
        }

        resolved.resolved = false;
        return undefined;
    }

    toString(): string {
        return this.propertyName;
    }
}
