﻿import { TimeSpan } from 'typescript-dotnet-commonjs/System/Time/TimeSpan';
import { DateTime } from 'typescript-dotnet-commonjs/System/Time/DateTime';
import List from "typescript-dotnet-commonjs/System/Collections/List";
import StringBuilder from "typescript-dotnet-commonjs/System/Text/StringBuilder";
import { switchPartData } from "./parts/switchPartData";
import { expressionCompiler, ICompileFlags, compileFlags } from "./expressionCompiler";
import { operationPart } from "./parts/operationPart";
import { indicatorPart } from "./parts/indicatorPart";
import { nullPart } from "./parts/nullPart";
import { badExpressionException } from "../error/badExpressionException";
import { unterminatedEnclosureException } from "../error/unterminatedEnclosureException";
import { unterminatedStringLiteralException } from "../error/unterminatedStringLiteralException";
import { Type } from "typescript";
import Types from "typescript-dotnet-commonjs/System/Types";
import { propertyInfo, getRuntimeProperties, getType } from "../utils/reflectionExtender";
import { typeMapper } from "../types/typeMapper";
import { expressionPart } from "./parts/expressionPart";
import { outputVariables } from "../utils/outputVariables";
import legacyBuiltInProperty from '../utils/legacyBuiltInProperty';
import { DictionaryObject } from '../utils/dictionaryObject';
import { partList } from './parts/partList';
import { propertyPart } from './parts/propertyPart';
import { indexerPart } from './parts/indexerPart';
import { literalPart } from './parts/literalPart';
import { isFloatType } from '../utils/general';

export class utils {

    static IsNumericType(v: Object): boolean {
        return Types.isNumber(v);
    }

    static isDecimalValue(val: number): boolean {
        if (val % 1 != 0) {
            return true;
        }
        return false;
    }
    static supportsOperators(v: Object): boolean {
        return (utils.IsNumericType(v) || v instanceof DateTime || v instanceof TimeSpan || v === undefined || v === null);
    }

    static toBiggerType(v1: Object, v2: Object, target: Object): Object {
        if ((v1 instanceof Number || v2 instanceof Number) && !this.isDecimalValue(target as number)) {
            return typeMapper.toType<number>(target) as Object;
        }
        else if ((v1 instanceof Number || v2 instanceof Number) && this.isDecimalValue(target as number)) {

            return typeMapper.toType<number>(target) as Object;
        }
        // else if ((v1 instanceof ((long || v2) instanceof long))) {
        //     return typeMapper.toType<number>(target);
        // }
        // else if ((v1 instanceof ((int || v2) instanceof int))) {
        //     return typeMapper.toType<number>(target);
        // }
        // else if ((v1 instanceof ((short || v2) instanceof short))) {
        //     return typeMapper.toType<number>(target);
        // }
        // else if ((v1 instanceof ((sbyte || v2) instanceof sbyte))) {
        //     return typeMapper.toType<System.SByte>(target);
        // }

        // let ||: number;
        // let Unknown: number;
        // return typeMapper.toType<number>(target);
        // if ((v1 instanceof ((uint || v2) instanceof uint))) {
        //     return typeMapper.toType<number>(target);
        // }
        // else if ((v1 instanceof ((ushort || v2) instanceof ushort))) {
        //     return typeMapper.toType<number>(target);
        // }
        // else if ((v1 instanceof ((byte || v2) instanceof byte))) {
        //     return typeMapper.toType<number>(target);
        // }

        return target;
    }

    static CheckUndefined(v1: any, v2: any) {
        if (v2 != undefined)
            return v2;
        if (v1 != undefined)
            return v1;
        return null;
    }
    static toBiggerTypeOperation(v1: Object, v2: Object, op: string): Object | null {
        if ((typeof v1 == 'number' || typeof v2 == 'number')) {
            switch (op) {
                case '+':
                    if (v1 == undefined || v2 == undefined) {
                        return utils.CheckUndefined(v1, v2);
                    }
                    return ((v1) as number + ((v2) as number));
                case '-':
                    if (v1 == undefined || v2 == undefined) {
                        return utils.CheckUndefined(v1, v2);
                    }
                    return ((v1) as number - ((v2) as number));
                case '*':
                    return ((v1) as number * ((v2) as number));
                case '/':
                    /// Please feel free to remove the isFloat type check 
                    /// whenever V2 complete rollout and V1 would discard
                    if (!isFloatType(v1 as number) && !isFloatType(v2 as number))
                        return Math.floor(((v1) as number / ((v2) as number)));

                    return ((v1) as number / ((v2) as number));
                    break;
                case '%':
                    return ((v1) as number % ((v2) as number));
                    break;
                case '^':
                    return ((v1) as number ^ ((v2) as number));
                    break;
                case '=':
                    return ((v1) == (v2));
                    break;
                case '!':
                    return ((v1) != (v2));
                    break;
                case '>':
                    let result = ((v1) as number > ((v2) as number));
                    return result;
                    break;
                case '<':
                    return (((v1) as number) < ((v2) as number));
                    break;
            }

        }
        // else if ((v1 instanceof ((short || v2) instanceof short))) {
        //     switch (op) {
        //         case '+':
        //             return (typeMapper.toType<number>(v1) + typeMapper.toType<number>(v2));
        //             break;
        //         case '-':
        //             return (typeMapper.toType<number>(v1) - typeMapper.toType<number>(v2));
        //             break;
        //         case '*':
        //             return (typeMapper.toType<number>(v1) * typeMapper.toType<number>(v2));
        //             break;
        //         case '/':
        //             return (typeMapper.toType<number>(v1) / typeMapper.toType<number>(v2));
        //             break;
        //         case '%':
        //             return (typeMapper.toType<number>(v1) % typeMapper.toType<number>(v2));
        //             break;
        //         case '^':
        //             return (typeMapper.toType<number>(v1) | typeMapper.toType<number>(v2));
        //             break;
        //         case '=':
        //             return (typeMapper.toType<number>(v1) == typeMapper.toType<number>(v2));
        //             break;
        //         case '!':
        //             return (typeMapper.toType<number>(v1) != typeMapper.toType<number>(v2));
        //             break;
        //         case '>':
        //             return (typeMapper.toType<number>(v1) > typeMapper.toType<number>(v2));
        //             break;
        //         case '<':
        //             return (typeMapper.toType<number>(v1) < typeMapper.toType<number>(v2));
        //             break;
        //     }

        // }
        // else if ((v1 instanceof ((sbyte || v2) instanceof sbyte))) {
        //     switch (op) {
        //         case '+':
        //             return (typeMapper.toType<System.SByte>(v1) + typeMapper.toType<System.SByte>(v2));
        //             break;
        //         case '-':
        //             return (typeMapper.toType<System.SByte>(v1) - typeMapper.toType<System.SByte>(v2));
        //             break;
        //         case '*':
        //             return (typeMapper.toType<System.SByte>(v1) * typeMapper.toType<System.SByte>(v2));
        //             break;
        //         case '/':
        //             return (typeMapper.toType<System.SByte>(v1) / typeMapper.toType<System.SByte>(v2));
        //             break;
        //         case '%':
        //             return (typeMapper.toType<System.SByte>(v1) % typeMapper.toType<System.SByte>(v2));
        //             break;
        //         case '^':
        //             return (typeMapper.toType<System.SByte>(v1) | typeMapper.toType<System.SByte>(v2));
        //             break;
        //         case '=':
        //             return (typeMapper.toType<System.SByte>(v1) == typeMapper.toType<System.SByte>(v2));
        //             break;
        //         case '!':
        //             return (typeMapper.toType<System.SByte>(v1) != typeMapper.toType<System.SByte>(v2));
        //             break;
        //         case '>':
        //             return (typeMapper.toType<System.SByte>(v1) > typeMapper.toType<System.SByte>(v2));
        //             break;
        //         case '<':
        //             return (typeMapper.toType<System.SByte>(v1) < typeMapper.toType<System.SByte>(v2));
        //             break;
        //     }

        // }

        // The operator should be an XOR ^ instead of an OR, but not available in CodeDOM
        // The operator should be an XOR ^ instead of an OR, but not available in CodeDOM
        // The operator should be an XOR ^ instead of an OR, but not available in CodeDOM
        // The operator should be an XOR ^ instead of an OR, but not available in CodeDOM
        // The operator should be an XOR ^ instead of an OR, but not available in CodeDOM
        // The operator should be an XOR ^ instead of an OR, but not available in CodeDOM
        // The operator should be an XOR ^ instead of an OR, but not available in CodeDOM
        // let ||: number;
        // let Unknown: number;
        switch (op) {
            // case '+':
            //     return (typeMapper.toType<number>(v1) + typeMapper.toType<number>(v2));
            //     break;
            // case '-':
            //     return (typeMapper.toType<number>(v1) - typeMapper.toType<number>(v2));
            //     break;
            // case '*':
            //     return (typeMapper.toType<number>(v1) * typeMapper.toType<number>(v2));
            //     break;
            // case '/':
            //     return (typeMapper.toType<number>(v1) / typeMapper.toType<number>(v2));
            //     break;
            // case '%':
            //     return (typeMapper.toType<number>(v1) % typeMapper.toType<number>(v2));
            //     break;
            // case '^':
            //     return (typeMapper.toType<number>(v1) | typeMapper.toType<number>(v2));
            // break;
            case '=':
                return (typeMapper.toType<number>(v1) == typeMapper.toType<number>(v2));
                break;
            case '!':
                return (typeMapper.toType<number>(v1) != typeMapper.toType<number>(v2));
                break;
            case '>':
                return (<Object>typeMapper.toType<number>(v1) > <Object>typeMapper.toType<number>(v2));
                break;
            case '<':
                return (<Object>typeMapper.toType<number>(v1) < <Object>typeMapper.toType<number>(v2));
                break;
        }

        // The operator should be an XOR ^ instead of an OR, but not available in CodeDOM
        if ((v1 instanceof Number)) {
            switch (op) {
                // case '+':
                //     return (typeMapper.toType<number>(v1) + typeMapper.toType<number>(v2));
                //     break;
                // case '-':
                //     return (typeMapper.toType<number>(v1) - typeMapper.toType<number>(v2));
                //     break;
                // case '*':
                //     return (typeMapper.toType<number>(v1) * typeMapper.toType<number>(v2));
                //     break;
                // case '/':
                //     return (typeMapper.toType<number>(v1) / typeMapper.toType<number>(v2));
                //     break;
                // case '%':
                //     return (typeMapper.toType<number>(v1) % typeMapper.toType<number>(v2));
                //     break;
                // case '^':
                //     return (typeMapper.toType<number>(v1) | typeMapper.toType<number>(v2));
                //     break;
                case '=':
                    return (typeMapper.toType<number>(v1) == typeMapper.toType<number>(v2));
                    break;
                case '!':
                    return (typeMapper.toType<number>(v1) != typeMapper.toType<number>(v2));
                    break;
                case '>':
                    return (<Object>typeMapper.toType<number>(v1) > <Object>typeMapper.toType<number>(v2));
                    break;
                case '<':
                    return (<Object>typeMapper.toType<number>(v1) < <Object>typeMapper.toType<number>(v2));
                    break;
            }

        }
        // else if ((v1 instanceof ((ushort || v2) instanceof ushort))) {
        //     switch (op) {
        //         case '+':
        //             return (typeMapper.toType<number>(v1) + typeMapper.toType<number>(v2));
        //             break;
        //         case '-':
        //             return (typeMapper.toType<number>(v1) - typeMapper.toType<number>(v2));
        //             break;
        //         case '*':
        //             return (typeMapper.toType<number>(v1) * typeMapper.toType<number>(v2));
        //             break;
        //         case '/':
        //             return (typeMapper.toType<number>(v1) / typeMapper.toType<number>(v2));
        //             break;
        //         case '%':
        //             return (typeMapper.toType<number>(v1) % typeMapper.toType<number>(v2));
        //             break;
        //         case '^':
        //             return (typeMapper.toType<number>(v1) | typeMapper.toType<number>(v2));
        //             break;
        //         case '=':
        //             return (typeMapper.toType<number>(v1) == typeMapper.toType<number>(v2));
        //             break;
        //         case '!':
        //             return (typeMapper.toType<number>(v1) != typeMapper.toType<number>(v2));
        //             break;
        //         case '>':
        //             return (typeMapper.toType<number>(v1) > typeMapper.toType<number>(v2));
        //             break;
        //         case '<':
        //             return (typeMapper.toType<number>(v1) < typeMapper.toType<number>(v2));
        //             break;
        //     }

        // }
        // else if ((v1 instanceof ((byte || v2) instanceof byte))) {
        //     switch (op) {
        //         case '+':
        //             return (typeMapper.toType<number>(v1) + typeMapper.toType<number>(v2));
        //             break;
        //         case '-':
        //             return (typeMapper.toType<number>(v1) - typeMapper.toType<number>(v2));
        //             break;
        //         case '*':
        //             return (typeMapper.toType<number>(v1) * typeMapper.toType<number>(v2));
        //             break;
        //         case '/':
        //             return (typeMapper.toType<number>(v1) / typeMapper.toType<number>(v2));
        //             break;
        //         case '%':
        //             return (typeMapper.toType<number>(v1) % typeMapper.toType<number>(v2));
        //             break;
        //         case '^':
        //             return (typeMapper.toType<number>(v1) | typeMapper.toType<number>(v2));
        //             break;
        //         case '=':
        //             return (typeMapper.toType<number>(v1) == typeMapper.toType<number>(v2));
        //             break;
        //         case '!':
        //             return (typeMapper.toType<number>(v1) != typeMapper.toType<number>(v2));
        //             break;
        //         case '>':
        //             return (typeMapper.toType<number>(v1) > typeMapper.toType<number>(v2));
        //             break;
        //         case '<':
        //             return (typeMapper.toType<number>(v1) < typeMapper.toType<number>(v2));
        //             break;
        //     }

        // }
        else if (v1 instanceof DateTime && v2 instanceof DateTime) {
            switch (op) {
                case '+':
                    return <Object>typeMapper.toType<DateTime>(v1);
                    break;
                    // case '-':
                    //     return (typeMapper.toType<DateTime>(v1) - typeMapper.toType<DateTime>(v2));
                    break;
                case '*':
                    return <Object>typeMapper.toType<DateTime>(v1);
                    break;
                case '/':
                    return <Object>typeMapper.toType<DateTime>(v1);
                    break;
                case '%':
                    return <Object>typeMapper.toType<DateTime>(v1);
                    break;
                case '^':
                    return <Object>typeMapper.toType<DateTime>(v1);
                    break;
                case '=':
                    return (typeMapper.toType<DateTime>(v1) == typeMapper.toType<DateTime>(v2));
                    break;
                case '!':
                    return (typeMapper.toType<DateTime>(v1) != typeMapper.toType<DateTime>(v2));
                    break;
                case '>':
                    return (<Object>typeMapper.toType<DateTime>(v1) > <Object>typeMapper.toType<DateTime>(v2));
                    break;
                case '<':
                    return (<Object>typeMapper.toType<DateTime>(v1) < <Object>typeMapper.toType<DateTime>(v2));
                    break;
            }

        }
        else if (v1 instanceof TimeSpan && v2 instanceof TimeSpan) {
            switch (op) {
                // case '+':
                //     return (typeMapper.toType<TimeSpan>(v1) + typeMapper.toType<TimeSpan>(v2));
                //     break;
                // case '-':
                //     return (typeMapper.toType<TimeSpan>(v1) - typeMapper.toType<TimeSpan>(v2));
                //     break;
                case '*':
                    return <Object>typeMapper.toType<TimeSpan>(v1);
                    break;
                case '/':
                    return <Object>typeMapper.toType<TimeSpan>(v1);
                    break;
                case '%':
                    return <Object>typeMapper.toType<TimeSpan>(v1);
                    break;
                case '^':
                    return <Object>typeMapper.toType<TimeSpan>(v1);
                    break;
                case '=':
                    return (typeMapper.toType<TimeSpan>(v1) == typeMapper.toType<TimeSpan>(v2));
                    break;
                case '!':
                    return (typeMapper.toType<TimeSpan>(v1) != typeMapper.toType<TimeSpan>(v2));
                    break;
                case '>':
                    return (<Object>typeMapper.toType<TimeSpan>(v1) > <Object>typeMapper.toType<TimeSpan>(v2));
                    break;
                case '<':
                    return (<Object>typeMapper.toType<TimeSpan>(v1) < <Object>typeMapper.toType<TimeSpan>(v2));
                    break;
            }

        }
        else if (v1 instanceof DateTime && v2 instanceof TimeSpan) {
            switch (op) {
                // case '+':
                //     return (typeMapper.toType<DateTime>(v1) + typeMapper.toType<TimeSpan>(v2));
                //     break;
                // case '-':
                //     return (typeMapper.toType<DateTime>(v1) - typeMapper.toType<TimeSpan>(v2));
                //     break;
                case '*':
                    return <Object>typeMapper.toType<DateTime>(v1);
                    break;
                case '/':
                    return <Object>typeMapper.toType<DateTime>(v1);
                    break;
                case '%':
                    return <Object>typeMapper.toType<DateTime>(v1);
                    break;
                case '^':
                    return <Object>typeMapper.toType<DateTime>(v1);
                    break;
                case '=':
                    return false;
                    break;
                case '!':
                    return true;
                    break;
                case '>':
                    return true;
                    break;
                case '<':
                    return false;
                    break;
            }

        }
        else if (v1 instanceof TimeSpan && v2 instanceof DateTime) {
            switch (op) {
                // case '+':
                //     return (typeMapper.toType<DateTime>(v2) + typeMapper.toType<TimeSpan>(v1));
                //     break;
                // case '-':
                //     return (typeMapper.toType<DateTime>(v2) - typeMapper.toType<TimeSpan>(v1));
                //     break;
                case '*':
                    return <Object>typeMapper.toType<DateTime>(v2);
                    break;
                case '/':
                    return <Object>typeMapper.toType<DateTime>(v2);
                    break;
                case '%':
                    return <Object>typeMapper.toType<DateTime>(v2);
                    break;
                case '^':
                    return <Object>typeMapper.toType<DateTime>(v2);
                    break;
                case '=':
                    return false;
                    break;
                case '!':
                    return true;
                    break;
                case '>':
                    return false;
                    break;
                case '<':
                    return true;
                    break;
            }

        }

        // The operator should be an XOR ^ instead of an OR, but not available in CodeDOM
        // The operator should be an XOR ^ instead of an OR, but not available in CodeDOM
        // The operator should be an XOR ^ instead of an OR, but not available in CodeDOM
        return null;
    }

    static markEnclosure(path: string, startIndex: number, startChar: string, endChar: string, /* out */endIndex: outputVariables) {
        this.splitEnclosure(path, startIndex, startChar, endChar, /* out */endIndex);
    }

    static splitEnclosure(path: string, startIndex: number, startChar: string, endChar: string, /* out */endIndex: outputVariables, delimiter: string = "0"): List<string> {
        let tokens: List<string> | undefined = new List<string>();
        tokens = delimiter != "0" ? new List<string>() : undefined;

        let level: number = 1;
        let curlyLevel: number = 0;
        let squareLevel: number = 0;
        let roundLevel: number = 0;
        let inString: number = 0;
        let inChar: number = 0;
        let sb: StringBuilder = new StringBuilder();
        let i: number = (startIndex + 1);
        for (; (i < path.length); i++) {
            let c: string = path[i];
            let otherLevels: number = curlyLevel + squareLevel + roundLevel + inString + inChar;;
            // We do not ignore the whitespaces here, they should be ignored by the outer expression compiler code or the function param parsing code
            // if (char.IsWhiteSpace(c) && inString == 0 && inChar == 0)
            // {
            //     continue;
            // }
            // else
            if ((c == startChar)) {
                level++;
            }
            else if ((c == endChar)) {
                if ((level > 0)) {
                    level--;
                    if ((level == 0)) {
                        break;
                    }

                }
                else {
                    throw new unterminatedEnclosureException(startChar, endChar);
                }

            }
            else if ((c == "\"")) {
                inString = inString == 0 ? 1 : 0;
            }
            else if ((c == "\'")) {
                inChar = inChar == 0 ? 1 : 0;
            }
            else if ((c == "(")) {
                roundLevel++;
            }
            else if ((c == ")")) {
                if ((roundLevel > 0)) {
                    roundLevel--;
                }
                else {
                    throw new unterminatedEnclosureException('(', ')');
                }

            }
            else if ((c == "{")) {
                curlyLevel++;
            }
            else if ((c == "}")) {
                if ((curlyLevel > 0)) {
                    curlyLevel--;
                }
                else {
                    throw new unterminatedEnclosureException('{', '}');
                }

            }
            else if ((c == "[")) {
                squareLevel++;
            }
            else if ((c == "]")) {
                if ((squareLevel > 0)) {
                    squareLevel--;
                }
                else {
                    throw new unterminatedEnclosureException('[', ']');
                }

            }

            if (delimiter != "0" && level == 1 && otherLevels == 0 && c == delimiter) {
                tokens?.add(sb.toString());
                sb.clear();
            }
            else {
                sb.append(c);
            }

        }

        if ((inString > 0)) {
            throw new unterminatedStringLiteralException();
        }

        if (((delimiter != "0")
            && (sb.toString().length > 0))) {
            tokens?.add(sb.toString().trim());
        }

        if ((level > 0)) {
            throw new unterminatedEnclosureException(startChar, endChar);
        }
        else {
            endIndex.endIndex = i;
        }

        return tokens as unknown as List<string>;
    }

    static buildSwitchCase(path: string, startIndex: number, /* out */endIndex: outputVariables, flags: compileFlags): switchPartData {
        return this.buildSwitchCaseOverload(this.splitEnclosure(path, startIndex, "{", "}", /* out */endIndex, ","), flags);
    }

    static buildSwitchCaseOverload(caseList: List<string>, flags: compileFlags): switchPartData {
        let data: switchPartData = new switchPartData();
        let foundDefaultPart: boolean = false;
        let foundNullPart: boolean = false;
        for (let caseString of caseList.toArray()) {
            let expression = expressionCompiler.compile(caseString, undefined, undefined, flags);
            if ((expression instanceof operationPart && ((<operationPart>(expression)).operator?.name == "="))) {
                let op = (<operationPart>(expression));
                if ((op.parameter1 instanceof indicatorPart && (<indicatorPart>(op.parameter1)).value.equals("default"))) {
                    foundDefaultPart = true;
                    data.DefaultValue = op.parameter2 as expressionPart;
                }
                else if ((op.parameter1 instanceof nullPart)) {
                    if (foundNullPart) {
                        throw new badExpressionException("Switch can have only one null value case");
                    }

                    data.NullValue = op.parameter2 as expressionPart;
                    foundNullPart = true;
                }
                else {
                    data.Cases.push({ key: op.parameter1 as expressionPart, value: op.parameter2 as expressionPart });
                }

            }
            else {
                if (foundDefaultPart) {
                    throw new badExpressionException("Switch can have only one default case");
                }

                data.DefaultValue = expression as expressionPart;
                foundDefaultPart = true;
            }

        }

        return data;
    }

    static getIndexer(type: Type): propertyInfo | undefined {
        let indexerProperty: propertyInfo | undefined = undefined;
        this.getProperties(type).forEach((prop) => {
            if ((prop.name == "Items")) {
                indexerProperty = prop;
            }
        });
        return indexerProperty;
    }

    static getProperties(type: Type): propertyInfo[] {
        return this.getPropertiesMap(type).getValues();
    }

    static getPropertiesMap(type: Type): DictionaryObject {
        let infos: DictionaryObject = new DictionaryObject();
        getRuntimeProperties(type).forEach((pi) => {
            infos.setValue(pi.name, pi);
        });
        return infos;
    }

    static getProperty(type: Type, name: string): propertyInfo | null {
        let props: DictionaryObject = this.getPropertiesMap(type);
        try {
            return props.containsKey(name) ? props.getValue(name) as propertyInfo : this.getBuilInProperty(name, type);
        }
        catch (e) {
            return this.getBuilInProperty(name, type);
        }
    }
    static getKeyParamCache(param: object) {
        let strKey: string = "";
        if (param instanceof partList) {
            for (let index = 0; index < param.Parts.length; index++) {
                const propPart = param.Parts[index];
                if (propPart instanceof propertyPart) {
                    const propName = (propPart as propertyPart).propertyName;
                    if (propName === "Controls" || propName === 'Login' || propName === 'Var' || propName === 'In') {
                        strKey += propName;
                    }
                }
                if (strKey !== "" && propPart instanceof indexerPart) {
                    if (((propPart as indexerPart).indexerExpression as literalPart)) {
                        strKey += ((propPart as indexerPart).indexerExpression as literalPart).value;
                    }

                }
            }
        }
        return strKey;
    }
    static getBuilInProperty(name: string, obj: Object): propertyInfo {
        let prop = new propertyInfo();
        const pr = (legacyBuiltInProperty as any)[name.toLowerCase()];
        prop.name = name.toLowerCase();
        prop.parameterType = pr;
        prop.type = obj as any;
        return prop;
    }

    static matchDates(d1: DateTime, d2: DateTime, matchYear: boolean): boolean {
        if (d1 instanceof DateTime && d2 instanceof DateTime)
            return matchYear ? d1.date.equals(d2.date) : d1.day == d2.day && d1.month == d2.month;
        else if (typeof d1 == "string") {
            try {
                let d = new DateTime(d1)
                return d.date.equals(d2.date);
            }
            catch (e) {
                return false;
            }
        }
        return true;
    }

    static compare(v1: Object, v2: Object, op: string): boolean {
        if (((v1 == v2)
            || ((v1 != null)
                && v1 === v2))) {
            return (op == '=');
        }
        // else if (((v1 != null && v2 != null&& v1 instanceof IComparable && v1.GetType().GetTypeInfo().IsAssignableFrom(v2.GetType().GetTypeInfo())))) {
        //     let compareResult: number = (<IComparable>(v1)).CompareTo(v2);
        //     switch (op) {
        //         case '=':
        //             return (compareResult == 0);
        //             break;
        //         case '!':
        //             return (compareResult != 0);
        //             break;
        //         case '>':
        //             return (compareResult > 0);
        //             break;
        //         case '<':
        //             return (compareResult < 0);
        //             break;
        //     }

        // }
        else if (((v1 != null)
            && ((v2 != null)
                && (this.supportsOperators(v1) && this.supportsOperators(v2))))) {
            let result: Object = this.toBiggerTypeOperation(v1, v2, op) as Object;
            if ((result != null)) {
                return (result) as boolean;
            }

        }

        return (op == '!');
        //  throw new InvalidOperationException(string.Format("Could not compare {0} with {1} for operator {2}", v1 != null ? v1.GetType().ToString() : "null", v2 != null ? v2.GetType().ToString() : "null", op));
    }
}
