﻿import { Stack } from "./utils/stack";
import List from 'typescript-dotnet-commonjs/System/Collections/List';
import { makeenBindingContext } from './bindingContext';
import { expressionCompiler } from "./compiler/expressionCompiler";
// import { getHashCode } from "typescript-dotnet-commonjs/System/Text/Utility";
import './utils/stringUtil';

export default class expressionData {

    private pageContext!: makeenBindingContext;

    private globalContext!: makeenBindingContext;

    parent!: expressionData;

    private subExpressions: List<expressionData> = new List<expressionData>();

    expression!: string

    contexts: Stack<Object> = new Stack<object>();

    convertType: boolean = false;

    monitorExpression: boolean = false;
    cacheEnabled: boolean = false;

    legacyMode: boolean = false;

    isAlreadyLegacy: Boolean = false;

    dtFormat: string | undefined = undefined;

    constructor(expression: string, pageContext?: Object, globalContext?: Object, legacyMode: boolean = true, convertType: boolean = true) {
        this.constructorInit(expression, pageContext, globalContext, legacyMode, convertType, false)
        this.cacheEnabled = true;
    }

    private constructorInit(expression: string, pageContext?: Object, globalContext?: Object, legacyMode?: boolean, convertType: boolean = true, isAlreadyLegacy: boolean = false) {
        this.expression = expression;
        this.convertType = convertType;
        this.globalContext = this.globalContext;
        this.pageContext = this.pageContext;
        this.isAlreadyLegacy = isAlreadyLegacy;
        this.legacyMode = legacyMode as boolean;
        this.cacheEnabled = true;
        this.contexts = new Stack<Object>();
        if ((this.globalContext != null)) {
            this.contexts.push(this.globalContext);
        }

        if ((this.pageContext != null)) {
            this.contexts.push(this.pageContext);
        }

        this.toLegacyIfNeeded();
    }

    private privateConstructor(expression: string, contexts: Stack<Object>, pageContext: makeenBindingContext, globalContext: makeenBindingContext, legacyMode: boolean, convertType: boolean = true, isAlreadyLegacy: Boolean = false) {
        this.expression = expression;

        this.convertType = convertType;
        this.globalContext = globalContext;
        this.pageContext = pageContext;
        this.contexts = contexts.clone();

        this.isAlreadyLegacy = isAlreadyLegacy;
        this.legacyMode = legacyMode;
        this.cacheEnabled = true;
        this.toLegacyIfNeeded();
        return this;
    }

    toSubEx(currentContext: Object | null = null): expressionData {
        return this.toSubExInit(this.expression, currentContext);
    }

    toSubExInit(newExpression: string, currentContext: Object | null = null): expressionData {
        let subExpressionData: expressionData = this.privateConstructor(newExpression, this.contexts, this.pageContext, this.globalContext, this.legacyMode, this.convertType, this.isAlreadyLegacy);
        if (((currentContext != null)
            && ((currentContext != this.pageContext)
                && ((currentContext != this.globalContext)
                    && !subExpressionData.contexts.contains(currentContext))))) {
            subExpressionData.contexts.push(currentContext);
        }

        if (this.parent != null) {
            subExpressionData.parent = this.parent;
        }
        else {
            subExpressionData = this;
        }
        subExpressionData.cacheEnabled = this.cacheEnabled;
        this.subExpressions.add(subExpressionData);
        return subExpressionData;
    }

    /* override */ equals(obj: Object): boolean {
        if ((obj instanceof expressionData)) {
            let ed: expressionData = (<expressionData>(obj));
            return (this.expression == ed.expression);
        }

        return false;
    }

    // getHashCode(): number {
    //     let hash: number = 13;
    //     //  hash = (hash * 7) + (this.Binding != null ? this.Binding.GetHashCode() : 0);
    //     hash = ((hash * 7) + getHashCode(this.expression));
    //     return hash;
    // }

    private toLegacyIfNeeded() {
        if ((!this.isAlreadyLegacy
            && this.legacyMode)) {
            this.isAlreadyLegacy = true;
            this.expression = expressionCompiler.ProcessLegacy(this.expression);
        }

    }
    toString(): string {
        return this.expression;
    }
}

