﻿import { expressionPart } from './compiler/parts/expressionPart';
import expressionData from './expressionData';
import { expressionCompiler, compileFlags, ICompileFlags } from './compiler/expressionCompiler';
import List from 'typescript-dotnet-commonjs/System/Collections/List';
import { getType } from './utils/reflectionExtender';
import { Type } from 'typescript';
import { outputVariables } from './utils/outputVariables';
import { ItemSourceContexts } from './utils/ItemSourceContexts';

export class expressionEngine {

    static instance: expressionEngine;

    private static DEFAULT_LEGACY_MODE: boolean = true;

    static getInstance(): expressionEngine {
        if ((this.instance == null)) {
            this.instance = new expressionEngine();
        }

        return this.instance;
    }

    constructor() {

    }

    static evalGeneric<T>(context: Object | undefined, expression: string, secondContext?: Object, globalContext?: Object, legacyMode: boolean = false, data?: expressionData, setValue: Object | undefined = undefined): any {
        let tType: Type | undefined;
        if (!expression) {
            return undefined;
        }

        if (tType == undefined)
            tType = getType(context);
        if (data != undefined) {
            return expressionEngine.eval(context, "", legacyMode, data, tType, setValue) as T;
        }
        let result = expressionEngine.evalPrivate(context, expression, secondContext, globalContext, tType, legacyMode);

        return (result as T);
    }

    static eval(context: Object | undefined, expression: string, legacyMode: boolean = true, data?: expressionData, targetType?: Type, setValue: Object | undefined = undefined): Object {
        if (data == undefined) {
            return expressionEngine.evalPrivate(context, expression, undefined, undefined, getType(Object), legacyMode, null);
        }
        else {
            if (context != null && !data.contexts.contains(context))
                data.contexts.push(context);
            return expressionEngine.valueFor(context, data, targetType as Type, setValue) as Object;
        }
    }

    private static evalPrivate(context: Object | undefined, expression: string, secondContext?: Object, globalContext?: Object, targetType?: Type, legacyMode: boolean = true, setValue: Object | null = null): Object {
        let data: expressionData = new expressionData(expression, secondContext, globalContext, legacyMode);
        if ((context != null)) {
            data.contexts.push(context);
        }
        if (secondContext != null) {
            data.contexts.push(secondContext);
        }
        if (globalContext != null) {
            data.contexts.push(globalContext);
        }
        let result: Object | undefined = undefined;
        result = expressionEngine.eval(context, "", legacyMode, data, undefined, targetType);
        return result;
    }

    static valueFor(subject: Object | undefined, data: expressionData, targetType: Type, setValue: Object | undefined): Object | null {
        try {
            let expression: expressionPart | undefined;
            let compileflag = new compileFlags();
            compileflag.legacyMode = data.legacyMode;
            expression = expressionCompiler.compile(data.expression, undefined, undefined, compileflag);
            let resolved: outputVariables = new outputVariables();
            resolved.resolved = true;
            if (data.contexts.peek() instanceof ItemSourceContexts) {
                let secondExpressionPart: expressionPart | undefined = undefined;
                const itemSourceContext = data.contexts.pop() as ItemSourceContexts;
                if (itemSourceContext.secondExpression) {
                    secondExpressionPart = expressionCompiler.compile(itemSourceContext.secondExpression, undefined, undefined, compileflag);
                }
                let results = [];
                let contexts = itemSourceContext.contexts instanceof List ? itemSourceContext.contexts.toArray() : itemSourceContext.contexts;
                for (const item of contexts) {
                    data.contexts.push(item);
                    let result: Object = expression?.resolveClass(data, resolved, targetType) as Object;
                    let result2: Object = secondExpressionPart?.resolveClass(data, resolved, targetType) as Object;

                    results.push({
                        result: result,
                        result2: result2,
                        context: item
                    });
                    data.contexts.pop();
                }
                // if (results.length !== 0)
                return results;
            }
            let result: Object = expression?.resolveClass(data, resolved, targetType) as Object;
            return result;
        }
        catch (e) {
            throw e;
        }

    }
}