﻿import { aggregateFunction } from './aggregateFunction'
import { find, findAll, limit, exists } from "./globals/find";
import { groupBy } from "./globals/groupBy";
import { select } from "./globals/select";
import { matches, substring, replace, split, concat, concatNonEmpty, toString, toLower, toUpper, regexpReplace, contains, stringformat } from "./globals/string";
import { coalesce, toNumber, toObjectList, toDate, toList, toDictionary, toObject } from "./globals/literals";
import { math_Abs, math_Acos, math_Min, math_Pow, math_Round, math_Sqrt, math_Pi, math_Sum, math_Tan, math_Cosh, math_Tanh, math_Exp, math_Sin, math_Asin, math_Atan, math_Atan2, math_Cos, math_Log, math_Max } from "./globals/math";
import { Bool, eval1, percent } from "./globals/evals";
import { sum } from "./aggregates/sum";
import { max } from "./aggregates/max";
import { min } from "./aggregates/min";
import { last } from "./aggregates/last";
import { count } from "./aggregates/count";
import { groupConcat } from "./aggregates/groupConcat";
import { first } from "./aggregates/first";
import { toUtc, addPeriod, now, utcNow, isToday, isDate, toLocalTime } from "./globals/date";
import { innerJoin, join, leftJoin, rightJoin, fullJoin } from "./globals/join";
import { orderBy } from "./globals/sort";
import { aswitch } from "./globals/switch";
import { DictionaryObject } from "../utils/dictionaryObject";
export class functionsList {

    private static aggregateFunctions: DictionaryObject;

    private static functions: DictionaryObject;

     static get ExpressionFunctions(): DictionaryObject {
        if (functionsList.functions == undefined) {
            this.init();
        }
        return functionsList.functions;
    }

     static get AggregateFunctions(): DictionaryObject {
        return this.aggregateFunctions;
    }

    private static init() {
        functionsList.functions = new DictionaryObject();
        functionsList.functions.add("Find", new find());
        functionsList.functions.add("FindAll", new findAll());
        functionsList.functions.add("Limit", new limit());
        functionsList.functions.add("Exists", new exists());
        functionsList.functions.add("Eval", new eval1());

        functionsList.functions.add("GroupBy", new groupBy());
        let groupByNamed = new groupBy();
        groupByNamed.namedSelection = true;
        functionsList.functions.add("GroupByNamed", groupByNamed);
        functionsList.functions.add("Select", new select());
        let selectNamed = new select();
        selectNamed.namedSelection = true;
        functionsList.functions.add("SelectNamed", selectNamed);
        functionsList.functions.add("Matches", new matches());
        functionsList.functions.add("Substring", new substring());
        functionsList.functions.add("Replace", new replace());
        functionsList.functions.add("Split", new split());
        functionsList.functions.add("Concat", new concat());
        functionsList.functions.add("ConcatNonEmpty", new concatNonEmpty());
        functionsList.functions.add("Coalesce", new coalesce());
        functionsList.functions.add("Math_Abs", new math_Abs());
        functionsList.functions.add("Math_Acos", new math_Acos());
        functionsList.functions.add("Math_Min", new math_Min());
        functionsList.functions.add("Math_Pow", new math_Pow());
        functionsList.functions.add("Math_Pi", new math_Pi());
        functionsList.functions.add("Math_Round", new math_Round());
        functionsList.functions.add("Math_Sum", new math_Sum());
        functionsList.functions.add("Math_Tan", new math_Tan());
        functionsList.functions.add("Math_Cosh", new math_Cosh());
        functionsList.functions.add("Math_Tanh", new math_Tanh());
        functionsList.functions.add("Math_Exp", new math_Exp());
        functionsList.functions.add("Math_Sin", new math_Sin());
        functionsList.functions.add("Bool", new Bool());
        functionsList.functions.add("Math_Asin", new math_Asin());
        functionsList.functions.add("Math_Atan", new math_Atan());
        functionsList.functions.add("Math_Atan2", new math_Atan2());
        functionsList.functions.add("Math_Cos", new math_Cos());
        functionsList.functions.add("Math_Log", new math_Log());
        functionsList.functions.add("Math_Max", new math_Max());
        functionsList.functions.add("Contains", new contains());
        functionsList.functions.add("ToDate", new toDate());
        functionsList.functions.add("Now", new now());
        functionsList.functions.add("UTCNow", new utcNow());
        functionsList.functions.add("IsToday", new isToday());
        functionsList.functions.add("IsDate", new isDate());
        functionsList.functions.add("ToList", new toList());
        functionsList.functions.add("ToDictionary", new toDictionary());
        functionsList.functions.add("Percent", new percent());

        functionsList.functions.add("ToLower", new toLower());
        functionsList.functions.add("ToUpper", new toUpper());
        functionsList.functions.add("ToLocalTime", new toLocalTime());


        functionsList.functions.add("Math_Sqrt", new math_Sqrt());
        functionsList.functions.add("ToNumber", new toNumber());
        functionsList.functions.add("ToString", new toString());
        functionsList.functions.add("ToObjectList", new toObjectList());
        functionsList.functions.add("ToObject", new toObject());
        functionsList.functions.add("OrderBy", new orderBy());
        functionsList.functions.add("Switch", new aswitch());

        functionsList.functions.add("RegexReplace", new regexpReplace());
        functionsList.functions.add("ToUtc", new toUtc());
        functionsList.functions.add("AddPeriod", new addPeriod());
        functionsList.functions.add("Format", new stringformat());
        functionsList.functions.add("InnerJoin", new innerJoin());
        functionsList.functions.add("Join", new join());
        functionsList.functions.add("LeftJoin", new leftJoin());
        functionsList.functions.add("RightJoin", new rightJoin());
        functionsList.functions.add("FullJoin", new fullJoin());

        functionsList.aggregateFunctions = new DictionaryObject();
        functionsList.aggregateFunctions.add("Sum", new sum());
        functionsList.aggregateFunctions.add("Count", new count());
        functionsList.aggregateFunctions.add("Max", new max());
        functionsList.aggregateFunctions.add("Min", new min());
        functionsList.aggregateFunctions.add("GroupConcat", new groupConcat());
        functionsList.aggregateFunctions.add("First", new first());
        functionsList.aggregateFunctions.add("Last", new last());


    }
}