﻿import { aggregateFunction } from './../aggregateFunction'
import expressionData from '../../expressionData'
import { expressionPart } from './../../compiler/parts/expressionPart'
import { expressionCompiler, compileFlags } from './../../compiler/expressionCompiler'
import { typeMapper } from './../../types/typeMapper'
import { List } from 'typescript-dotnet-commonjs/System/Collections/List'
import { groupDataContext } from './../../functions/groupDataContext'
import { mfunction } from './../mfunction'
import { getType } from '../../utils/reflectionExtender'
import { outputVariables } from '../../utils/outputVariables'

export class Bool extends mfunction {

     /* internal */ /* override */ invoke(expressionData: expressionData, parameters: expressionPart[], /* out */resolved: outputVariables, running: groupDataContext | undefined): Object {
        if ((parameters.length > 0)) {
            let result = parameters[0].resolveGeneric<boolean>(expressionData, /* out */resolved, getType(Object), /* running:*/ running);
            return result;
        }
        resolved.resolved = false;
        return false;
    }
}

export class percent extends mfunction {

     /* override */ get minParameters(): number {
        return 1;
    }

     /* override */ get maxParameters(): number {
        return 2;
    }

     invoke(expressionData: expressionData, parameters: expressionPart[], /* out */resolved: outputVariables, running?: groupDataContext): Object {
        let obj: Object = expressionData.contexts.pop();
        let decimals: number = 2;
        if ((parameters.length > 0)) {
            let value: number = parameters[0].resolveGeneric<number>(expressionData, resolved, getType(Number), running, true);
            if ((resolved
                && (obj != null))) {
                resolved.resolved = true;
                return value > 0 ? Math.round((typeMapper.toType1<number>(obj) as number * (value / 100))) : 0.0;

            }

        }

        resolved.resolved = false;
        return obj;
    }
}

export class eval1 extends mfunction {

     /* override */ get minParameters(): number {
        return 1;
    }

     /* override */ get maxParameters(): number {
        return 2;
    }

     /* internal */ /* override */ invoke(expressionData: expressionData, parameters: expressionPart[], /* out */resolved: outputVariables, running: groupDataContext | undefined): Object {
        let obj: Object = expressionData.contexts.peek();
        if ((parameters.length > 0)) {
            let value: Object = parameters[0].resolve(expressionData, /* out */resolved, /* running:*/ running) as Object;
            if (resolved && obj != null) {
                resolved.resolved = true;
                let newPart: expressionPart = expressionCompiler.compile(value as string, undefined, undefined, new compileFlags()) as expressionPart;
                let newValue: Object = newPart.resolveGeneric<Object>(expressionData, resolved, getType(Object), running);
                if ((resolved.resolved
                    && (obj != null))) {
                    resolved.resolved = true;
                    return newValue;
                }

            }

        }

        resolved.resolved = false;
        return obj;
    }
}