﻿import { List } from "typescript-dotnet-commonjs/System/Collections/List";
import expressionData from "../../expressionData";
import { expressionPart } from "../../compiler/parts/expressionPart";
import { mfunction } from "../mfunction";
import { groupDataContext } from "../groupDataContext";
import { getType } from "../../utils/reflectionExtender";
import { outputVariables } from "../../utils/outputVariables";

export class limit extends mfunction {
   get /* override */ minParameters(): number {
    return 1;
  }

   get /* override */ maxParameters(): number {
    return 2;
  }

   invoke(expressionData: expressionData, parameters: expressionPart[], resolved: outputVariables, running: groupDataContext | undefined): Object {
    let list: List<Object> = new List<Object>();

    let tempData: expressionData = expressionData.toSubEx();
    let obj: Object = expressionData.contexts.peek();
    let subResolved: outputVariables = new outputVariables();
    if (obj instanceof List) {
      let limitStart: number = 0;
      let limitCount: number = 0;
      if (parameters.length > 1) {
        limitStart = parameters[0].resolve(expressionData, /* out */ subResolved) as number;
        limitCount = parameters[1].resolve(expressionData, /* out */ subResolved) as number;
      } else if (parameters.length == 1) {
        limitStart = 0;
        limitCount = parameters[0].resolve(expressionData, /* out */ subResolved) as number;
      }

      resolved.resolved = true;
      let count: number = 0;
      let includeCount: number = 0;
      for (let item of (<List<Object>>obj).toArray()) {
        if (count >= limitStart) {
          list.add(item);
          includeCount++;
          if (limitCount > 0 && includeCount >= limitCount) {
            break;
          }
        }

        count++;
      }
    }

    return list;
  }
}

class baseFind extends mfunction {
  invoke(
    expressionData: expressionData,
    parameters: expressionPart[],
    resolved: outputVariables,
    running?: groupDataContext | undefined
  ): Object | undefined {
    throw new Error("Method not implemented.");
  }

  protected static applyFilter(expressionData: expressionData, parameters: expressionPart[], firstMatch: boolean, resolved: outputVariables,
    running: groupDataContext | undefined
  ): Object | undefined {
    let result: List<Object> = new List<Object>();
    resolved.resolved = false;
    let tempData: expressionData = expressionData.toSubEx();
    let obj: Object = expressionData.contexts.peek();
    let filterExpression: expressionPart = parameters[0];
    if (obj instanceof List || obj instanceof Array) {
      let done: boolean = false;
      while (!done) {
        result.clear();
        //  Clear();
        try {
          let forObj = [];
          if (obj instanceof List) {
            forObj = (<List<Object>>obj).toArray();
          } else forObj = obj;

          for (let item of forObj) {
            tempData.contexts.push(item);
            let subResolved: outputVariables = new outputVariables();
            let bResult: boolean = filterExpression.resolveGeneric<boolean>(
              tempData,
              /* out */ subResolved,
              getType(Boolean),
              running
            );
            tempData.contexts.pop();
            if (subResolved.resolved && bResult) {
              if (firstMatch) {
                resolved.resolved = true;
                return item;
              } else {
                result.add(item);
              }
            }
          }

          done = true;
        } catch (e) {
          throw e;
        }
      }
    }

    //  If 'single' was true and the loop didnt return before reaching here, it means no items matched.
    if (firstMatch) {
      resolved.resolved = true;
      return undefined;
    } else {
      resolved.resolved = true;
      return result;
    }
  }
}

export class findAll extends baseFind {
   invoke(
    expressionData: expressionData,
    parameters: expressionPart[],
    /* out */ resolved: outputVariables,
    running: groupDataContext | undefined
  ): Object | undefined {
    return baseFind.applyFilter(
      expressionData,
      parameters,
      false,
      /* out */ resolved,
      running
    );
  }
}

export class find extends baseFind {
   invoke(
    expressionData: expressionData,
    parameters: expressionPart[],
    /* out */ resolved: outputVariables,
    running: groupDataContext | undefined
  ): Object | undefined {
    return baseFind.applyFilter(
      expressionData,
      parameters,
      true,
      /* out */ resolved,
      running
    );
  }
}

export class exists extends baseFind {
   invoke(
    expressionData: expressionData,
    parameters: expressionPart[],
    /* out */ resolved: outputVariables,
    running: groupDataContext | undefined
  ): Object {
    return (
      baseFind.applyFilter(
        expressionData,
        parameters,
        true,
        /* out */ resolved,
        running
      ) != null
    );
  }
}
