﻿import { IEnumerable } from 'typescript-dotnet-commonjs/System/Collections/Enumeration/IEnumerable'
import { List } from 'typescript-dotnet-commonjs/System/Collections/List'
import { mfunction } from './../mfunction'
import expressionData from '../../expressionData'
import { expressionPart } from './../../compiler/parts/expressionPart'
import { groupDataContext } from './../../functions/groupDataContext'
import { selection } from './../selection'
import { getType } from '../../utils/reflectionExtender'
import { outputVariables } from '../../utils/outputVariables'
enum joinType {

    REGULAR,

    FULL,

    INNER,

    LEFT,

    RIGHT,
}
export class joinContext {

     constructor() {

    }
     item1: Object | undefined;
    //  get item1(): Object {
    // }
    //  set item1(value: Object)  {
    // }

     item2: Object | undefined;
    //  get item2(): Object {
    // }
    //  set item2(value: Object)  {
    // }

     get left(): Object {
        return <Object>this.item1;
    }
     set left(value: Object) {
        this.item1 = value;
    }

     get right(): Object {
        return <Object>this.item2;
    }
     set right(value: Object) {
        this.item2 = value;
    }
}

abstract class baseJoin extends mfunction {

     /* override */ get minParameters(): number {
        return 2;
    }

     /* override */ get maxParameters(): number {
        return 2;
    }

    protected abstract get type(): joinType;

     invoke(expressionData: expressionData, parameters: expressionPart[], /* out */resolved: outputVariables, running: groupDataContext | undefined): Object {
        return this.join(expressionData, parameters, this.type, /* out */resolved, running);
    }


    protected /* internal */ join(expressionData: expressionData, joinParams:expressionPart[], type: joinType, /* out */resolved: outputVariables, running: groupDataContext | undefined): Object {
        let left: Object = expressionData.contexts.peek();
        let tempData: expressionData = expressionData.toSubEx();
        tempData.cacheEnabled = false;
        if ((left as IEnumerable<any>).getEnumerator() !== undefined) {
            let rightResolved: outputVariables = new outputVariables();
            let right: Object | undefined = undefined;
            if ((joinParams.length > 0)) {
                right = joinParams[0].resolve(expressionData, /* out */rightResolved, running);
            }

            let main: List<any> | undefined = undefined;
            let sub: List<any> | undefined = undefined;
            let joinCondition: expressionPart | undefined = undefined;
            switch (type) {
                case joinType.REGULAR:
                case joinType.FULL:
                case joinType.INNER:
                case joinType.LEFT:
                    main = (<List<any>>(left));
                    sub = (<List<any>>(right));
                    joinCondition = type == joinType.REGULAR ? undefined : joinParams.length > 1 ? joinParams[1] : undefined;
                    break;
                case joinType.RIGHT:
                    main = (<List<any>>(right));
                    sub = (<List<any>>(left));
                    joinCondition = joinParams.length > 1 ? joinParams[1] : undefined;
                    break;
            }

            let done: boolean = false;
            while (!done) {
                try {
                    if ((main != null) && (sub != null)) {
                        let tempResult: List<joinContext> = new List<joinContext>();
                        let mainCounter: number = 0;
                        let _joinContext: joinContext = new joinContext();
                        for (let item1 of main.toArray()) {
                            let subSelection: List<joinContext> = new List<joinContext>();
                            let subCounter: number = 0;
                            for (let item2 of sub.toArray()) {
                                _joinContext.item1 = type == joinType.RIGHT ? item2 : item1;

                                _joinContext.item2 = type == joinType.RIGHT ? item1 : item2;

                                tempData.contexts.push(_joinContext);
                                let subResolved: outputVariables = new outputVariables();
                                let condition: boolean = ((joinCondition == null)
                                    || joinCondition.resolveGeneric<boolean>(tempData, /* out */subResolved, getType(Boolean),  /* running:*/ running));

                                if (subResolved.resolved && condition) {
                                    var sjoinContext = new joinContext();
                                    sjoinContext.item1 = _joinContext.item1;
                                    sjoinContext.item2 = _joinContext.item2;
                                    subSelection.add(sjoinContext);
                                }

                                tempData.contexts.pop();
                                subCounter++;
                            }

                            if ((subSelection.count > 0)) {
                                for (let item of subSelection.toArray()) {
                                    tempResult.add(item);
                                }

                            }
                            else {
                                switch (type) {
                                    case joinType.RIGHT:
                                    case joinType.LEFT:
                                        let jContext = new joinContext();
                                        jContext.item1 = type == joinType.RIGHT ? null : item1;
                                        jContext.item2 = type == joinType.RIGHT ? item1 : null
                                        tempResult.add(jContext);
                                        break;
                                }

                            }

                            mainCounter++;
                        }

                        resolved.resolved = true;
                        return baseJoin.normalize(tempResult);
                    }

                    done = true;
                }
                catch (ex) {
                    let d = ex;
                }

            }

        }

        resolved.resolved = false;
        return left;

    }

    private static normalize(items: List<joinContext>): List<Object> {
        let result: List<Object> = new List<Object>();
        for (let item of items.toArray()) {
            let rSelection: selection = new selection(undefined);
            if ((item.item1 as selection).clear !== undefined) {
                for (let sItem of (<selection>(item.item1)).toArray()) {
                    rSelection.add(sItem);
                }

            }
            else {
                rSelection.add(item.item1 as Object);
            }

            if ((item.item2 instanceof selection)) {
                for (let sItem in (<selection>(item.item2))) {
                    rSelection.add(sItem);
                }

            }
            else {
                rSelection.add(item.item2 as Object);
            }

            result.add(rSelection);
        }

        return result;
    }

    private toArray(v: Object): Object[] {
        if ((v as Object[]).values !== undefined) {
            return (v as Object[]);
        }
        else {
            let arrayObject = new List<Object>();
            arrayObject.add(v);
            return v instanceof Object ? (v as object[]) : arrayObject.toArray();

        }
    }
}

export class join extends baseJoin {

     /* override */ get minParameters(): number {
        return 1;
    }

     /* override */ get maxParameters(): number {
        return 1;
    }

    protected /* override */ get type(): joinType {
        return joinType.REGULAR;
    }
}

export class fullJoin extends baseJoin {

    protected /* override */ get type(): joinType {
        return joinType.FULL;
    }
}

export class innerJoin extends baseJoin {

    protected /* override */ get type(): joinType {
        return joinType.INNER;
    }
}

export class leftJoin extends baseJoin {

    protected /* override */ get type(): joinType {
        return joinType.LEFT;
    }
}

export class rightJoin extends baseJoin {

    protected /* override */ get type(): joinType {
        return joinType.RIGHT;
    }
}
