﻿import expressionData from '../../expressionData'
import { expressionPart } from '../../compiler/parts/expressionPart'
import { mfunction } from '../mfunction'
import { List } from 'typescript-dotnet-commonjs/System/Collections/List'
import { groupDataContext } from '../groupDataContext'
import { RegexOptions, Regex } from 'typescript-dotnet-commonjs/System/Text/RegularExpressions'
import { outputVariables } from '../../utils/outputVariables'
import * as dotnet from 'typescript-dotnet-commonjs/System/Text/Utility';
import DateTime from 'typescript-dotnet-commonjs/System/Time/DateTime';
import { format } from 'date-fns';


export class substring extends mfunction {

     /* override */ get minParameters(): number {
        return 1;
    }

     /* override */ get maxParameters(): number {
        return 2;
    }

     /* internal */ /* override */ invoke(expressionData: expressionData, parameters: expressionPart[], /* out */resolved: outputVariables, running: groupDataContext | undefined): Object {
        let obj: Object = expressionData.contexts.peek();
        if (obj != null && typeof obj == "string") {
            if ((parameters.length == 1)) {
                let startIndex: number = parameters[0].resolve(expressionData, /* out */resolved, /* running:*/ running) as number;
                if (resolved) {
                    return (<string>(obj)).substring(startIndex);
                }

            }
            else if ((parameters.length >= 2)) {
                let startIndex: number = parameters[0].resolve(expressionData, /* out */resolved, /* running:*/ running) as number;
                let length: number = parameters[1].resolve(expressionData, /* out */resolved, /* running:*/ running) as number;
                if (resolved) {
                    let str: string = (<string>(obj));
                    if (((startIndex >= 0)
                        && (str.length
                            >= (startIndex + length)))) {
                        return str.substring(startIndex, length);
                    }
                    else {
                        return str;
                    }

                }

            }

        }

        resolved.resolved = false;
        return obj;
    }
}

export class replace extends mfunction {

     /* override */ get minParameters(): number {
        return 2;
    }

     /* override */ get maxParameters(): number {
        return 2;
    }

    invoke(expressionData: expressionData, parameters: expressionPart[], /* out */resolved: outputVariables, running: groupDataContext | undefined): Object {
        let obj: Object = expressionData.contexts.peek();
        if (obj != null && typeof obj == "string" && parameters.length >= 2) {
            let str1: string = parameters[0].resolve(expressionData, /* out */resolved, /* running:*/ running) as string;
            let str2: string = parameters[1].resolve(expressionData, /* out */resolved, /* running:*/ running) as string;
            if (resolved) {
                return (<string>(obj)).replace(str1, str2);
            }

        }

        resolved.resolved = false;
        return obj;
    }
}

export class split extends mfunction {

    get minParameters(): number {
        return 1;
    }

    get maxParameters(): number {
        return 1;
    }

    invoke(expressionData: expressionData, parameters: expressionPart[], /* out */resolved: outputVariables, running: groupDataContext | undefined): Object {
        let obj: Object = expressionData.contexts.peek();
        if (obj != null && typeof obj == "string" && parameters.length >= 1) {
            let str1: string = parameters[0].resolve(expressionData, /* out */resolved, /* running:*/ running) as string;
            if (resolved) {
                return (<string>(obj)).split(str1);
            }
        }

        resolved.resolved = false;
        return obj;
    }
}

export class toUpper extends mfunction {

     /* override */ get minParameters(): number {
        return 0;
    }

     /* override */ get maxParameters(): number {
        return 0;
    }

     /* internal */ /* override */ invoke(expressionData: expressionData, parameters: expressionPart[], /* out */resolved: outputVariables, running: groupDataContext | undefined): Object {
        let obj: Object = expressionData.contexts.peek();
        if (obj != null && typeof obj == 'string') {
            resolved.resolved = true;
            return (<string>(obj)).toUpperCase();
        }

        resolved.resolved = false;
        return obj;
    }
}

export class toLower extends mfunction {

     /* override */ get minParameters(): number {
        return 0;
    }

     /* override */ get maxParameters(): number {
        return 0;
    }

    invoke(expressionData: expressionData, parameters: expressionPart[], /* out */resolved: outputVariables, running: groupDataContext | undefined): Object {
        let obj: Object = expressionData.contexts.peek();
        if ((obj != null)
            && typeof obj == 'string') {
            resolved.resolved = true;
            return (<string>(obj)).toLowerCase();
        }

        resolved.resolved = false;
        return obj;
    }
}

export class toString extends mfunction {

     /* override */ get minParameters(): number {
        return 0;
    }

     /* override */ get maxParameters(): number {
        return 0;
    }

    invoke(expressionData: expressionData, parameters: expressionPart[], /* out */resolved: outputVariables, running: groupDataContext | undefined): Object {
        let obj: Object = expressionData.contexts.peek();
        resolved.resolved = true;
        return obj.toString();
        // TODO: Warning!!!, inline IF is not supported ?
        (obj != null);
        "";
    }
}

export class stringformat extends mfunction {

     /* override */ get minParameters(): number {
        return 1;
    }

     /* override */ get maxParameters(): number {
        return 1;
    }

    dateFormat(dt: any, sformat: string): Object {
        sformat = sformat.replace('0:', '').
            replace('{', '').
            replace('}', '').replace('tt', 'aaa').replace('dddd', 'EEEE').replace('ddd', 'eee');
        return format(dt, sformat);
    }
     /* internal */ /* override */ invoke(expressionData: expressionData, parameters: expressionPart[], /* out */resolved: outputVariables, running: groupDataContext | undefined): Object {
        let obj: Object = expressionData.contexts.peek();
        if (((obj != null)
            && (parameters.length == 1))) {
            let sformat: string = parameters[0].resolve(expressionData, /* out */resolved, /* running:*/ running) as string;
            expressionData.dtFormat = sformat;
            resolved.resolved = true;
            if (obj instanceof DateTime || obj instanceof Date) {
                const dt = obj instanceof DateTime ? obj.toJsDate() : obj// let d = Date();
                return this.dateFormat(dt, sformat);
            }
            else if (typeof obj === 'number' || typeof obj === 'string') {
                if (typeof obj === 'number' && sformat.includes("00.")) {
                    return dotnet.format(sformat, obj);
                }
                const dt = new Date(obj);
                if (!isNaN(dt.getTime())) {
                    return this.dateFormat(dt, sformat);
                }
            }
            else {
                return dotnet.format(sformat, obj);
            }
        }
        resolved.resolved = false;
        return obj;
    }
}

export class concatNonEmpty extends mfunction {

     /* override */ get minParameters(): number {
        return 2;
    }

     /* override */ get maxParameters(): number {
        return -1;
    }

    invoke(expressionData: expressionData, parameters: expressionPart[], /* out */resolved: outputVariables, running: groupDataContext | undefined): Object | undefined {
        let obj: Object = expressionData.contexts.peek();
        if ((parameters.length > 1)) {
            let items: List<Object> = new List<Object>();//(parameters.length - 1);
            let subResolved: outputVariables = new outputVariables();
            let delimiter: string = parameters[0].resolve(expressionData, /* out */subResolved, /* running:*/ running) as string;
            if (delimiter! = "" && delimiter != null)// String.isNullOrEmpty(delimiter)) 
            {
                delimiter = ",";
            }

            for (let i: number = 1; (i < parameters.length); i++) {
                let subParamResolved: outputVariables = new outputVariables();
                let val = parameters[i].resolve(expressionData, /* out */subParamResolved, /* running:*/ running);
                if ((subParamResolved
                    && ((val != null)
                        && (!(typeof val == 'string')
                            || !String.isNullOrEmpty((<string>(val))))))) {
                    items.add(val);
                }

            }

            if (subResolved.resolved) {
                resolved.resolved = true;
                if (items.count == 1 && items.get(0) instanceof List && !(typeof items.get(0) == 'string')) {
                    return (items.get(0) as List<Object>).toArray().join(delimiter);
                }
                else {
                    return items.toArray().join(delimiter);
                }

            }

            resolved.resolved = false;
            return obj;
        }
    }
}

export class concat extends mfunction {

     /* override */ get minParameters(): number {
        return 2;
    }

     /* override */ get maxParameters(): number {
        return -1;
    }

    invoke(expressionData: expressionData, parameters: expressionPart[], /* out */resolved: outputVariables, running: groupDataContext | undefined): Object | undefined {
        let obj: Object = expressionData.contexts.peek();
        if ((parameters.length > 1)) {
            let items: Object[] = new Array((parameters.length - 1));
            let subResolved: outputVariables = new outputVariables();
            let delimiter: string = parameters[0].resolve(expressionData, /* out */subResolved, /* running:*/ running) as string;
            if (delimiter != "" && delimiter != null)//string.IsNullOrEmpty(delimiter)) 
            {
                delimiter = ",";
            }

            for (let i: number = 1; (i < parameters.length); i++) {

                let subParamResolved: outputVariables = new outputVariables();

                items[i - 1] = parameters[i].resolve(expressionData, /* out */subParamResolved, /* running:*/ running) as Object;
                subResolved = (subResolved || subParamResolved);
            }

            if (subResolved) {
                resolved.resolved = true;
                if (items.length == 1 && items[0] instanceof List && !(typeof items[0] == 'string')) {
                    return items[0].toArray().join(delimiter);
                }
                else {
                    return items.join(delimiter);
                }

            }

        }

        resolved.resolved = false;
        return obj;
    }
}

export class contains extends mfunction {

     /* override */ get minParameters(): number {
        return 1;
    }

     /* override */ get maxParameters(): number {
        return 2;
    }

    invoke(expressionData: expressionData, parameters: expressionPart[], /* out */resolved: outputVariables, running: groupDataContext | undefined): Object {
        let context: Object = expressionData.contexts.peek();
        if ((parameters.length > 0)) {
            let subResolved: outputVariables = new outputVariables();
            let obj: Object = context;
            let param: Object = parameters[0].resolve(expressionData, /* out */subResolved, /* running:*/ running) as Object;
            if ((parameters.length > 1)) {
                obj = param;
                param = parameters[1].resolve(expressionData, /* out */subResolved, /* running:*/ running) as Object;
            }

            if (subResolved) {
                if (typeof obj == 'string' && typeof param == 'string') {//|| param instanceof char
                    resolved.resolved = true;
                    let strObj: string = (<string>(obj));
                    if (!String.isNullOrEmpty(param)) {
                        return strObj.indexOf(param) >= 0;
                    }
                    // else if (typeof param === 'char') {
                    //     return strObj.indexOf(param.toString())>0;
                    // }

                }
                else if (obj instanceof List) {
                    resolved.resolved = true;
                    for (let item of (<List<Object>>(obj)).toArray()) {
                        if (((item != null)
                            && (item as string).equals(param))) {
                            return true;
                        }
                        else if ((item == param)) {
                            return true;
                        }

                    }

                }

            }

        }

        resolved.resolved = false;
        return context;
    }
}

export class matches extends mfunction {

     /* override */ get minParameters(): number {
        return 1;
    }

     /* override */ get maxParameters(): number {
        return 2;
    }

    invoke(expressionDataObj: expressionData, parameters: expressionPart[], /* out */resolved: outputVariables, running: groupDataContext | undefined): Object | undefined {

        let context: Object = expressionDataObj.contexts.peek();
        if ((parameters.length > 0)) {
            let subResolved: outputVariables = new outputVariables();
            let obj: string = (<string>(context));
            let regex: string = parameters[0].resolve(expressionDataObj, /* out */subResolved, /* running:*/ running) as string;
            let ro: RegexOptions.Literal = RegexOptions.GLOBAL;// RegexOptions.None;
            if ((parameters.length > 1)) {
                let param: string = parameters[1].resolve(expressionDataObj, /* out */subResolved, /* running:*/ running) as string;
                if (param.indexOf('m') > -1) {
                    ro = RegexOptions.MULTI_LINE; //(ro | RegexOptions.MULTI_LINE);
                }

                if (param.indexOf('i') > -1) {
                    ro = RegexOptions.IGNORE_CASE; //(ro | RegexOptions.IGNORE_CASE);
                }

                if (param.indexOf('w') > -1) {
                    ro = RegexOptions.IGNORE_PATTERN_WHITESPACE; //(ro | RegexOptions.IGNORE_PATTERN_WHITESPACE);
                }

                // if (param.Contains('s')) {
                //     ro = (ro | RegexOptions.SINGLE_LINE);
                // }

                /*if (param.indexOf('r')>-1) {
                    ro = //(ro | RegexOptions.RightToLeft);
                }
                
                if (param.indexOf('e')>-1) {
                    ro = (ro | RegexOptions.ExplicitCapture);
                }*/

            }

            if ((subResolved.resolved && ((obj != null) && (regex != null)))) {
                resolved.resolved = true;
                if (ro != null)// RegexOptions.)
                {
                    const regexLiteral: RegexOptions.Literal[] = [ro];
                    let result: boolean = false;
                    let reg = new Regex(regex, RegexOptions.MULTI_LINE);

                    result = (Regex.isMatch(obj, regex, regexLiteral));
                    return result;
                }
                else {
                    return (Regex.isMatch(obj, regex));
                }

            }

        }

        resolved.resolved = false;
        return false;
    }
}

export class regexpReplace extends mfunction {

     /* override */ get minParameters(): number {
        return 2;
    }

     /* override */ get maxParameters(): number {
        return 3;
    }

     /* internal */ /* override */ invoke(expressionData: expressionData, parameters: expressionPart[], /* out */resolved: outputVariables, running: groupDataContext | undefined): Object {
        let context: Object = expressionData.contexts.peek();
        if ((parameters.length > 1)) {
            let subResolved: outputVariables = new outputVariables();
            let obj: string = (<string>(context));
            let regex: string = parameters[0].resolve(expressionData, /* out */subResolved, /* running:*/ running) as string;
            let replace: string = parameters[1].resolve(expressionData, /* out */subResolved, /* running:*/ running) as string;
            let ro: RegexOptions.Literal[] = []; //RegexOptions.None;
            let flags: string | undefined;
            if ((parameters.length > 2)) {
                let param: string = parameters[2].resolve(expressionData, /* out */subResolved, /* running:*/ running) as string;
                flags = param;
                if (param.indexOf('m') > 0) {
                    ro.push(RegexOptions.MULTI_LINE);
                }

                if (param.indexOf('i') > 0) {
                    ro.push(RegexOptions.IGNORE_CASE);
                }

                if (param.indexOf('w') > 0) {
                    ro.push(RegexOptions.IGNORE_PATTERN_WHITESPACE);
                }

                // if (param.indexOf('s')>0) {
                //     ro = (ro | RegexOptions.Singleline);
                // }

                // if (param.indexOf('r')>0) {
                //     ro = (ro | RegexOptions.RightToLeft);
                // }

                // if (param.indexOf('e')>0) {
                //     ro = (ro | RegexOptions.ExplicitCapture);
                // }

            }

            if ((subResolved.resolved
                && ((obj != null)
                    && (regex != null)))) {
                resolved.resolved = true;
                if ((ro != null)) {
                    const regexLiteral: RegexOptions.Literal[] = ro;
                    let d = new RegExp(regex, flags);

                    d.test(replace)
                    return obj.replace(d, replace);
                }
                else {
                    return Regex.replace(obj, regex, replace);
                }
            }
        }

        resolved.resolved = false;
        return context;
    }
}