import { List } from 'typescript-dotnet-commonjs/System/Collections/List'
import expressionData from '../expressionData'
import { expressionPart } from '../compiler/parts/expressionPart'
import { expressionCompiler } from '../compiler/expressionCompiler'
import { compileFlags } from '../compiler/expressionCompiler'
import { invalidParametersException } from '../error/invalidParametersException'
import { groupDataContext } from './../functions/groupDataContext'
import { Type } from 'typescript'
import { outputVariables } from '../utils/outputVariables'

export abstract class mfunction {

     get functionName(): string {
        return typeof this;
    }

    protected getFunctionName () : string{
        return this.functionName;
    }
    
     /* virtual */ get minParameters(): number {
        return 1;
    }

     /* virtual */ get maxParameters(): number {
        return 1;
    }

     compileParameters(paramList: List<string>, flags: compileFlags): expressionPart[] {
        if ((((this.minParameters > 0)
            && (paramList.count < this.minParameters))
            || ((this.maxParameters >= 0)
                && (paramList.count > this.maxParameters)))) {
            throw new invalidParametersException(this.functionName, this.minParameters, this.maxParameters);
        }

        let parameters: expressionPart[] = [];
        for (let item of paramList.toArray()) {
            parameters.push(expressionCompiler.compile(item, undefined, undefined, flags) as expressionPart);
        }

        return parameters;
    }

    abstract invoke(expressionData: expressionData, parameters: expressionPart[], /* out */resolved: outputVariables, running?: groupDataContext): Object | undefined;

     resolveParameter<T>(parameter: expressionPart, tType: Type, data: expressionData, resolved: outputVariables, evaluateAllContexts: boolean = true, running: groupDataContext | undefined = undefined): T {
        let result: Object = parameter.resolveGeneric<T>(data, /* out */resolved, tType, running, /* evaluateAllContexts:*/ evaluateAllContexts);
        if ((data.parent != null)) {
            // data.parent.monitorIfRequired(parameter, result);
        }
        else {
            // data.monitorIfRequired(parameter, result);
        }

        return result as T;
    }
}