import { DateTime } from 'typescript-dotnet-commonjs/System/Time/DateTime';
import { isFloatType } from './general';

const legacyBuiltInProperty = {
    ["length"]: undefined,
    ["count"]: undefined,
    ["ticks"]: function (value: any) { return value; },
    ["date"]: function (value: any) { return Date.parse(new Date(value).toDateString()); },
    ["timeofday"]: function (value: any) {
        const dt = new Date(value);
        return dt.toLocaleString();
    },
    ["year"]: function (value: any) { return (new DateTime(value)).year; },
    ["month"]: function (value: any) { return (new DateTime(value)).month; },
    ["minute"]: function (value: any) { return (new DateTime(value)).timeOfDay.minute; },
    ["day"]: function (value: any) { return (new DateTime(value)).toTimeStamp().day; },
    ["second"]: function (value: any) { return (new DateTime(value)).timeOfDay.second; },
    ["hour"]: function (value: any) { return (new DateTime(value)).timeOfDay.hour; },
    ["totaldays"]: function (value: any) { return (new DateTime(value)).timeOfDay.total.days; },
    ["totalhours"]: function (value: any) { return (((new Date(value).getTime()) / 1000) / 60) / 60; },
    ["totalminutes"]: function (value: any) { return ((new Date(value).getTime()) / 1000) / 60; },
    ["totalseconds"]: function (value: any) {
        const totalSeconds = ((new Date(value).getTime()) / 1000);
        if(isFloatType(totalSeconds))
            return totalSeconds.toFixed(2)
        return totalSeconds;
    },
}

export default legacyBuiltInProperty;