import { testContext, getContext } from "../testContext";
import { expressionEngine } from "../../src/expressionEngine";
import { expect } from "chai";
import { getType } from "../../src/utils/reflectionExtender";
import List from "typescript-dotnet-commonjs/System/Collections/List";

describe('  Expression engine - Aggregates', () => {
    it('Case 1 - pass case - GroupBy', () => {
      let context: testContext = getContext();
 
      let result: List<Object> = expressionEngine.evalGeneric<List<object>>(context, "SubList.GroupBy(Address)");
      expect(2).to.equal(result.count);
    });

    it('Case 2 - pass case - GroupBy with Sum', () => {
      let context: testContext = getContext();
  
      let result: List<Object> = expressionEngine.evalGeneric<List<object>>(context, "SubList.GroupBy(Address,Sum(Age))");
      expect(2).to.equal(result.count);
    });

    it('Case 3 - pass case - GroupBy with Sum by name', () => {
      let context: testContext = getContext();
  
      let result: List<Object> = expressionEngine.evalGeneric<List<object>>(context, "SubList.GroupBy(Name,Sum(Age))");
      expect(4).to.equal(result.count);
    });

    it('Case 4 - pass case - GroupBy with Sum by name and address', () => {
      let context: testContext = getContext();
  
      let result: List<Object> = expressionEngine.evalGeneric<List<object>>(context, "SubList.GroupBy(Name,Address, Sum(Age))");
      expect(4).to.equal(result.count);
    });

    it('Case 5 - pass case - GroupBy with Sum by name and address', () => {
      let context: testContext = getContext();
  
      let result: List<Object> = expressionEngine.evalGeneric<List<object>>(context, "SubList.GroupBy(Name, Address, Sum(Age) + 1)");
      expect(4).to.equal(result.count);
    });

    it('Case 6 - pass case - GroupBy with Sum and count', () => {
      let context: testContext = getContext();
  
      let result: List<Object> = expressionEngine.evalGeneric<List<object>>(context, "SubList.GroupBy(Name, Address, Sum(Age) / Count(Age))");
      expect(4).to.equal(result.count);
    });

    it('Case 7 - pass case - GroupBy with Sum', () => {
      let context: testContext = getContext();
  
      let result: List<Object> = expressionEngine.evalGeneric<List<object>>(context, "SubList.GroupBy(Address @ \"Address\", Sum(Age) @ \"Sum\", Sum(Age) @ \"Sum\", Sum(Age) @ \"Sum\")");
      expect(2).to.equal(result.count);
    });

    it('Case 8 - pass case - GroupBy with Max', () => {
      let context: testContext = getContext();
  
      let result: List<Object> = expressionEngine.evalGeneric<List<object>>(context, "SubList.GroupBy(Address, Max(Age))");
      expect(2).to.equal(result.count);
    });

    it('Case 9 - pass case - GroupBy with Max and Sum', () => {
      let context: testContext = getContext();
  
      let result: List<Object> = expressionEngine.evalGeneric<List<object>>(context, "SubList.GroupBy(Name, Max(Age + 1) + Max(Age), Sum(Age))");
      expect(4).to.equal(result.count);
    });

    it('Case 10 - pass case - GroupBy with Min and Sum', () => {
      let context: testContext = getContext();
  
      let result: List<Object> = expressionEngine.evalGeneric<List<object>>(context, "SubList.GroupBy(Name, Min(Age + 1) + Min(Age), Sum(Age))");
      expect(4).to.equal(result.count);
    });

    it('Case 10 - pass case - GroupConcat', () => {
      let context: testContext = getContext();
  
      let result: List<Object> = expressionEngine.evalGeneric<List<object>>(context, "SubList.GroupBy(Name, GroupConcat(Age +1))");
      expect(4).to.equal(result.count);
    });

    it('Case 11 - pass case - First', () => {
      let context: testContext = getContext();
  
      let result: List<Object> = expressionEngine.evalGeneric<List<object>>(context, "SubList.GroupBy(Name, First(Age))");
      expect(4).to.equal(result.count);
    });

    it('Case 12 - pass case - Last', () => {
      let context: testContext = getContext();
  
      let result: List<Object> = expressionEngine.evalGeneric<List<object>>(context, "SubList.GroupBy(Name, Last(Age))");
      expect(4).to.equal(result.count);
    });

    
});

