import { getContext, testContext } from "../testContext";
import { expressionEngine } from "../../src/expressionEngine";
import { expect } from "chai";

describe('  Expression engine - Built-in property', () => {
    it('Case 1 - pass case - Count', () => {
        let context: testContext = testContext.getInstance();
        context.List.add("Value 1");
        context.List.add("Value 2");
        context.List.add("Value 3");
        context.List.add("Value 4");

        let result: number = expressionEngine.evalGeneric<number>(context, "List.Count");
        expect(4).to.equal(result);
    });

    it('Case 1 - pass case - Length', () => {
        let context: testContext = testContext.getInstance();
        let result: number = expressionEngine.evalGeneric<number>(context, "'abcd'.Length", undefined, undefined, true);
        expect(4).to.equal(result);
    });
});

describe('  Expression engine - Built-in property', () => {

    const partExpressions = "ToDate(\"10-10-2010 10:50:03 AM\")";
    let context: testContext = testContext.getInstance();

    it('Case 1 - pass case - Hour', () => {
        let result: number = expressionEngine.evalGeneric<number>(context, partExpressions + ".Hour", undefined, undefined, true);
        expect(10).to.equal(result);
    });

    it('Case 2 - pass case - Minute', () => {
        let result: number = expressionEngine.evalGeneric<number>(context, partExpressions + ".Minute");
        expect(50).to.equal(result);
    });

    it('Case 3 - pass case - Second', () => {

        let result: number = expressionEngine.evalGeneric<number>(context, partExpressions + ".Second");
        expect(3).to.equal(result);
    });

    it('Case 4 - pass case - Month', () => {
        let result: number = expressionEngine.evalGeneric<number>(context, partExpressions + ".Month");
        expect(10).to.equal(result + 1);
    });

    it('Case 5 - pass case - Year', () => {
        let result: number = expressionEngine.evalGeneric<number>(context, partExpressions + ".Year");
        expect(2010).to.equal(result);
    });

    it('Case 6 - pass case - TotalDays', () => {
        let result: number = expressionEngine.evalGeneric<number>(context, partExpressions + ".TotalDays");
        expect(0.4514236111111111).to.equal(result);
    });

    it('Case 7 - pass case - Total Hours', () => {
        let result: number = expressionEngine.evalGeneric<number>(context, partExpressions + ".TotalHours");
        expect(357413.83416666667).to.equal(result);
    });

    it('Case 8 - pass case - Total minutes', () => {
        let result: number = expressionEngine.evalGeneric<number>(context, partExpressions + ".TotalMinutes");
        expect(21444830).to.equal(Math.round(result));
    });

    it('Case 9 - pass case - TimeOfDay', () => {
        let result: number = expressionEngine.evalGeneric<number>(context, partExpressions + ".TimeOfDay");
        expect('10/10/2010, 10:50:03 AM').to.equal(result);
    });

    it('Case 10 - pass case - TotalSeconds', () => {

        let result: number = expressionEngine.evalGeneric<number>(context, partExpressions + ".TotalSeconds", undefined, undefined, true);
        expect(1286689803).to.equal(result);
    });

    it('Case 11 - pass case - Day', () => {

        let result: number = expressionEngine.evalGeneric<number>(context, partExpressions + ".Day", undefined, undefined, true);
        expect(10).to.equal(result);
    });

    it('Case 12 - pass case - Ticks', () => {

        let result: number = expressionEngine.evalGeneric<number>(context, partExpressions + ".Ticks", undefined, undefined, true);
        expect(1286689803000).to.equal(result);
    });

    // it('Case 11 - pass case - TotalDays', () => {
    //     
    //     let result: number = expressionEngine.evalGeneric<number>(undefined, "'abcd'.Length", undefined, undefined, true);
    //     expect(4).to.equal(result);
    // });

    // it('Case 12 - pass case - TotalHours', () => {
    //     
    //     let result: number = expressionEngine.evalGeneric<number>(undefined, "'abcd'.Length", undefined, undefined, true);
    //     expect(4).to.equal(result);
    // });

    // it('Case 13 - pass case - TotalMinutes', () => {
    //     
    //     let result: number = expressionEngine.evalGeneric<number>(undefined, "'abcd'.Length", undefined, undefined, true);
    //     expect(4).to.equal(result);
    // });

    
});