import { getContext, testContext } from "../testContext";
import { expressionEngine } from "../../src/expressionEngine";
import { expect } from "chai";
import { getType } from "../../src/utils/reflectionExtender";
import DateTime from "typescript-dotnet-commonjs/System/Time/DateTime";
import TimeSpan from "typescript-dotnet-commonjs/System/Time/TimeSpan";

describe('  Expression engine - DateTime', () => {
    it('Case 1 - pass case - ToUTC', () => {
        let context: testContext = testContext.getInstance();
        let now = DateTime.now;
        let result: DateTime = expressionEngine.evalGeneric<DateTime>(now, "ToUtc()");
        expect(true).to.equal(result.equals(now.toUniversalTime()));

    });

    it('Case 1 - pass case - AddPeriod - Add days', () => {
        let context: testContext = testContext.getInstance();
        let now = DateTime.now;
        let result: DateTime = expressionEngine.evalGeneric<DateTime>(now, "AddPeriod(1, 'day')", undefined, undefined, true);
        expect(true).to.equal(result.equals(now.addDays(1)));

        result = expressionEngine.evalGeneric<DateTime>(now, "AddPeriod(1, 'days')", undefined, undefined, true);
        expect(true).to.equal(result.equals(now.addDays(1)));
    });

    it('Case 2 - pass case - AddPeriod - Add hours', () => {
        let context: testContext = testContext.getInstance();
        let now = DateTime.now;
        let result: DateTime = expressionEngine.evalGeneric<DateTime>(now, "AddPeriod(1, 'hour')", undefined, undefined, true);
        expect(true).to.equal(result.equals(now.addHours(1)));

        result = expressionEngine.evalGeneric<DateTime>(now, "AddPeriod(1, 'hours')", undefined, undefined, true);
        expect(true).to.equal(result.equals(now.addHours(1)));
    });

    it('Case 3 - pass case - AddPeriod - Add millisecond', () => {
        let context: testContext = testContext.getInstance();
        let now = DateTime.now;
        let result: DateTime = expressionEngine.evalGeneric<DateTime>(now, "AddPeriod(1, 'ms')", undefined, undefined, true);
        expect(true).to.equal(result.equals(now.addMilliseconds(1)));

        result = expressionEngine.evalGeneric<DateTime>(now, "AddPeriod(1, 'millis')", undefined, undefined, true);
        expect(true).to.equal(result.equals(now.addMilliseconds(1)));

        result = expressionEngine.evalGeneric<DateTime>(now, "AddPeriod(1, 'millisecond')", undefined, undefined, true);
        expect(true).to.equal(result.equals(now.addMilliseconds(1)));

        result = expressionEngine.evalGeneric<DateTime>(now, "AddPeriod(1, 'milliseconds')", undefined, undefined, true);
        expect(true).to.equal(result.equals(now.addMilliseconds(1)));
    });

    it('Case 4 - pass case - AddPeriod - Add minutes', () => {
        let context: testContext = testContext.getInstance();
        let now = DateTime.now;
        let result: DateTime = expressionEngine.evalGeneric<DateTime>(now, "AddPeriod(1, 'min')", undefined, undefined, true);
        expect(true).to.equal(result.equals(now.addMinutes(1)));

        result = expressionEngine.evalGeneric<DateTime>(now, "AddPeriod(1, 'mins')", undefined, undefined, true);
        expect(true).to.equal(result.equals(now.addMinutes(1)));

        result = expressionEngine.evalGeneric<DateTime>(now, "AddPeriod(1, 'minute')", undefined, undefined, true);
        expect(true).to.equal(result.equals(now.addMinutes(1)));

        result = expressionEngine.evalGeneric<DateTime>(now, "AddPeriod(1, 'minutes')", undefined, undefined, true);
        expect(true).to.equal(result.equals(now.addMinutes(1)));
    });

    it('Case 5 - pass case - AddPeriod - Add Month', () => {
        let context: testContext = testContext.getInstance();
        let now = DateTime.now;
        let result: DateTime = expressionEngine.evalGeneric<DateTime>(now, "AddPeriod(1, 'month')", undefined, undefined, true);
        expect(true).to.equal(result.equals(now.addMonths(1)));

        result = expressionEngine.evalGeneric<DateTime>(now, "AddPeriod(1, 'months')", undefined, undefined, true);
        expect(true).to.equal(result.equals(now.addMonths(1)));
    });

    it('Case 6 - pass case - AddPeriod - Add Seconds', () => {
        let context: testContext = testContext.getInstance();
        let now = DateTime.now;
        let result: DateTime = expressionEngine.evalGeneric<DateTime>(now, "AddPeriod(1, 'sec')", undefined, undefined, true);
        expect(true).to.equal(result.equals(now.addSeconds(1)));

        result = expressionEngine.evalGeneric<DateTime>(now, "AddPeriod(1, 'secs')", undefined, undefined, true);
        expect(true).to.equal(result.equals(now.addSeconds(1)));

        result = expressionEngine.evalGeneric<DateTime>(now, "AddPeriod(1, 'second')", undefined, undefined, true);
        expect(true).to.equal(result.equals(now.addSeconds(1)));

        result = expressionEngine.evalGeneric<DateTime>(now, "AddPeriod(1, 'seconds')", undefined, undefined, true);
        expect(true).to.equal(result.equals(now.addSeconds(1)));
    });

    it('Case 7 - pass case - AddPeriod - Add Tick', () => {
        let context: testContext = testContext.getInstance();
        let now = DateTime.now;
        let result: DateTime = expressionEngine.evalGeneric<DateTime>(now, "AddPeriod(1, 'tick')", undefined, undefined, true);
        expect(true).to.equal(result.equals(now.add(TimeSpan.fromTicks(1))));

        result = expressionEngine.evalGeneric<DateTime>(now, "AddPeriod(1, 'ticks')", undefined, undefined, true);
        expect(true).to.equal(result.equals(now.add(TimeSpan.fromTicks(1))));
    });

    it('Case 8 - pass case - AddPeriod - Add year', () => {
        let context: testContext = testContext.getInstance();
        let now = DateTime.now;
        let result: DateTime = expressionEngine.evalGeneric<DateTime>(now, "AddPeriod(1, 'year')", undefined, undefined, true);
        expect(true).to.equal(result.equals(now.addYears(1)));

        result = expressionEngine.evalGeneric<DateTime>(now, "AddPeriod(1, 'years')", undefined, undefined, true);
        expect(true).to.equal(result.equals(now.addYears(1)));
    });

    it('Case 8 - pass case - AddPeriod - Add week', () => {
        let context: testContext = testContext.getInstance();
        let now = DateTime.now;
        let result: DateTime = expressionEngine.evalGeneric<DateTime>(now, "AddPeriod(1, 'week')", undefined, undefined, true);
        expect(true).to.equal(result.equals(now.add(TimeSpan.fromDays(1 * 7))));


        result = expressionEngine.evalGeneric<DateTime>(now, "AddPeriod(1, 'weeks')", undefined, undefined, true);
        expect(true).to.equal(result.equals(now.add(TimeSpan.fromDays(1 * 7))));

    });

    it('Case 9 - pass case - ToDate', () => {
        let result: Object = expressionEngine.evalGeneric<DateTime>(undefined, "ToDate('9/7/20')", undefined, undefined, true);
        console.log(result);
        expect(true).to.equal(new Date(result.toString()) instanceof Date);
    });

    it('Case 10 - pass case - Now', () => {
        let result: DateTime = expressionEngine.evalGeneric<DateTime>(undefined, "Now()");
        expect(true).to.equal(DateTime.now.toJsDate().toString().equals(result));
    });

    it('Case 11 - pass case - UTCNow', () => {
        let result: DateTime = expressionEngine.evalGeneric<DateTime>(undefined, "UTCNow()");
        expect(false).to.equal(DateTime.now.toUniversalTime().toJsDate().toString().equals(result));
    });
    it('Case 1 - pass case - IsToday', () => {
        let context: testContext = testContext.getInstance();
        context.dtTime = DateTime.now;

        let result: boolean = expressionEngine.evalGeneric<boolean>(context, "dtTime.IsToday()");
        expect(true).to.equal(result);

    });

    it('Case 1 - pass case IsDate', () => {
        let context: testContext = testContext.getInstance();
        context.dtTime = DateTime.now;

        let result = expressionEngine.evalGeneric<boolean>(context, "IsDate(dtTime)");
        expect(true).to.equal(result);

    });

    it('Case 1 - pass case Format', () => {
        let context: testContext = testContext.getInstance();
        context.dtTime = new DateTime('9/13/2020 01:20:10');

        let result = expressionEngine.evalGeneric<string>(context, "dtTime.Format(\"{0:MM/dd/yyyy}\")", undefined, undefined, true);
        expect('09/13/2020').to.equal(result);

        result = expressionEngine.evalGeneric<string>(context, "dtTime.Format('0:dd/MM/yyyy')", undefined, undefined, true);
        expect('13/09/2020').to.equal(result);

        result = expressionEngine.evalGeneric<string>(context, "dtTime.Format(\"{0:MMM dd}\")", undefined, undefined, true);
        expect('Sep 13').to.equal(result);

        result = expressionEngine.evalGeneric<string>(context, "dtTime.Format(\"{0:dd MMM}\")", undefined, undefined, true);
        expect('13 Sep').to.equal(result);

        result = expressionEngine.evalGeneric<string>(context, "dtTime.Format(\"{0:dd/MM/yy}\")", undefined, undefined, true);
        expect('13/09/20').to.equal(result);

        result = expressionEngine.evalGeneric<string>(context, "dtTime.Format(\"{0:dd/MMM/yyyy hh:mm:ss tt}\")", undefined, undefined, true);
        expect('13/Sep/2020 01:20:10 am').to.equal(result);

        result = expressionEngine.evalGeneric<string>(context, "dtTime.Format(\"{0:dd.MMM.yyyy hh:mm:ss tt}\")");
        expect('13.Sep.2020 01:20:10 am').to.equal(result);

        result = expressionEngine.evalGeneric<string>(context, "dtTime.Format(\"{0:d/M/yy}\")");
        expect('13/9/20').to.equal(result);

        context.dtTime = new DateTime('9/13/2020 13:20:10');

        result = expressionEngine.evalGeneric<string>(context, "dtTime.Format(\"{0:dd.MMM.yyyy hh:mm:ss tt}\")");
        expect('13.Sep.2020 01:20:10 pm').to.equal(result);

        result = expressionEngine.evalGeneric<string>(context, "dtTime.Format(\"{0:ss}\")");
        expect('10').to.equal(result);
    });

    it('Case 1 - pass case - ToLocalTime', () => {
        let context: testContext = getContext();
        context.dtTime = DateTime.now;


        let result: DateTime = expressionEngine.evalGeneric<DateTime>(context, "dtTime.ToLocalTime()");
        expect(context.dtTime.toJsDate().toLocaleString()).to.equals(result);

    });
});