import { expect } from 'chai';
import { expressionEngine } from '../../src/expressionEngine';
import { testContext, getContext } from '../testContext';
import {EntityObject} from '../../src/utils/entityObject';
import List from 'typescript-dotnet-commonjs/System/Collections/List';

describe('  Expression engine - findAll - With Array', () => {
  class contextLogin {
    Login: any;
    constructor(login: any) {
      this.Login = login;
    }
  }
  it('Case 1 - pass case', () => {
    const context = new contextLogin({
      Name: "Khurram",
      Email: "khurram@khan.com",
      Roles: [{ Name: "role 1" }, { Name: "role 2" }],
    });
    const result = expressionEngine.eval(
      context,
      "Login.Roles.FindAll(Name='role 1').Count>0"
    );
  
    expect(true).to.equal(result);
  });
});

describe('  Expression engine - findAll', () => {
  it('Case 1 - pass case', () => {
    let context: testContext = getContext();

    let result: number = expressionEngine.evalGeneric<number>(context, "Entity[\"Employee\"].FindAll([\"ID\"]<1).Count");
    expect(1).to.equal(result);
  });

  it('Case 2 - pass case', () => {
    let context: testContext = getContext();

    let result: string = expressionEngine.evalGeneric<string>(context, "Entity[\"Employee\"].FindAll([\"ID\"]<1)[0][\"Name\"]");
    expect("Employee 0").to.equal(result);
  });

  it('Case 3 - pass case', () => {
    let context: testContext = getContext();

    let result: number = expressionEngine.evalGeneric<number>(context, "Entity[\"Employee\"].FindAll([\"ID\"]<1)[0][\"ID\"]");
    expect(0).to.equal(result);
  });

});

describe('  Expression engine - Find', () => {
  it('Case 1 - pass case ', () => {
    let context: testContext = getContext();

    let result: string = expressionEngine.evalGeneric<string>(context, "Entity[\"Employee\"].Find([\"ID\"]<1)[\"Name\"]");
    expect("Employee 0").to.equal(result);
  });

  it('Case 2 - pass case ', () => {
    let context: testContext = getContext();

    let result: number = expressionEngine.evalGeneric<number>(context, "Entity[\"Employee\"].Find([\"ID\"]<1)[\"ID\"]");
    expect(0).to.equal(result);
  });
});

describe('  Expression engine - Inner Join', () => {

  it('Case 1 - pass case ', () => {
    let context: testContext = getContext();

    let result: List<Object> = expressionEngine.evalGeneric<List<Object>>(context, "Entity[\"Employee\"].InnerJoin(Entity[\"EmployeeProject\"], item1.ID=item2.EmployeeID)");
    expect(20).to.equal(result.count);
  });

  it('Case 2 - pass case ', () => {
    let context: testContext = getContext();

    let result: List<Object> = expressionEngine.evalGeneric<List<Object>>(context, "Entity[\"Employee\"].InnerJoin(Entity[\"EmployeeProject\"], item1.ID=item2.EmployeeID).InnerJoin(Entity[\"Project\"], item1[1].ProjectID=item2.ID).Select([0], [2].Name)");
    expect(20).to.equal(result.count);
  }).timeout(50000);

  it('Case 3 - pass case ', () => {
    let context: testContext = getContext();

    let result: List<Object> = expressionEngine.evalGeneric<List<Object>>(context, "Entity[\"Employee\"].FindAll([\"ID\"]=1).InnerJoin(Entity[\"EmployeeProject\"], item1.ID=item2.EmployeeID).InnerJoin(Entity[\"Project\"], item1[1].ProjectID=item2.ID).Select([0], [2].Name)");
    expect(10).to.equal(result.count);
  }).timeout(50000);

  it('Case 4 - pass case ', () => {
    let context: testContext = getContext();

    let result: number = expressionEngine.evalGeneric<number>(context, "Entity[Employee].FindAll([\"ID\"]=2).InnerJoin(Entity[\"EmployeeProject\"], item1.ID=item2.EmployeeID).InnerJoin(Entity[\"Project\"], item1[1].ProjectID=item2.ID).Select([0], [2].Name).Count");
    expect(10).to.equal(result);
  }).timeout(50000);
});

describe('  Expression engine - Left Join', () => {

  it('Case 1 - pass case ', () => {
    let context: testContext = getContext();

    let result: List<Object> = expressionEngine.evalGeneric<List<Object>>(context, "Entity[\"Employee\"].LeftJoin(Entity[\"EmployeeProject\"], item1.ID=item2.EmployeeID)");
    expect(118).to.equal(result.count);
  }).timeout(50000);
});

describe('  Expression engine - Right Join', () => {

  it('Case 1 - pass case ', () => {
    let context: testContext = getContext();

    let result: List<Object> = expressionEngine.evalGeneric<List<Object>>(context, "Entity[\"Employee\"].RightJoin(Entity[\"EmployeeProject\"], item1.ID=item2.EmployeeID)");
    expect(20).to.equal(result.count);
  }).timeout(50000);
});

describe('  Expression engine - Full Join', () => {

  it('Case 1 - pass case ', () => {
    let context: testContext = getContext();

    let result: List<Object> = expressionEngine.evalGeneric<List<Object>>(context, "Entity[\"Employee\"].FullJoin(Entity[\"EmployeeProject\"], item1.ID=item2.EmployeeID)");
    expect(20).to.equal(result.count);
  }).timeout(50000);
});

describe('  Expression engine - Join', () => {

  it('Case 1 - pass case ', () => {
    let context: testContext = getContext();

    let result: List<Object> = expressionEngine.evalGeneric<List<Object>>(context, "Entity[\"Employee\"].Join(Entity[\"EmployeeProject\"])");
    expect(0).to.equal(result.count);
  }).timeout(50000);
});

describe('  Expression engine - Order By', () => {

  it('Case 1 - pass case - Asc ', () => {
    let context: testContext = getContext();

    let result: List<Object> = expressionEngine.evalGeneric<List<object>>(context, "Entity[\"Employee\"].OrderBy(Name)");
    expect("Employee 14").to.equal((result.get(6) as EntityObject).getItem("Name"));
  });

  it('Case 2 - pass case - Desc ', () => {
    let context: testContext = getContext();

    let result: List<Object> = expressionEngine.evalGeneric<List<object>>(context, "Entity[\"Employee\"].OrderBy(Name desc)");
    expect("Employee 93").to.equal((result.get(6) as EntityObject).getItem("Name"));
  });

  it('Case 3 - pass case - ASC with Then case ', () => {
    let context: testContext = getContext();

    let result: List<Object> = expressionEngine.evalGeneric<List<object>>(context, "Entity[\"Employee\"].OrderBy(Name, Age desc)");
    expect("Employee 96").to.equal((result.get(6) as EntityObject).getItem("Name"));
  });

  it('Case 4 - pass case - With GroupBy ', () => {
    let context: testContext = getContext();

    let result: List<Object> = expressionEngine.evalGeneric<List<object>>(context, "Entity[\"Employee\"].GroupBy(Age).OrderBy(Name)");
    expect(10).to.equal(result.count);
  });

  it('Case 4 - pass case - With FindAll ', () => {
    let context: testContext = getContext();

    let result: List<Object> = expressionEngine.evalGeneric<List<object>>(context, "Entity[\"Employee\"].FindAll(Age=20).OrderBy(Name)");
    expect("Employee 6").to.equal((result.get(6) as EntityObject).getItem("Name"));
    expect(10).to.equal(result.count);

  });
});

describe('  Expression engine - Switch', () => {

  it('Case 1 - pass case - Without Switch function', () => {
    let context: testContext = getContext();

    let result: number = expressionEngine.evalGeneric<number>(undefined, "\"True\"{\"True\"=100,\"False\"=0}");
    expect(100).to.equal(result);
  });

  it('Case 2 - pass case - With Switch function', () => {
    let context: testContext = getContext();

    let result: number = expressionEngine.evalGeneric<number>(context, "firstNumber.Switch(0=100,1=0,400)");
    expect(100).to.equal(result);
    
    //Else case
    context.firstNumber=1;
    result = expressionEngine.evalGeneric<number>(context, "firstNumber.Switch(0=100,1=0,400)");
    expect(0).to.equal(result);

    //Default value
    context.firstNumber=3;
    result = expressionEngine.evalGeneric<number>(context, "firstNumber.Switch(0=100,1=0,400)");
    expect(400).to.equal(result);

  });

  it('Case 4 - pass case - Without Switch function - context', () => {
    let context: testContext = getContext();
    context.firstNumber=1;
    let result: number = expressionEngine.evalGeneric<number>(context, "firstNumber{0=100,1=0,400}");
    expect(0).to.equal(result);
  });
}); 

describe('  Expression engine - Switch recursive', () => {

  it('Case 1 - pass case - Without Switch function', () => {
    let context: testContext = getContext();
    context.firstBool = true;
    context.firstNumber = 1;
    let result: number = expressionEngine.evalGeneric<number>(context, "firstBool{true=firstNumber{1=10, 2=20, 30}, 1000}");
    expect(10).to.equal(result);
  });

  it('Case 2 - pass case - Without Switch function', () => {
    let context: testContext = getContext();
    context.firstBool = true;
    context.firstNumber = 1;
    let result: number = expressionEngine.evalGeneric<number>(context, "firstBool{true=firstNumber{1=secondNumber{2=200, 300}, 2=20, 30}, 1000}");
    expect(200).to.equal(result);
  });
  it('Case 3 - pass case - Without Switch function', () => {
    let context: testContext = getContext();
    context.firstBool = true;
    context.firstNumber = 1;
    let result: number = expressionEngine.evalGeneric<number>(context, "firstBool{true=firstNumber{1=secondNumber{2=200, 300}, 2=20, 30}, 1000}");
    expect(200).to.equal(result);
  });
  it('Case 4 - pass case - Without Switch function', () => {
    let context: testContext = getContext();
    context.firstBool = true;
    context.firstNumber = 1;
    let result: number = expressionEngine.evalGeneric<number>(context, "firstBool{true=firstNumber{1=secondNumber{2=thirdNumber{3=300}, 3000}, 2=20, 2000}, 1000}");
    expect(300).to.equal(result);
  });
});