import { getContext, testContext } from "../testContext";
import { expressionEngine } from "../../src/expressionEngine";
import { expect } from "chai";
import { getType } from "../../src/utils/reflectionExtender";
import List from "typescript-dotnet-commonjs/System/Collections/List";
import DateTime from "typescript-dotnet-commonjs/System/Time/DateTime";

describe('  Expression engine - List Or Array', () => {
  it('Case 1 - pass case - Exists', () => {
    let context: testContext = getContext();

    let result = expressionEngine.evalGeneric<boolean>(context, "SubList.Exists(Name='name 1')", undefined, undefined, true);
    expect(true).to.equal(result);

    result = expressionEngine.evalGeneric<boolean>(context, "SubList.Exists(Name='name')", undefined, undefined, true);
    expect(false).to.equal(result);
  });

  it('Case 2 - pass case - ToList', () => {
    let context: testContext = getContext();

    let result = expressionEngine.evalGeneric<List<Object>>(context, "ToList('One Way', 'Return Trip')", undefined, undefined, true);
    expect(2).to.equal(result.count);

    result = expressionEngine.evalGeneric<List<Object>>(context, "ToList(ArrayString, ArrayString)", undefined, undefined, false);
    expect(4).to.equal((result.get(0) as string[]).length);
    expect(2).to.equal(result.count);
  });

  it('Case 3 - pass case - SelectNamed and ToObjectList', () => {
    let context: testContext = getContext();

    let result: List<Object> = expressionEngine.evalGeneric<List<object>>(context, "SubList.SelectNamed(Address, Age, Name).ToObjectList()");
    expect(6).to.equal(result.count);
  });

  it('Case 4 - pass case - Select and ToObjectList', () => {
    let context: testContext = getContext();

    let result: List<Object> = expressionEngine.evalGeneric<List<object>>(context, "SubList.Select(Address @ \"Address\", Age @ \"Age\", Name @ \"Name\").ToObjectList()");
    expect(6).to.equal(result.count);
  });

  it('Case 5 - pass case - Limit', () => {
    let context: testContext = getContext();

    let result: List<Object> = expressionEngine.evalGeneric<List<object>>(context, "SubList.Limit(2)");
    expect(2).to.equal(result.count);
  });

  it('Case 6 - pass case - Limit - 1', () => {
    let context: testContext = getContext();

    let result: List<Object> = expressionEngine.evalGeneric<List<object>>(context, "SubList.Limit(1, 3)");
    expect(3).to.equal(result.count);
  });

  it('Case 7 - pass case - GroupByNamed ', () => {
    let context: testContext = getContext();

    let result: List<Object> = expressionEngine.evalGeneric<List<Object>>(context, "SubList.GroupByNamed(Address, Sum(Age) / Count(Age), Sum(Age)).ToObjectList()");
    expect(2).to.equal(result.count);
  });

  it('Case 6 - pass case - SelectName and ToObject', () => {
    let context: testContext = getContext();

    let result: Object = expressionEngine.evalGeneric<Object>(context, "SubList.SelectNamed(Address, Age, Name)[0].ToObject()");
    expect("Address 1").to.equal(Reflect.get(result,"0"));
  });

  it('Case 6 - pass case - Select and ToObject', () => {
    let context: testContext = getContext();

    let result: Object = expressionEngine.evalGeneric<Object>(context, "SubList.Select(Address, Age, Name)[0].ToObject()");
    expect("Address 1").to.equal(Reflect.get(result,"0"));
  });
}); 