import { expect } from 'chai';
// import { testContext } from './context';
import { expressionEngine } from '../../src/expressionEngine';
import { testContext, getContext } from '../testContext';


describe('  Expression engine - concat', () => {
  it('Case 1 - pass case - concat', () => {
    let context: testContext = testContext.getInstance();
    context.FirstString = "ab cd";
    context.firstNumber = 0;
    context.secondNumber = 2;
    context.thirdNumber = 3;
    context.forthNumber = 4;

    let result: string = expressionEngine.evalGeneric<string>(context, "Concat(\", \", firstNumber, FirstString, null, null)");
    expect('0,ab cd,,').to.equal(result);
  });
});

describe('  Expression engine - ConcatNonEmpty', () => {
  it('Case 1 - pass case - ConcatNonEmpty', () => {
    let context: testContext = testContext.getInstance();
    context.FirstString = "ab cd";
    context.firstNumber = 1;

    let result: string = expressionEngine.evalGeneric<string>(context, "ConcatNonEmpty(\", \", firstNumber, FirstString, null, null, \"\")");
    expect("1ab cd").to.equal(result);
  });
});

describe('  Expression engine - Matches cases', () => {

  it('Case 1 - pass case - Small letter', () => {
    let context: testContext = testContext.getInstance();
    context.FirstString = "abcd";
    let result: boolean = expressionEngine.evalGeneric<boolean>(context, "FirstString.Matches(\"[a-z]+\", \"im\")");
    expect(true).to.equal(result);
  });

  it('Case 2 - fail case - capital letters', () => {
    let context: testContext = testContext.getInstance();
    context.FirstString = "abcd";
    let result: boolean = expressionEngine.evalGeneric<boolean>(context, "FirstString.Matches(\"[A-Z]+\", \"m\")");
    expect(false).to.equal(result);
  });

  it('Case 2 - fail case - number match string', () => {
    let context: testContext = testContext.getInstance();
    context.FirstString = "abcd";
    let result: boolean = expressionEngine.evalGeneric<boolean>(context, "FirstString.Matches(\"[A-Z]+\", \"564\")");
    expect(false).to.equal(result);
  });
});

describe('  Expression engine - Substring cases', () => {

  it('Case 1 - pass case - get substring from start index', () => {
    let context: testContext = testContext.getInstance();
    context.FirstString = "abcd";
    let result: string = expressionEngine.evalGeneric<string>(context, "FirstString.Substring(2)");
    expect("cd").to.equal(result);
  });

  it('Case 1 - pass case - get substring from start to end index', () => {
    let context: testContext = testContext.getInstance();
    context.FirstString = "abcd";
    let result: string = expressionEngine.evalGeneric<string>(context, "FirstString.Substring(1,3)");
    expect("bc").to.equal(result);
  });
});

describe('  Expression engine - Replace', () => {
  it('Case 1 - pass case - replace', () => {
    let context: testContext = testContext.getInstance();
    context.FirstString = "ab cd";

    let result: string = expressionEngine.evalGeneric<string>(context, "FirstString.Replace(\" \",\"\")");
    expect("abcd").to.equal(result);
  });
});

describe('  Expression engine - regexreplace', () => {
  it('Case 1 - pass case', () => {
    let context: testContext = testContext.getInstance();
    context.FirstString = "abcd";

    let result: string = expressionEngine.evalGeneric<string>(context, "FirstString.RegexReplace(\"([a-z])(.*)\", \"ABC$2\", \"im\")");
    expect("ABCbcd").to.equal(result);

  });

  it('Case 2 - pass case', () => {
    let context: testContext = testContext.getInstance();
    context.FirstString = "abcd";

    let result: string = expressionEngine.evalGeneric<string>(context, "FirstString.RegexReplace(\"([A-Z]+)\", \"ABC$1\", \"im\")");
    expect("ABCabcd").to.equal(result);

  });

  it('Case 3 - pass case', () => {
    let context: testContext = testContext.getInstance();
    context.FirstString = "abcd";

    let result: string = expressionEngine.evalGeneric<string>(context, "FirstString.RegexReplace(\"([A-Z]+)\", \"ABC$1\", \"m\")");
    expect("abcd").to.equal(result);

  });

});

describe('  Expression engine - tolower', () => {
  it('Case 1 - pass case - tolower', () => {
    let context: testContext = testContext.getInstance();
    context.FirstString = "abCDE";

    let result: string = expressionEngine.evalGeneric<string>(context, "FirstString.ToLower()");
    expect("abcde").to.equal(result);
  });
});

describe('  Expression engine - ToUpper', () => {
  it('Case 1 - pass case - ToUpper', () => {
    let context: testContext = testContext.getInstance();
    context.FirstString = "abCDE";

    let result: string = expressionEngine.evalGeneric<string>(context, "FirstString.ToUpper()");
    expect("ABCDE").to.equal(result);
  });
});

describe('  Expression engine - ToString', () => {
  it('Case 1 - pass case - number to String', () => {
    let context: testContext = testContext.getInstance();
    context.FirstString = "abCDE";
    context.firstNumber = 20;

    let result: string = expressionEngine.evalGeneric<string>(context, "firstNumber.ToString()");
    expect("20").to.equal(result);
  });

  it('Case 1 - pass case - bool to String', () => {
    let context: testContext = testContext.getInstance();
    context.FirstString = "abCDE";
    context.firstNumber = 20;
    context.boolTest = true;

    let result: string = expressionEngine.evalGeneric<string>(context, "boolTest.ToString()");
    expect("true").to.equal(result);
  });
});



describe('  Expression engine - Split', () => {
  it('Case 1 - pass case - split', () => {
    let context: testContext = testContext.getInstance();
    context.FirstString = "ab cd";

    let result: string[] = expressionEngine.evalGeneric<string[]>(context, "FirstString.Split(\" \")");
    expect(2).to.equal(result.length);
  });
});

describe('  Expression engine - coalesce', () => {
  it('Case 1 - pass case - coalesce', () => {
    let context: testContext = testContext.getInstance();
    context.FirstString = "ab cd";
    context.firstNumber = 1,
      context.secondNumber = 2,
      context.thirdNumber = 3,
      context.forthNumber = 4,
      context.FirstString = ""

    let result: string = expressionEngine.evalGeneric<string>(context, "Coalesce(\"1234\", \"7678\")");
    expect("1234").to.equal(result);
  });
  it('Case 1 - pass case - coalesce with null', () => {
    let context: testContext = testContext.getInstance();
    context.FirstString = "ab cd";
    context.firstNumber = 1,
      context.secondNumber = 2,
      context.thirdNumber = 3,
      context.forthNumber = 4,
      context.FirstString = ""

    let result: string = expressionEngine.evalGeneric<string>(context, "Coalesce(null, \"7678\", true)");
    expect("7678").to.equal(result);
  });
});

describe('  Expression engine - number test', () => {

  it('Case 1 - pass case - subtract simple decimal', () => {
    let result: number = expressionEngine.evalGeneric<number>(undefined, "1.5 - 0.5",undefined);
    expect(1.0).to.equal(result);
  });

  it('Case 2 - pass case - multiply simple decimal', () => {
    let result: number = expressionEngine.evalGeneric<number>(undefined, "2.5 * 2.0");
    expect(5.0).to.equal(result);
  });

  it('Case 3 - pass case - divide simple decimal', () => {
    let result: number = expressionEngine.evalGeneric<number>(undefined, "1.0/4.0");
    expect(0.25).to.equal(result);
  });

  it('Case 4 - pass case - simple integer multiple', () => {
    let result: number = expressionEngine.evalGeneric<number>(undefined, "(1) * (4)");
    expect(4).to.equal(result);
  });

  it('Case 5 - pass case - handle precedence', () => {
    let result: number = expressionEngine.evalGeneric<number>(undefined, "4 ^ 2 - 5 + 1");
    expect(4 ^ 2 - 5 + 1).to.equal(result);
  });

  it('Case 6 - pass case - complex equation 1', () => {
    let result: number = expressionEngine.evalGeneric<number>(undefined, "(3 + 2) + (1 + 7)");
    expect(13).to.equal(result);
  });

  it('Case 7 - pass case - complex equation 2', () => {
    let result: number = expressionEngine.evalGeneric<number>(undefined, "(3*1 - 2*7) + (3*7 + 2*1)");
    expect(12).to.equal(result);
  });


});

describe('  Expression engine - xor and bool test', () => {
  it('Case 1 - pass case ', () => {
    let result: number = expressionEngine.evalGeneric<number>(undefined, "3 ^ 1");
    expect(3 ^ 1).to.equal(result);
  });

  it('Case 2 - pass case complex equation 3', () => {
    let result: number = expressionEngine.evalGeneric<number>(undefined, "(3 + 2) + (1 + 7) = (4 + 9)");
    expect(true).to.equal(result);
  });

  it('Case 3 - pass case complex equation 4', () => {
    let result: number = expressionEngine.evalGeneric<number>(undefined, "((3 + 1) * (7 - 4))/2 = ((3*1 - 2*7) + (3*7 + 2*1))/2");
    expect(true).to.equal(result);
  });
});

describe('  Expression engine - complex equation with data context ', () => {
  it('Case 1 - pass case - equation 1', () => {
    let context: testContext = testContext.getInstance();
    context.firstNumber = 1;
    context.secondNumber = 2;
    context.thirdNumber = 3;
    context.forthNumber = 4;

    let result: number = expressionEngine.evalGeneric<number>(context, "(firstNumber + secondNumber) * (thirdNumber - forthNumber)");
    expect(-3).to.equal(result);
  });

  it('Case 2 - pass case - equation 2', () => {
    let context: testContext = testContext.getInstance();
    context.firstNumber = 1;
    context.secondNumber = 2;
    context.thirdNumber = 3;
    context.forthNumber = 4;

    let result: number = expressionEngine.evalGeneric<number>(context, "firstNumber + secondNumber * thirdNumber - forthNumber");
    expect(3).to.equal(result);
  });

  it('Case 3 - pass case - equation 3', () => {
    let context: testContext = testContext.getInstance();
    context.firstNumber = 1;
    context.secondNumber = 2;
    context.thirdNumber = 3;
    context.forthNumber = 4;

    let result: number = expressionEngine.evalGeneric<number>(context, "firstNumber + secondNumber * (thirdNumber - forthNumber)");
    expect(-1).to.equal(result);
  });
});

describe('  Expression engine - Math function test', () => {
  it('Case 1 - pass case - Math Round ', () => {
    let result: number = expressionEngine.evalGeneric<number>(undefined, "Math_Round(5.1235, 0)");
    expect(5.0).to.equal(result);
  });

  it('Case 2 - pass case - Math Round ', () => {
    let result: number = expressionEngine.evalGeneric<number>(undefined, "Math_Round(5.1235)");
    expect(5.0).to.equal(result);
  });

  it('Case 3 - pass case - Math Round ', () => {
    let result: number = expressionEngine.evalGeneric<number>(undefined, "Math_Round(5.1235, 2)");
    expect(5.12).to.equal(result);
  });

  it('Case 4 - pass case - Math pow ', () => {
    let result: number = expressionEngine.evalGeneric<number>(undefined, "Math_Pow(12, 2)");
    expect(144.0).to.equal(result);
  });

  it('Case 5 - pass case - Math Sqrt ', () => {
    let result: number = expressionEngine.evalGeneric<number>(undefined, "Math_Sqrt(4)");
    expect(2.0).to.equal(result);
  });

  it('Case 6 - pass case - Math PI ', () => {
    let result: number = expressionEngine.evalGeneric<number>(undefined, "Math_Pi(2)");
    expect(3.14).to.equal(result);
  });

  it('Case 7 - pass case - Math ABS ', () => {
    let result: number = expressionEngine.evalGeneric<number>(undefined, "Math_Abs(-2.5)");
    expect(2.5).to.equal(result);
  });

  it('Case 7 - pass case - Math Sum ', () => {
    let result: number = expressionEngine.evalGeneric<number>(undefined, "Math_Sum(-2.5, 3)");
    expect(0.5).to.equal(result);
  });

  it('Case 8 - pass case - Math_Tan  ', () => {
    let result: number = expressionEngine.evalGeneric<number>(undefined, "Math_Tan(8)");
    expect(-6.799711455220379).to.equal(result);
  });

  it('Case 9 - pass case - Math_Cosh  ', () => {
    let result: number = expressionEngine.evalGeneric<number>(undefined, "Math_Cosh(0)");
    expect(1).to.equal(result);
  });

  it('Case 10 - pass case - Math_Tanh  ', () => {
    let result: number = expressionEngine.evalGeneric<number>(undefined, "Math_Tanh(0)");
    expect(0).to.equal(result);
  });

  it('Case 11 - pass case - Math_Exp  ', () => {
    let result: number = expressionEngine.evalGeneric<number>(undefined, "Math_Exp(0)");
    expect(1).to.equal(result);
  });

  it('Case 12 - pass case - Math_Sin  ', () => {
    let result: number = expressionEngine.evalGeneric<number>(undefined, "Math_Sin(2)");
    expect(0.9092974268256817).to.equal(result);
  });

  it('Case 13 - pass case - Math_Acos  ', () => {
    let result: number = expressionEngine.evalGeneric<number>(undefined, "Math_Acos(8/10)");
    expect(0.6435011087932843).to.equal(result);
  });

  it('Case 14 - pass case - Math_Asin   ', () => {
    let result: number = expressionEngine.evalGeneric<number>(undefined, "Math_Asin(6/10)");
    expect(0.6435011087932844).to.equal(result);
  });

  it('Case 15 - pass case - Math_Atan    ', () => {
    let result: number = expressionEngine.evalGeneric<number>(undefined, "Math_Atan(8/10)");
    expect(0.6747409422235527).to.equal(result);
  });

  it('Case 16 - pass case - Math_Atan2    ', () => {
    let result: number = expressionEngine.evalGeneric<number>(undefined, "Math_Atan2(5,5)");
    expect(0.7853981633974483).to.equal(result);
  });

  it('Case 17 - pass case - Math_Cos ', () => {
    let result: number = expressionEngine.evalGeneric<number>(undefined, "Math_Cos(1)");
    expect(0.5403023058681398).to.equal(result);
  });

  it('Case 18 - pass case - Math_Log ', () => {
    let result: number = expressionEngine.evalGeneric<number>(undefined, "Math_Log(5)");
    expect(1.6094379124341003).to.equal(result);
  });

  it('Case 19 - pass case - Math_Max ', () => {
    let result: number = expressionEngine.evalGeneric<number>(undefined, "Math_Max(5,3,2)");
    expect(5).to.equal(result);
  });

  it('Case 20 - pass case - Math_Min', () => {
    let result: number = expressionEngine.evalGeneric<number>(undefined, "Math_Min(5,3,2)");
    expect(2).to.equal(result);
  });

  it('Case 1 - pass case - 1', () => {
    let context: testContext = getContext();

    context.firstNumber = 200;
    context.secondNumber = 10;

    let result: number = expressionEngine.evalGeneric<number>(context, "firstNumber.Percent(secondNumber)", undefined);
    expect(20).to.equals(result);

  });

});

describe('  Expression engine - Contains', () => {
  let context: testContext = testContext.getInstance();

  it('Case 1 - pass case', () => {
    let context: testContext = testContext.getInstance();
    context.FirstString = "google.com";

    context.List.add("Value 1");
    context.List.add("Value 2");
    context.List.add("Value 3");
    context.List.add("Value 4");
    let result: boolean = expressionEngine.evalGeneric<boolean>(context, "Contains(FirstString, \"google\")");
    expect(true).to.equal(result);
  });

  it('Case 2 - pass case', () => {
    let result: boolean = expressionEngine.evalGeneric<boolean>(context, "FirstString.Contains(\"google\")");
    expect(true).to.equal(result);
  });

  it('Case 3 - pass case', () => {
    let result: boolean = expressionEngine.evalGeneric<boolean>(context, "Contains(List, \"Value 1\")");
    expect(true).to.equal(result);
  });

  it('Case 4 - pass case', () => {
    let result: boolean = expressionEngine.evalGeneric<boolean>(context, "List.Contains(\"Value 1\")");
    expect(true).to.equal(result);
  });
});